/***************************************************************************
                          airmovement.h  -  description
                             -------------------
    begin                : Wed Oct 17 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef AIRMOVEMENT_H
#define AIRMOVEMENT_H


/**Basic movement methods for air units
  *@author T Kontkanen
  */

#include "renderer.h"
#include "ground.h"
#include "game.h"
#include "allincludes.h"
#include "gamedefs.h"
#include "vector.h"

class AirMovement {
public:
//	AirMovement( Renderer *prenderer, Ground *pground );
	AirMovement( Game *pgame );
	~AirMovement();

	Vector getPosition() {
		return (Vector(0,0,altitude)+position);
	}
	float getSpeed() { return velocity; }
	float getAngle() { return angle; }
	float getPitch() { return pitch; }
	float getTilt() { return tilt; }

	//action
	void moveSetDestination( Vector destpos ) { destPosition=destpos; }
	Vector moveGetDestination() { return destPosition; }
	void moveStep(); // make a move (one call per frame)
	
	void setPosition( Vector &pos ) {
		position=pos;
		position.setZ(mGround->getElevation( pos.x(), pos.y() ) );//set elevation
		moveSetDestination( pos );
	}
	void setTurnRate( float rate ) { turnrate=rate; }
	void setTopSpeed( float speed ) { topspeed=speed; }
	void setAcceleration( float a ) { acceleration=a; }
	/// Set altitude where unit should stay when stopped
	void setRestAltitude( float alt ) { restalt=alt; }
	/// Set altitude where unit should rise when moving
	void setTravelAltitude( float alt )  { travelalt=alt; }
protected:
	float velocity; /* m/s */

private:
	// own data
	bool isSomethingOnMyWay( Vector direction );
	
	Vector position, destPosition;
	float angle, pitch, tilt; //angle=z-axis rotation, pitch=x-axis and tilt=y-axis
	float turnrate /* rad/s */, acceleration /* m/s */, topspeed /* m/s */;
	float positionTolerance; // destination position tolerance in meters (radius)
	float turnrange_near, turnrange_far, radarresolution; // distance when unit starts turnign if wall or other unit is detected ahead
      float restalt, travelalt, altitude;
		
	//needed information, don't use same names with Unit to avoid conficts (may be unnecessary?)
	Renderer *mRenderer;
	Ground *mGround;
	Game *mGame;
};

#endif
