/***************************************************************************
                          bulletweapon.cpp  -  description
                             -------------------
    begin                : Fri Sep 14 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "bulletweapon.h"
#include "unit.h"
#include "player.h"

BulletWeapon::BulletWeapon( Game *pgame, Unit *thisunit )
{
	wgame=pgame;
	me=thisunit;
	
//	targetplayer=-1;
//	targetunit=-1;
	
	//set defs
	maxrange=100; //meters
	minrange=0;
	damage=10; //hitpoints
	firerate=10; //10/sec
	accuracy=0;
	
	shootcounter=float(wgame->rand()%100)/100.0;
	shooting_now=false;
}

BulletWeapon::~BulletWeapon()
{
}

void BulletWeapon::weaponThink()
{
	bool targetexist, shootnow;
	Unit *utarget;
	Vector distance;

	shootcounter+=PER_FRAME(firerate);
	if( shootcounter >= 1.0 ) // time to shoot
	{
		shootcounter=0;
		shootnow=true;
	} else
	{
		shootnow=false;
		return;
	}
	
	// is target available?
	utarget=wgame->getUnit(target);
	if( utarget==NULL) targetexist=false;
	else targetexist=true;	
	
	if( !targetexist ) return; //nothing to do
	
	distance=utarget->getUnitPosition()-me->getUnitPosition();
	
	if( distance.length() < minrange ) return; // target too close, can't do anything
	if( distance.length() > maxrange ) return; // target too far, can't do anything
	
	if( shootnow && targetexist ) //shoot
	{
		//calculate accuracy
		Vector hitdiff;
		float rndx=float(wgame->fastRand()%10000)*0.0001-0.5; // -.5 - .5
		float rndy=float(wgame->fastRand()%10000)*0.0001-0.5;
		float rndz=float(wgame->fastRand()%10000)*0.0001-0.5;
		hitdiff.set( distance.length()*accuracy*rndx, distance.length()*accuracy*rndy, distance.length()*accuracy*rndz );
		
		shooting_at=utarget->getUnitPosition()+hitdiff;
		shooting_now=true;
		me->setShootingAt( target );
		if( hitdiff.length() < utarget->getMaxSize() ) //if we hit
			utarget->decreaseHitpoints( damage*(1-(hitdiff.length()/utarget->getSize())), shooting_at );
	}
}


