/***************************************************************************
                          bulletweapon.h  -  description
                             -------------------
    begin                : Fri Sep 14 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef BULLETWEAPON_H
#define BULLETWEAPON_H


#include "renderer.h"
#include "ground.h"
#include "game.h"
#include "allincludes.h"
#include "gamedefs.h"
#include "vector.h"
#include "unitid.h"

/**Basic bullet shooting weapon for units
  *@author T Kontkanen
  */

class Unit;

class BulletWeapon {
public: 
	BulletWeapon( Game *pgame, Unit *thisunit );
	~BulletWeapon();

	/// do progress for one frame
	void weaponThink();
	
	/// Set damage for bullet
	void setDamage( float hitpoints ) { damage=hitpoints; }
	/// Set firing rate
	void setFireRate( float bulletspersec ) { if( firerate >0.001 ) firerate=bulletspersec; else firerate=0.001; }
	/// Set maximum shooting range in meters
	void setSightRange( float range ) { maxrange=range; }
	/// Set minimum shooting range in meters, default 0
	void setMinRange( float range ) { minrange=range; }
	/// Sets weapon accuracy. Param acc is radius in meters where shots will be shot if distance to target is one meter.
	void setAccuracy( float acc ) { accuracy=acc; }
	/// Set target unit. Example: to set unit number 50 of player 1 as target, call setTargetUnit( 1, 50 );
//	void setTargetUnit( int playernum, int unitnum ) { targetplayer=playernum; targetunit=unitnum; }
	/// Set target unit.
	void setTargetUnit( UnitID enemy ) { target=enemy; }
	/// Returns true if gun just has fired. ShootingNow state resets after one call.
	bool shootingNow() {
		bool t=shooting_now;
		shooting_now=false;
		return t;
	 }
	/// Returns target position.
	Vector shootingAtPos() { return shooting_at; }
private:
	Game *wgame; // parent game
	Renderer *wrenderer; //parent renderer
	
	// params
	float damage, firerate, maxrange, minrange;
	bool shooting_now;
	Vector shooting_at;
	float accuracy;
	
	// vars
	float shootcounter;
	UnitID target;
	Unit *me;
};

#endif
