/***************************************************************************
                          color.cpp  -  description
                             -------------------
    begin                : Tue Sep 11 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "color.h"

Color::Color( float r=1, float g=1, float b=1, float a=1 )
{
	autoclamp=false;
	rgba[0]=r;
	rgba[1]=g;
	rgba[2]=b;
	rgba[3]=a;
	
	if( autoclamp ) clamp();
}

Color::Color( float *color )
{
	int i;
	for( i=0; i<4; i++ )
		rgba[i]=color[i];
		
	if( autoclamp ) clamp();
}

/// Destructor
Color::~Color() { }
	
void Color::set( float r, float g, float b, float a )
{
	if( r!=-99.0 ) rgba[0]=r;
	if( g!=-99.0 ) rgba[1]=g;
	if( b!=-99.0 ) rgba[2]=b;
	if( a!=-99.0 ) rgba[3]=a;
	if( autoclamp ) clamp();
}

void Color::set( float *color )
{
	set( color[0], color[1], color[2], color[3] );
}

void Color::clamp()
{
	if( rgba[0] < 0.0 ) rgba[0]=0;
	if( rgba[0] > 1.0 ) rgba[0]=1;
	
	if( rgba[1] < 0.0 ) rgba[1]=0;
	if( rgba[1] > 1.0 ) rgba[1]=1;
	
	if( rgba[2] < 0.0 ) rgba[2]=0;
	if( rgba[2] > 1.0 ) rgba[2]=1;
	
	if( rgba[3] < 0.0 ) rgba[3]=0;
	if( rgba[3] > 1.0 ) rgba[3]=1;
}

void Color::clamp_public()
{
	if( clamp_rgba[0] < 0.0 ) clamp_rgba[0]=0;
	if( clamp_rgba[0] > 1.0 ) clamp_rgba[0]=1;
	
	if( clamp_rgba[1] < 0.0 ) clamp_rgba[1]=0;
	if( clamp_rgba[1] > 1.0 ) clamp_rgba[1]=1;
	
	if( clamp_rgba[2] < 0.0 ) clamp_rgba[2]=0;
	if( clamp_rgba[2] > 1.0 ) clamp_rgba[2]=1;
	
	if( clamp_rgba[3] < 0.0 ) clamp_rgba[3]=0;
	if( clamp_rgba[3] > 1.0 ) clamp_rgba[3]=1;
}

/// Multiply color values with other Color
Color Color::operator*(const Color c)
{
	rgba[0]*=c.rgba[0];
	rgba[1]*=c.rgba[1];
	rgba[2]*=c.rgba[2];
	rgba[3]*=c.rgba[3];
	if( autoclamp ) clamp();
	
	return *this;
}

Color Color::operator*(const float f)
{
	rgba[0]*=f;
	rgba[1]*=f;
	rgba[2]*=f;
	rgba[3]*=f;
	if( autoclamp ) clamp();
	
	return *this;
}

/// Add color values with other Color
Color Color::operator+(const Color c)
{
	rgba[0]+=c.rgba[0];
	rgba[1]+=c.rgba[1];
	rgba[2]+=c.rgba[2];
	rgba[3]+=c.rgba[3];
	if( autoclamp ) clamp();
	
	return *this;
}

void Color::scaleValues( float scalar )
{
	rgba[0]*=scalar;
	rgba[1]*=scalar;
	rgba[2]*=scalar;
	rgba[3]*=scalar;
	if( autoclamp ) clamp();
}


/// Subtract color values with other Color
Color Color::operator-(const Color c)
{
	rgba[0]-=c.rgba[0];
	rgba[1]-=c.rgba[1];
	rgba[2]-=c.rgba[2];
	rgba[3]-=c.rgba[3];
	if( autoclamp ) clamp();
	
	return *this;
}

Color Color::operator-=(const Color c)
{
	rgba[0]-=c.rgba[0];
	rgba[1]-=c.rgba[1];
	rgba[2]-=c.rgba[2];
	rgba[3]-=c.rgba[3];
	if( autoclamp ) clamp();

	return *this;
}

Color Color::operator+=(const Color c)
{
	rgba[0]+=c.rgba[0];
	rgba[1]+=c.rgba[1];
	rgba[2]+=c.rgba[2];
	rgba[3]+=c.rgba[3];
	if( autoclamp ) clamp();

	return *this;
}

Color Color::operator=(const Color c)
{
	int i;
	for( i=0; i<4; i++ ) rgba[i]=c.rgba[i];
	if( autoclamp ) clamp();
	return *this;
}
