/***************************************************************************
                          color.h  -  description
                             -------------------
    begin                : Tue Sep 11 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef COLOR_H
#define COLOR_H

#include <string.h>

/**Simple colour class
  *@author T Kontkanen
  */

class Color {
public:
	/// Create new colour with initial RGBA values. All values are clamped between 0.0 and 1.0.
	Color( float r=1, float g=1, float b=1, float a=1 );
	/// Create new colour with initial RGBA values. All values are clamped between 0.0 and 1.0. Vales are copied from array of 4 floats.
	Color( float *color );
	/// Destructor
	~Color();
	
	/// Set new RGBA-values, value -99 doesn't modify existing value
	void set( float r, float g=-99, float b=-99, float a=-99 );
	/// Set new RGBA-values, value -99 doesn't modify existing value. Values are read from array of four floats.
	void set( float *color );
	/// Return RGBA-values in array of four floats
	const float* RGBA() {
		memcpy( clamp_rgba, rgba,  4*sizeof(float) );
		clamp_public();
		return clamp_rgba;
	}
	
	/// Return red value
	float r() {
		memcpy( clamp_rgba, rgba,  4*sizeof(float) );
		clamp_public();
		return clamp_rgba[0];
	}
	/// Return green value
	float g() {
		memcpy( clamp_rgba, rgba,  4*sizeof(float) );
		clamp_public();
		return clamp_rgba[1];
	}
	/// Return blue value
	float b() {
		memcpy( clamp_rgba, rgba,  4*sizeof(float) );
		clamp_public();
		return clamp_rgba[2];
	}
	/// Return alpha value
	float a() {
		memcpy( clamp_rgba, rgba,  4*sizeof(float) );
		clamp_public();
		return clamp_rgba[3];
	}
	
	/// Multiply RGBA values with scalar.
	void scaleValues( float scalar );
	/// Multiply color values with other Color
	Color operator*(const Color c);
	/// Multiply color values with float
	Color operator*(const float f);
	/// Add color values with other Color
	Color operator+(const Color c);
	/// Subtract color values with other Color
	Color operator-(const Color c);
	Color operator-=(const Color c);
	Color operator+=(const Color c);
	/// Equal operator
	Color operator=(const Color c);
	
	/// Clamp RGBA values between 0.0 and 1.0
	void clamp();
private:
	// Clamps clamp_rgba-array
	void clamp_public();
	
	float rgba[4];
	float clamp_rgba[4];
	bool autoclamp;
};

#endif
