/***************************************************************************
                          config.cpp  -  description
                             -------------------
    begin                : Mon Nov 5 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "configfile.h"

Config::Config(){
	group="";
	bufsize=0;
	verbose=true;
	buffer=NULL;
}

Config::~Config(){
/*	printf("buf %x\n", buffer);
	puts( buffer);*/
	if( buffer!=NULL ) free( buffer );
/*	puts( buffer);*/
	buffer=NULL;
}

/// Read config file. Returns true if success.
bool Config::readFile( string cfgfile )
{
	FILE *f;
	int len, i;
	
	//open
	f=fopen( cfgfile.c_str(), "rb" );
	if( f==NULL ) {
		msgOut( MSG_ERROR, "Unable to open config file %s", cfgfile.c_str() );
		return false;
	}
	
	// get file size
	fseek( f, 0, SEEK_END );
	len=ftell( f );
	fseek( f, 0, SEEK_SET );
	if( len<=0 ) {
		msgOut( MSG_ERROR, "Unable to read config file %s", cfgfile.c_str() );
		return false;
	}
	
	//alloc buffer
	buffer=(char*)malloc( sizeof(char)*(len+1) );
//	buffer = new char[len];
	if( buffer==NULL ) {
		msgOut( MSG_ERROR, "Unable to allocate config file buffer (%s), size %d bytes", cfgfile.c_str(), len );
		return false;
	}
	bufsize=len+1;
	bzero( buffer, bufsize );
	
	// read file
	for( i=0; i<len; i++ )
	{
		buffer[i]=fgetc( f );
	}
	buffer[len]=0; //terminate string

	fclose( f );
	if( verbose ) msgOut( MSG_INIT, "Config file (%s) read. %d bytes", cfgfile.c_str(), len );
	return true;
}

/// Set group name, for example "game" or "marine". Note: all names are case sensitive.
void Config::setGroup( string grp )
{
	group=grp;
}

/// Read integer value from cfg file. Returns true if success.
int Config::getInt( string argname, int defaultvalue )
{
	char readkey[100], readtype[11]; //big enough??
	int pos, readvalue;
	string key=group+"."+argname;
	
	pos=seek( key );
	if( pos==-1 ) {
		if( verbose ) msgOut( MSG_WARN, "Key %s (int) not found from config", key.c_str() );
		return defaultvalue;
	}
	
	if( sscanf( buffer+pos, "%99s %10s %d", readkey, readtype, &readvalue ) != 3 ) {
		msgOut( MSG_WARN, "Config: error parsing key %s (int)", key.c_str() );
		return defaultvalue;
	}

	if( strcmp(readkey, key.c_str()) || strcmp(readtype, "int") ) {
		msgOut( MSG_WARN, "Config: invalid key %s (int)", key.c_str() );
		return defaultvalue;
	}

	if( verbose ) msgOut( MSG_INFO, "Key %s=%d (int) read from config", key.c_str(), readvalue );
	return readvalue;
}

float Config::getFloat( string argname, float defaultvalue )
{
	char readkey[100], readtype[11]; //big enough??
	int pos;
	float readvalue;
	string key=group+"."+argname;
	
	pos=seek( key );
	if( pos==-1 ) {
		if( verbose ) msgOut( MSG_WARN, "Key %s (float) not found from config", key.c_str() );
		return defaultvalue;
	}
	
	if( sscanf( buffer+pos, "%99s %10s %f", readkey, readtype, &readvalue ) != 3 ) {
		msgOut( MSG_WARN, "Config: error parsing key %s (float)", key.c_str() );
		return defaultvalue;
	}

	if( strcmp(readkey, key.c_str()) || strcmp(readtype, "float") ) {
		msgOut( MSG_WARN, "Config: invalid key %s (float)", key.c_str() );
		return defaultvalue;
	}

	if( verbose ) msgOut( MSG_INFO, "Key %s=%f (float) read from config", key.c_str(), readvalue );
	return readvalue;
}

/// Read bool value from cfg file. Returns true if success.
bool Config::getBool( string argname, bool defaultvalue )
{
	char readkey[100], readtype[11], readvalue[11]; //big enough??
	bool value;
	int pos;
	string key=group+"."+argname;
	
	pos=seek( key );
	if( pos==-1 ) {
		if( verbose ) msgOut( MSG_WARN, "Key %s (bool) not found from config", key.c_str() );
		return defaultvalue;
	}
	
	if( sscanf( buffer+pos, "%99s %10s %10s", readkey, readtype, readvalue ) != 3 ) {
		msgOut( MSG_WARN, "Config: error parsing key %s (bool)", key.c_str() );
		return defaultvalue;
	}

	if( strcmp(readkey, key.c_str()) || strcmp(readtype, "bool") ) {
		msgOut( MSG_WARN, "Config: invalid key %s (bool)", key.c_str() );
		return defaultvalue;
	}
	
	// compare reading
	if( !strcasecmp( readvalue, "true" ) ) value=true;
	else if( !strcasecmp( readvalue, "false" ) ) value=false;
	else if( !strcasecmp( readvalue, "1" ) ) value=true;
	else if( !strcasecmp( readvalue, "0" ) ) value=false;
	else if( !strcasecmp( readvalue, "yes" ) ) value=true;
	else if( !strcasecmp( readvalue, "no" ) ) value=false;
	else {
		msgOut( MSG_WARN, "Config: invalid key value %s (bool)", key.c_str() );
		return defaultvalue;
	}

	if( verbose ) msgOut( MSG_INFO, "Key %s=%s (bool) read from config", key.c_str(), readvalue );
	
	return value;
}

string Config::getString( string argname, string defaultvalue )
{
	char readkey[99], readtype[11]; //big enough??
	string readvalue;
	int pos, q1, q2, i;
	string key=group+"."+argname;
	
	pos=seek( key );
	if( pos==-1 ) {
		if( verbose ) msgOut( MSG_WARN, "Key %s (string) not found from config", key.c_str() );
		return defaultvalue;
	}
	
	if( sscanf( buffer+pos, "%99s %10s", readkey, readtype ) != 2 ) {
		msgOut( MSG_WARN, "Config: error parsing key %s (string)", key.c_str() );
		return defaultvalue;
	}

	if( strcmp(readkey, key.c_str()) || strcmp(readtype, "string") ) {
		msgOut( MSG_WARN, "Config: invalid key %s (string)", key.c_str() );
		return defaultvalue;
	}
	
	// read string in quotes:
	// get quote offsets
	q1=seek( "\"", pos );
	q2=seek( "\"", q1+1 );

	if( q1==-1 || q2==-1 ) {
		msgOut( MSG_WARN, "Config: error reading string value (key %s)", key.c_str() );
		return defaultvalue;
	}
	/* other way to do this:
	const char  *buffer = "   \"Eki E Esimerkki\"   ";
	char         name[64] = { '\0' };
	int          status;

	status = sscanf(buffer, " \"%64[a-zA-Z ]\" ", name);
	printf("Status: %d\nName: <%s>\n", status, name);	
	*/
	
	// read it
	readvalue="";
	for( i=q1+1; i<q2; i++ )
		readvalue+=buffer[i];

	if( verbose ) msgOut( MSG_INFO, "Key %s='%s' (string) read from config", key.c_str(), readvalue.c_str() );
	
	return readvalue;
}


int Config::seek( string str, int startoffset )
{
	int i;//, m=0;
	
	if( !buffer ) return -1;
	
	if( startoffset <0 ) startoffset=0;
	if( startoffset > signed(bufsize-str.size()) ) startoffset=signed(bufsize-str.size());
	
	for( i=startoffset; i<signed(bufsize-str.size()); i++ )
	{
		if( buffer[i]==str[0] )
		{
			if( !strncmp( str.c_str(), buffer+i, str.size() ) ) return i;
		}
	}
	return -1;
}


int Config::getInt( string argname )
{
	char readkey[100], readtype[11]; //big enough??
	int pos, readvalue;
	string key=group+"."+argname;
	
	pos=seek( key );
	if( pos==-1 ) {
		if( verbose ) msgOut( MSG_WARN, "Key %s (int) not found from config", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (int)", key.c_str() );
	}
	
	if( sscanf( buffer+pos, "%99s %10s %d", readkey, readtype, &readvalue ) != 3 ) {
		msgOut( MSG_WARN, "Config: error parsing key %s (int)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (int)", key.c_str() );
	}

	if( strcmp(readkey, key.c_str()) || strcmp(readtype, "int") ) {
		msgOut( MSG_WARN, "Config: invalid key %s (int)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (int)", key.c_str() );
	}

	if( verbose ) msgOut( MSG_INFO, "Key %s=%d (int) read from config", key.c_str(), readvalue );
	return readvalue;
}

float Config::getFloat( string argname )
{
	char readkey[100], readtype[11]; //big enough??
	int pos;
	float readvalue;
	string key=group+"."+argname;
	
	pos=seek( key );
	if( pos==-1 ) {
		if( verbose ) msgOut( MSG_WARN, "Key %s (float) not found from config", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (float)", key.c_str() );
	}
	
	if( sscanf( buffer+pos, "%99s %10s %f", readkey, readtype, &readvalue ) != 3 ) {
		msgOut( MSG_WARN, "Config: error parsing key %s (float)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (float)", key.c_str() );
	}

	if( strcmp(readkey, key.c_str()) || strcmp(readtype, "float") ) {
		msgOut( MSG_WARN, "Config: invalid key %s (float)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (float)", key.c_str() );
	}

	if( verbose ) msgOut( MSG_INFO, "Key %s=%f (float) read from config", key.c_str(), readvalue );
	return readvalue;
}

/// Read bool value from cfg file. Returns true if success.
bool Config::getBool( string argname )
{
	char readkey[100], readtype[11], readvalue[11]; //big enough??
	bool value;
	int pos;
	string key=group+"."+argname;
	
	pos=seek( key );
	if( pos==-1 ) {
		if( verbose ) msgOut( MSG_WARN, "Key %s (bool) not found from config", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (bool)", key.c_str() );
	}
	
	if( sscanf( buffer+pos, "%99s %10s %10s", readkey, readtype, readvalue ) != 3 ) {
		msgOut( MSG_WARN, "Config: error parsing key %s (bool)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (bool)", key.c_str() );
	}

	if( strcmp(readkey, key.c_str()) || strcmp(readtype, "bool") ) {
		msgOut( MSG_WARN, "Config: invalid key %s (bool)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (bool)", key.c_str() );
	}
	
	// compare reading
	if( !strcasecmp( readvalue, "true" ) ) value=true;
	else if( !strcasecmp( readvalue, "false" ) ) value=false;
	else if( !strcasecmp( readvalue, "1" ) ) value=true;
	else if( !strcasecmp( readvalue, "0" ) ) value=false;
	else if( !strcasecmp( readvalue, "yes" ) ) value=true;
	else if( !strcasecmp( readvalue, "no" ) ) value=false;
	else {
		msgOut( MSG_WARN, "Config: invalid key value %s (bool)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (bool)", key.c_str() );
	}

	if( verbose ) msgOut( MSG_INFO, "Key %s=%s (bool) read from config", key.c_str(), readvalue );
	
	return value;
}

string Config::getString( string argname )
{
	char readkey[99], readtype[11]; //big enough??
	string readvalue;
	int pos, q1, q2, i;
	string key=group+"."+argname;
	
	pos=seek( key );
	if( pos==-1 ) {
		if( verbose ) msgOut( MSG_WARN, "Key %s (string) not found from config", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (string)", key.c_str() );
	}
	
	if( sscanf( buffer+pos, "%99s %10s", readkey, readtype ) != 2 ) {
		msgOut( MSG_WARN, "Config: error parsing key %s (string)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (string)", key.c_str() );
	}

	if( strcmp(readkey, key.c_str()) || strcmp(readtype, "string") ) {
		msgOut( MSG_WARN, "Config: invalid key %s (string)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (string)", key.c_str() );
	}
	
	// read string in quotes:
	// get quote offsets
	q1=seek( "\"", pos );
	q2=seek( "\"", q1+1 );

	if( q1==-1 || q2==-1 ) {
		msgOut( MSG_WARN, "Config: error reading string value (key %s)", key.c_str() );
		msgOut( MSG_FATAL, "Missing required key %s (string)", key.c_str() );
	}
	/* other way to do this:
	const char  *buffer = "   \"Eki E Esimerkki\"   ";
	char         name[64] = { '\0' };
	int          status;

	status = sscanf(buffer, " \"%64[a-zA-Z ]\" ", name);
	printf("Status: %d\nName: <%s>\n", status, name);	
	*/
	
	// read it
	readvalue="";
	for( i=q1+1; i<q2; i++ )
		readvalue+=buffer[i];

	if( verbose ) msgOut( MSG_INFO, "Key %s='%s' (string) read from config", key.c_str(), readvalue.c_str() );
	
	return readvalue;
}

