/***************************************************************************
                          configfile.h  -  description
                             -------------------
    begin                : Mon Nov 5 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef CONFIGFILE_H
#define CONFIGFILE_H

#include "allincludes.h"

/**Config file parser.
  *@author T Kontkanen
  */

class Config {
public:
	Config();
	~Config();
	/// Read config file. Returns true if success.
	bool readFile( string cfgfile );
	
	/// Set group name, for example "game" or "level". Note: all names are case sensitive.
	void setGroup( string grp );
	/// Read integer value from cfg file. If reading fails, deafultvalue is returned.
	int getInt( string argname, int defaultvalue );
	/// Read floating point value from cfg file. If reading fails, deafultvalue is returned.
	float getFloat( string argname, float defaultvalue );
	/// Read bool value from cfg file. If reading fails, deafultvalue is returned.
	bool getBool( string argname, bool defaultvalue );
	/// Read text string from cfg file.  If reading fails, deafultvalue is returned.
	string getString( string argname, string defaultvalue );
	
	/// Read integer value from cfg file. If reading fails, a fatal error will be reported (app aborts).
	int getInt( string argname );
	/// Read floating point value from cfg file. If reading fails, a fatal error will be reported (app aborts).
	float getFloat( string argname );
	/// Read bool value from cfg file. If reading fails, a fatal error will be reported (app aborts).
	bool getBool( string argname );
	/// Read text string from cfg file.  If reading fails, a fatal error will be reported (app aborts).
	string getString( string argname );
	
	/// If onoff is false, no messages will be printed
	void setVerbosity( bool onoff ) { verbose=onoff; }
private:
	/// Searchs a string from buffer and returns str start offset. Returns -1 if string not found.
	int seek( string str, int startoffset=0 );
	
	string group;
	char *buffer; // cfg file buffer
	int bufsize;
	bool verbose;
};

#endif
