/***************************************************************************
                          dummymovement.h  -  description
                             -------------------
    begin                : Sun Aug 26 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef DUMMYMOVEMENT_H
#define DUMMYMOVEMENT_H


/**Movement for buildings and other stationary objects (not moving)
  *@author T Kontkanen
  */

#include "renderer.h"
#include "ground.h"
#include "game.h"
#include "allincludes.h"
#include "gamedefs.h"
#include "vector.h"

class DummyMovement {
public:
	DummyMovement( Game *pgame );
	~DummyMovement();

	Vector getPosition() { return position; }
	float getSpeed() { return velocity; }
	float getAngle() { return angle; }
	float getPitch() { return pitch; }
	float getTilt() { return tilt; }

	//action
	void moveSetDestination( Vector destpos ) { destPosition=destpos; }
	Vector moveGetDestination() { return destPosition; }
	void moveStep(); // make a move (one call per frame)
	
	void setPosition( Vector &pos ) {
		position=pos;
		position.setZ(mGround->getElevation( pos.x(), pos.y() ) );//set elevation
		moveSetDestination( pos );
	}
	void setTurnRate( float rate ) { turnrate=rate; }
	void setTopSpeed( float speed ) { topspeed=speed; }
	void setAcceleration( float a ) { acceleration=a; }
	
private:
	// own data
	Vector position, destPosition;
	float angle, pitch, tilt; //angle=z-axis rotation, pitch=x-axis and tilt=y-axis
	float velocity; /* m/s */
	float turnrate /* rad/s */, acceleration /* m/s */, topspeed /* m/s */;
	float positionTolerance; // destination position tolerance in meters (radius)
	float turnrange_near, turnrange_far, radarresolution; // distance when unit starts turnign if wall or other unit is detected ahead
	
	//needed information, don't use same names with Unit to avoid conficts (may be unnecessary?)
	Renderer *mRenderer;
	Ground *mGround;
	Game *mGame;
};


#endif
