/***************************************************************************
                          font.cpp  -  description
                             -------------------
    begin                : Tue Sep 4 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "font.h"



BFont::BFont( Game *pgame, const char *fontbitmapfile ){
	int i;
	game=pgame;
	
	r=g=b=a=ha=hr=hg=1.0;
	hb=0.0;
	
	if( game->renderer->loadTexture( fontbitmapfile ) )
		texture=game->renderer->getTexture( fontbitmapfile );
	else
		msgOut( MSG_ERROR, "Unable create font" );
	
	for(i=0; i<256; i++ ) widths[i]=texture->width()/16; //default width
	metricsloaded=false;
	stretching=1;
}

BFont::BFont( Game *pgame, string fontbitmapfile ){
	int i;
	game=pgame;
	
	r=g=b=a=ha=hr=hg=1.0;
	hb=0.0;
	
	if( game->renderer->loadTexture( fontbitmapfile ) )
		texture=game->renderer->getTexture( fontbitmapfile );
	else
		msgOut( MSG_ERROR, "Unable create font" );
	
	for(i=0; i<256; i++ ) widths[i]=texture->width()/16; //default width
	metricsloaded=false;
	stretching=1;
}

BFont::~BFont(){
	game->renderer->freeTexture( texture );
}

int BFont::getLineHeight()
{
	return texture->height()/16;
}

bool BFont::loadFontMetrics( const char *filename )
{
	int i;
	FILE *f;
	
	if( metricsloaded==true ) return true; // already loaded
	
	msgOut( MSG_INIT, "Loading font metrics %s", filename );
	
	f=fopen( filename, "rb" );
	if( f ==NULL ) {
		msgOut( MSG_ERROR, "Unable to open font metrics file %s", filename );
		return false;
	}
	for( i=0; i<256; i++ )
		widths[i]=fgetc(f);
	fclose(f);
	metricsloaded=true;
	
	return true;
}

int BFont::getCharWidth( unsigned char ch )
{
//	if( ch < 0 || ch>255 ) return 0;
	return widths[ch]+stretching;
}

void BFont::getRGBA( float &fr, float &fg, float &fb, float &fa )
{
	fr=r;
	fg=g;
	fb=b;
	fa=a;
}

void BFont::setRGBA( float fr, float fg, float fb, float fa )
{
	r=fr;
	g=fg;
	b=fb;
	a=fa;
}

void BFont::getHotkeyRGBA( float &fr, float &fg, float &fb, float &fa )
{
	fr=hr;
	fg=hg;
	fb=hb;
	fa=ha;
}

void BFont::setHotkeyRGBA( float fr, float fg, float fb, float fa )
{
	hr=fr;
	hg=fg;
	hb=fb;
	ha=fa;
}

int BFont::getBlockWidth()
{
	return texture->width()/16;
}

int BFont::getCharXPos( unsigned char ch  )
{
	return ((ch%16)*getBlockWidth());
}

int BFont::getCharYPos( unsigned char ch  )
{
	return ((ch/16)*getLineHeight());
}


