/***************************************************************************
                          font.h  -  description
                             -------------------
    begin                : Tue Sep 4 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef FONT_H
#define FONT_H


/**Font class for bitmapped fonts
  *@author T Kontkanen
  */

#include "game.h"
#include "texture.h"

class Game;

class BFont {
public: 
	BFont( Game *pgame, const char *fontbitmapfile );
	BFont( Game *pgame, string fontbitmapfile ); /*{ new BFont( pgame, fontbitmapfile.c_str() ); }*/
	~BFont();

	/// Load a font metrics file (*.fmi) to get correct font spacing
	bool loadFontMetrics( const char *filename );
	bool loadFontMetrics( string filename ) { return loadFontMetrics( filename.c_str() ); }
	int getLineHeight();
	int getCharWidth( unsigned char ch );
	int getCharXPos( unsigned char ch  );
	int getCharYPos( unsigned char ch  );
	/// Return a GL texture ID
	int getTextureID() { return texture->getTextureID(); }
	/// Return a pointer to Texture
	Texture* getTexture() { return texture; }
	/// Return a width of one character block
	int getBlockWidth();
	/// Get RGBA color values
	void getRGBA( float &fr, float &fg, float &fb, float &fa );
	/// Get hotkey RGBA color values
	void getHotkeyRGBA( float &fr, float &fg, float &fb, float &fa );
	/// Set RGBA color values
	void setRGBA( float fr, float fg, float fb, float fa );
	/// Set hotkey RGBA color values
	void setHotkeyRGBA( float fr, float fg, float fb, float fa );
private:
	float r,g,b,a, hr, hg, hb, ha; //color
	Game *game;
	bool metricsloaded;
	Texture *texture;
	int widths[256];
	int stretching;
};

#endif
