/***************************************************************************
                          game.cpp  -  description
                             -------------------
    begin                : Thu Jul 19 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "game.h"
#include "renderer.h"
#include "ground.h"
#include "player.h"
#include "font.h"
#include "guibutton.h"
#include "gamegui.h"
#include "unitid.h"
#include "unit.h"

using namespace msg;


Game::Game( string configfilename ){
	int i;	

	//read config file
	config.readFile( configfilename );

	// clear things
	for( i=0; i<MAX_PLAYERS; i++ ) players[i]=NULL;
	minimap=NULL;
	
	// init game
	if( init() ) { msgOut( MSG_FATAL, "Game initialization failed" ); }
	
	// load various settings
	config.setGroup( "game" );
	setDebugLevel( config.getInt("debugLevel",2) );

	// create randerer
	renderer = new Renderer( this, config.getInt("xResolution",1024), config.getInt("yResolution",768), config.getInt("screenDepth",24), config.getBool("fullscreen",false) );
//	renderer = new Renderer( this, 300,300, config.getInt("screenDepth",24), config.getBool("fullscreen",false) );
	
	// mouse
	config.setGroup( "game" );
	input.setSysCursor( config.getBool( "useSysCursor", false ) );
	input.setInputGrapping( config.getBool( "grapInput", false ) );
	
	/// load fonts
	defaultFont = new BFont( this, dataRoot()+FONTPATH+config.getString("defaultFont","v11")+".png" );
	defaultFont->loadFontMetrics( dataRoot()+FONTPATH+config.getString("defaultFont","v11")+".fmi" );
	tooltipFont = new BFont( this, dataRoot()+FONTPATH+config.getString("tooltipFont","v11")+".png" );
	tooltipFont->loadFontMetrics( dataRoot()+FONTPATH+config.getString("tooltipFont","v11")+".fmi" );
	buttonFont = new BFont( this, dataRoot()+FONTPATH+config.getString("buttonFont","menufont")+".png" );
	buttonFont->loadFontMetrics( dataRoot()+FONTPATH+config.getString("buttonFont","menufont")+".fmi" );
	
	iconNone=renderer->loadTexture( dataRoot()+ICONPATH+"noaction.png" );
	iconBuildMarine=renderer->loadTexture( dataRoot()+ICONPATH+"buildmarine.png" );
	iconAttack=renderer->loadTexture( dataRoot()+ICONPATH+"attack.png" );
	iconMove=renderer->loadTexture( dataRoot()+ICONPATH+"move.png" );
	iconCancel=renderer->loadTexture( dataRoot()+ICONPATH+"cancel.png" );

	ground = new Ground( this );
	input.setRenderer( renderer );
	gui=new GameGUI( this );
	unitIDcounter=0; //reset counter
}

Game::~Game(){	
	int i;
		
	for( i=0; i<MAX_PLAYERS; i++ ) if( players[i]!=NULL ) delete( players[i] );
	delete( ground );
	renderer->freeTexture( dataRoot()+ICONPATH+"noaction.png" );
	delete( gui );
	delete( renderer );
	unInit();
}


int Game::init(){
	//output colring
	config.setGroup( "game" );
	setColoring( config.getBool( "colouredOutput", true ) );
	
	// set data root
	data_root=config.getString( "dataRoot", "/files/executor/" );
	if( data_root[data_root.size()-1]!= '/' ) data_root+="/"; // add slash after path

	//init SDL stuff
	atexit(SDL_Quit);
	if (SDL_Init(SDL_INIT_VIDEO | SDL_INIT_TIMER ) < 0)
	{
		msgOut( MSG_FAILURE, "Unable to initialize SDL: %s", SDL_GetError() );
		return -1;
	}
	SDL_EnableUNICODE(1);
	
	return 0;
}

int Game::unInit(){
	//uninit SDL stuff
	input.setInputGrapping( false );
	SDL_Quit();
	
	return 0;
}


int Game::initGame(){
	//load map
//	ground->loadMap( "/files/images/omat/crater.map" );

	config.setGroup( "game" );
	loadLevel( dataRoot()+MAPPATH+config.getString("defaultLevel","isles.level") );
	config.setGroup( "game" );
//	defaultFont->setRGBA( 0.5,0,0,1 );
	
	thisplayernum=0;
	players[0]=new Player( this, 0 );
	players[0]->createInitialUnits();
	players[1]=new Player( this, 1 );
	players[1]->createInitialUnits();
	
	ground->setMapReservationLine( 96, 0, 96, 100, true );
	ground->setMapReservationLine( 200, 00, 300, 100, true );
	ground->setMapReservationLine( 202, 00, 302, 100, true );
		
	return 0;
}


void Game::render()
{
	int i;
	
	// Render everything
	renderer->applyCamera( camerasmooth );
	renderer->clearBuffer();
	if( renderer->cameraMoved() )
		ground->cameraMoved();
	
	ground->renderTerrain();
	
	// get cursor 3d position on terrain
	mouseonground=renderer->screenToWorld( input.mouseX(), input.mouseY() );
	
	for( i=0; i<MAX_PLAYERS; i++ )
		if( players[i]!=NULL ) players[i]->renderUnits();

	// get cursor 3d position in with objects and terrain
	mouse3dpos=renderer->screenToWorld( input.mouseX(), input.mouseY() );

	//calculate fps
	framecounter++;
	frame_difftime=input.diffTime();
	fpstime+=frame_difftime;
	if( fpstime > 5000 )
	{
		fps=1000*framecounter/fpstime;
		framecounter=0;
		fpstime=0;
	}
	renderer->processParticleEffects(frame_difftime);
	renderer->renderParticleEffects();
	renderer->renderParticleQueue();
	
	//render gui on top of everything else
	gui->render();

	// show finished frame
	renderer->swapBuffers();
}

int Game::mainLoop(){
	bool done=false;
	int main_diff=0, unit_diff=0;
	Vector pos, dest;
	int i;
	
	config.setGroup("game");
	float camerasmooth_rate=config.getFloat( "cameraSmoothning", 0.9 );
//	SDL_Delay( 1000 );
		
	msgOut(MSG_INFO, "Entering mainloop..." );
	
	input.pollEvents(); //zero difftime
	while( !done )
	{
//		input.frameDelay();
//		SDL_Delay( 10 );
		input.pollEvents();
	
		main_diff+=input.diffTime();
		unit_diff+=input.diffTime();

		camerasmooth=1.0;
		while( main_diff > 0 )
		{
			main_diff-=MAINLOOP_TIME;
			
			camerasmooth*=camerasmooth_rate; // reduce camera smoothing
			
			gui->refreshGUI(); // update gui
			
		} //end of time syncronized mainloop
		
		while( unit_diff> 0 ) //move units at FRAMERATE (default 30FPS)
		{
			unit_diff-=UNIT_CYCLE_TIME;
			for( i=0; i<MAX_PLAYERS; i++ )
				if( players[i]!=NULL ) players[i]->unitStep();
		}
		
		/**** non time syncronized code: ****/
		
		gui->updateFrame(); // call every frame. update camera, buttons etc
		for( i=0; i<MAX_PLAYERS; i++ )
			if( players[i]!=NULL ) players[i]->refresh();
		
		/* handle keyboard input */
		/// quit
		if( input.key(SDLK_ESCAPE) ) done=true;
		/// alt+enter toggles fullscreen
		if( input.key( SDLK_RALT ) || input.key( SDLK_LALT ) && input.keyClicked( SDLK_RETURN ) ) renderer->toggleFullScreen();
		if( input.keyClicked( SDLK_F12 ) ) renderer->toggleWireframe();
		if( input.keyClicked( SDLK_F11 ) ) ground->toggleTesslation();
		if( input.keyClicked( SDLK_F10 ) ) input.setInputGrapping( false );
		if( input.keyClicked( SDLK_F9 ) ) input.setInputGrapping( true );
		
		// finally render frame and display it
		render();
	}
	
	msgOut(MSG_INFO, "Leaving mainloop..." );
	return 0;
}


int Game::screenW()
{
	return renderer->getScreenW();
}
int Game::screenH()
{
	return renderer->getScreenH();
}

UnitID Game::createUnitID( Unit *unit )
{
	int p, u;
	UnitID newid;
	
	newid.setID(unitIDcounter++);

/*	for( p=0; p<MAX_PLAYERS; p++ )
		if( players[p]!=NULL )
		{
			printf("searchplayer %d\n", p);
			u=players[p]->getUnitNumber( unit );
			if( u==-1 ) continue;
			else {
				newid.setPlayernum( p );
				newid.setUnitnum( u );
				return newid;
			}
		}*/
		
	// unit not found
//	msgOut( MSG_ERROR, "Unable to create valid UnitID, Unit not found" );
	return newid;
}

Unit *Game::getUnit( UnitID &id )
{
	int p, u;
	static int c=0, h1=0, h2=0;
	
	// first check if id unit/player information helps. 99,5% of searchs end here.
/*	if( players[id.getPlayernum()]!=NULL )
		if( players[id.getPlayernum()]->units[id.getUnitnum()]!=NULL )
			if( players[id.getPlayernum()]->units[id.getUnitnum()]->getID()==id )
				return players[id.getPlayernum()]->units[id.getUnitnum()];*/
	if( players[id.playernum]!=NULL )
		if( players[id.playernum]->units[id.unitnum]!=NULL )
			if( players[id.playernum]->units[id.unitnum]->getID().id==id.id )
				return players[id.playernum]->units[id.unitnum];

	for( p=0; p<MAX_PLAYERS; p++ ) //loop for all players
	{
		if( players[p]==NULL ) continue; //skup NULL players
		for( u=0; u<MAX_UNITS; u++ )  //loop for all units
		{
			if( players[p]->units[u]==NULL ) continue;// skip NULL units
			
//printf( "p%d u%d->p%d u%d\n", id.getPlayernum(), id.getUnitnum(), p, u);
			if(  players[p]->units[u]->getID()==id ) return players[p]->units[u];
		}
	}
	
	//not found
	return NULL;
}

bool Game::insertNewUnit( Unit *upointer, int player )
{
	if( players[player]==NULL ) return false;
	
	return players[player]->insertNewUnit( upointer );
}

void Game::deleteUnit( Unit *upointer )
{
	int i;
	for( i=0; i<MAX_PLAYERS; i++ )
		if( players[i]!=NULL ) players[i]->deleteUnit( upointer );
}

void Game::deleteUnit( UnitID id )
{
	Unit *u=getUnit( id );
	if( u!=NULL )
		deleteUnit( u );
}

UnitID Game::firstUnitID( int player=-1 )
{
	int i;
	UnitID first;
	// TODO: advance if player has been deleted. single player search

	first.setSearchPlayernum( player );
	
	for( i=0; i<MAX_PLAYERS; i++ )
	{
		if( players[i] )
		{
			first=players[i]->firstUnitID();
			if( !players[i]->iterationEnd(first) )  //if there is 1 or more units
				i=MAX_PLAYERS; // end search
		}
	}
	
//	if( player == -1 )
//		first.setPlayernum( 0 ); //start searching from first player
//	else
//		first.setPlayernum( player ); //start searching from specific player
	
	return first;
}



void Game::nextUnitID( UnitID &id )
{
	int i, p;
//	UnitID find;
	
	// TODO: advance if player has been deleted. single player search
	
	p=id.getPlayernum();
	if( players[p] ) // do player exist?
	{
		if( players[p]->lastUnitID( id ) ) // advance to next player
		{
			for( i=p+1; i<MAX_PLAYERS; i++ ) // starting from next player
			{
				if( players[i] )
				{
					if( players[i]->unitCount() )  // if there is one or more units
					{
						id=players[i]->firstUnitID();
						return; // we've found a new player with units
					}
				}
			}
			players[p]->nextUnitID( id ); // no more players, cause iteration end
			return;
		}
		else
		{
			players[p]->nextUnitID( id );
			return;
		}
	}
	
	return;
}

bool Game::iterationEnd( UnitID id )
{
	int p, lastplayer=0, i;
	// TODO: advance if player has been deleted. single player search
	
	// end reached
	p=id.getPlayernum();
	if( !players[p] ) return true; //FIXME

	for( i=0; i<MAX_PLAYERS; i++ )
		if( players[i] && players[i]->unitCount() ) lastplayer=i;
	
	if( p >= lastplayer && players[p]->iterationEnd(id) ) return true;
	return false;
}

unsigned int Game::rand()
{
	// TODO: own algorithm
	return random();
}

bool Game::loadLevel( string levelconffile )
{
	Config lev;
	Color skycol, sdiff, samb;
	Vector spos;
	lev.readFile( levelconffile );
	lev.setGroup("level");
	string sur;
	
	//map
	if( minimap ) renderer->freeTexture( minimap ); // delete old
	minimap=renderer->loadTexture( dataRoot()+MAPPATH+lev.getString("minimapFile") );
	ground->loadMap( lev );
	
	sur=lev.getString("surroundingObj", "none");
	if( sur!="none" )// if surrounding specified
		ground->loadSurrounding( dataRoot()+MAPPATH+sur,
			lev.getFloat("surroundingScale", 1.0 ), lev.getFloat("surroundingAlt", 0.0 ) );
	else
		ground->loadSurrounding( "none",
			lev.getFloat("surroundingScale", 1.0 ), lev.getFloat("surroundingAlt", 0.0 ) );

	//sky
	skycol.set( lev.getFloat("skyColorR",0.5), lev.getFloat("skyColorG",0.5), lev.getFloat("skyColorB",0.5), 1.0 );
	renderer->setSkyColor( skycol );
	
	//sun
	sdiff.set( lev.getFloat("sunDiffuseR",1.0), lev.getFloat("sunDiffuseG",1.0), lev.getFloat("sunDiffuseB",1.0), 1.0 );
	samb.set( lev.getFloat("sunAmbientR",0.1), lev.getFloat("sunAmbientG",0.1), lev.getFloat("sunAmbientB",0.1), 1.0 );
	spos.set( lev.getFloat("sunPosX",100000),lev.getFloat("sunPosY",100000),lev.getFloat("sunPosZ",100000) );
	renderer->setSun( spos, sdiff, samb );
	
	//fog
	renderer->setFog( lev.getInt("fogType",1),
						lev.getFloat("fogStart",100),
						lev.getFloat("fogEnd",4400),
						lev.getFloat("fogDensity",0.0005) );
	renderer->setViewDistance( lev.getFloat("viewDistance",4500) );
	
	return true;
}

void Game::perfTest()
{
/*	 int i;
	int frames=0;
	int time=0;
	float avgfps, particleresult;
	
	for( i=0; i<5000; i++ )
	{
	}
	
	avgfps=float(frames)/float(time);
	msgOut( MSG_INFO, "Scene %.1f FPS", avgfps );*/
	
}

#define RANDTABLE_SIZE 10000
unsigned int Game::fastRand()
{
	int i;
	static int curr=-1;
	static unsigned int randtab[RANDTABLE_SIZE];
	
	if( curr==-1 ) //first call, generate table
	{
		for( i=0; i<RANDTABLE_SIZE; i++ ) randtab[i]=this->rand();
		curr=0;
	}
	
	curr++;
	if( curr >= RANDTABLE_SIZE ) curr=0; // repeat
	
	return randtab[curr];
}

