/***************************************************************************
                          game.h  -  description
                             -------------------
    begin                : Thu Jul 19 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/
#ifndef GAME_H
#define GAME_H


#include "SDL/SDL.h"
#include <tgl.h>

// own stuff
#include "allincludes.h"
#include "gamedefs.h"

#include "input.h"
#include "vector.h"
#include "color.h"
#include "configfile.h"

class Player;
class Unit;
class Renderer;
class Ground;
class BFont;
class GUIButton;
class GameGUI;
class UnitID;

class Game
{
public:
	// Constructor. Config file is read from configfilename.
	Game( string configfilename );
	~Game();
	/// Load stuff that are needed to run game.
	int initGame();
	/// Enter mainloop. Function returns when game quits.
	int mainLoop();
	/// Render everything and flip buffers
	void render();
	
	/// Shortcut to renderer->getScreenW()
	int screenW();
	/// Shortcut to renderer->getScreenH()
	int screenH();
	/// Get mouse cursor 3d position on terrain
	Vector getMouseGroundPosition() { return mouseonground; }
	/// Get mouse cursor 3d position on screen's 3d space (including terrain & objects).
	Vector getMouse3DPosition()  { return mouse3dpos; }
	
	
	/// Return number of this player.
	int getMyPlayerNum() { return thisplayernum; }
	Player *getMyPlayer() { return players[thisplayernum]; }
	/// Returns a random number. Use this instead of system random functions to ensure similar results on different platforms.
	unsigned int rand();
	/// Fast random function. Warning: function is repeating pre-defined pattern.
	unsigned int fastRand();
	
	//global game glasses
	Ground *ground;
	Renderer *renderer;
	Player *players[MAX_PLAYERS];
	GameGUI *gui;
        Config config;

        //global fonts
	BFont *defaultFont;
	BFont *tooltipFont;
	BFont *buttonFont;

	// global textures
	Texture *minimap;
	Texture *iconBuildMarine, *iconAttack, *iconMove, *iconCancel, *iconNone;

	/// Returns a root directory of data files (for e.g. "/usr/share/executor/").
	string dataRoot() { return data_root; }
	/// Returns a pointer to Unit for specified UnitID. NOTE: Returns NULL if Unit doesn't exist any more!
	Unit *getUnit( UnitID &id );
	
	bool insertNewUnit( Unit *upointer, int player );
	void deleteUnit( Unit *upointer );
	void deleteUnit( UnitID id );
	UnitID firstUnitID( int player=-1 );
	void nextUnitID( UnitID &id );
	bool iterationEnd( UnitID id );
	
protected:
	/// Returns an unique UnitID. This is used by Unit constructor.
	UnitID createUnitID( Unit *unit );
	/// Load new level. Returns true if successful.
	bool loadLevel( string levelconffile );
private:
	int init();
	int unInit();

	/// Run a simple performance test.
	void perfTest();
	
	Input input;
	
	friend Ground;
	friend Renderer;
	friend Player;
	friend BFont;
	friend GUIButton;
	friend GameGUI;
	friend Unit;
	
	float fps,framecounter, fpstime, frame_difftime, camerasmooth;
	int thisplayernum;
	string data_root; // root dir for data files
	int unitIDcounter;
	
	Vector mouseonground, mouse3dpos;
	
};


#endif

