/***************************************************************************
                          gamedefs.h  -  description
                             -------------------
    begin                : Tue Jul 24 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

 #ifndef GAMEDEFS_H
 #define GAMEDEFS_H


 #define M_ANY -1
 #define M_LEFT 0
 #define M_MIDDLE 1
 #define M_RIGHT 2

 #define SCREEN_BORDER_SIZE 10

 //limitations
#define MAX_PLAYERS 16
#define MAX_UNITS 500
#define MAX_TGL_OBJECTS 500
#define MAX_FILE_NAME_LEN 256
#define MAX_TEXTURES 128 //limit for Renderer
#define MAX_TGL_TEXTURES 128 //limit for TGL
#define MAX_GUIBUTTONS 128 //limit for GameGUI

#define MAINLOOP_FPS 100
#define MAINLOOP_TIME (1000/MAINLOOP_FPS) // how long time (ms) one mainloop cycle takes
#define MAINLOOP_FPSINV (1.0/float(MAINLOOP_FPS))
#define FRAMERATE 33 // units' framerate
#define UNIT_CYCLE_TIME (1000/FRAMERATE) // how often to step units
#define RAMERATEINV (1.0/float(FRAMERATE))
#define GROUND_GRID_RESOLUTION 2 // size in meter of resolution how tightly units and buildings can be positioned,
										// smaller number means higher resolution but it eats more memory

//macros:

#define MAINLOOP_PER_FRAME(x) (float(x)*MAINLOOP_FPSINV) //multiplying is faster than x/FRAMERATE on x86
#define PER_FRAME(x) (float(x)*RAMERATEINV) //multiplying is faster than x/FRAMERATE on x86
#define RAD_TO_DEG(x) (float(x)*57.2957795131)
#define DEG_TO_RAD(x) (float(x)*0.1745329252)

//Gui action buttons are sorted by action numbers. To make nice grouping of action numbers put new actions into right groups:
//ID,   Action types:
//0-1023: Unit standard actions
//1024-2047: Special actions
//2048-4095: Settings, i.e. togglable (on/off) features
//4096-8191: Constructing actions

//standard:
#define A_NONE 0 // not shown
#define A_MOVE 2
#define A_STOP 3
#define A_CANCEL 4
#define A_HARVEST_REPAIR 5 //harvest mineral or repair units
#define A_HOLD_POSITION 100
#define A_PATROL 101
#define A_ATTACK 102
//special actions:
#define A_TEST 1024 // for testing/debugging purposes only

//construct:
#define A_BUILD_MARINE 4100 // for testing/debugging purposes only
#define A_BUILD_DESTROYER 4101 // for testing/debugging purposes only



//unit types
#define UNIT_MARINE 1

///Data paths, Full path can be composed with game->dataRoot()+SOMEPATH+"somefile.dat".
#define CONFIGFILE "/etc/executor/game.conf" //filan path = $HOME+CONFIGFILE
#define MODELPATH "models/"
#define IMAGEPATH "images/"
#define FONTPATH "fonts/"
#define TEXTUREPATH "images/textures/"
#define ICONPATH "images/"
#define MAPPATH "maps/"
#define SOUNDPATH "sounds/"
#define UNITPATH "units/"

//model files
//#define MARINE_MODEL "models/marine.t3d"
//#define MARKER_MODEL "models/cross.t3d"
//#define DESTROYER_MODEL "models/bcruiser.t3d"
#define MARINE_MODEL "marine.t3d"
#define MARKER_MODEL "models/3dpointer.t3d"
#define DESTROYER_MODEL "models/bcruiser.t3d"
//#define MARINE_MODEL "/files/3d/executor/medic.t3d"


 #endif
