/***************************************************************************
                          gamegui.cpp  -  description
                             -------------------
    begin                : Sun Sep 9 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "gamegui.h"
#include "guibutton.h"
#include "game.h"
#include "ground.h"
#include "texture.h"
#include "player.h"
#include "unit.h"
#include "unitid.h"
#include <algorithm> //sort()

#define PANEL_W 256

GameGUI::GameGUI( Game *parent ){
	int i;

	msgOut( MSG_INIT, "Constructing GUI" );
	
	game=parent;
	renderer=game->renderer;
	players=game->players;
	
	//camera
	renderer->initCamera();
	game->config.setGroup( "gui" );
	cameraspeed=game->config.getFloat( "cameraSpeed", 1.0 );
	renderer->setCameraMode( CAMERA_ORBIT );
//	renderer->setCameraPosition( Vector(1000,1000,500) );
	renderer->setCameraLookAtDistance( 400 );
	renderer->setCameraAngle( 0.5, -0.5 );
	renderer->setCameraLookAt( Vector(100,100,100) );
	
	// lookat marker
	//marker
	renderer->loadTGLObject( game->dataRoot()+MARKER_MODEL );
	marker=renderer->getTGLObject( game->dataRoot()+MARKER_MODEL );
	marker_angle=0;

	buttonpanelcol.set(0,0.2,0,0.5);
	
	switch( game->config.getInt( "panelPosition", 1 ) ) {
		case 0: //left
			panelpos.set(0,0,0);
			break;
		case 1: //right
			panelpos.set(game->screenW()-PANEL_W,0,0);
			break;
		default:
			panelpos.set(game->screenW()-PANEL_W,0,0);
			break;
	}
	
	for( i=0; i<MAX_GUIBUTTONS; i++ )
	{
		buttons[i]=NULL;
	}
	
	actions.clear();
	actionbuttons.clear();
	
	selection_changed=true; //initial
	selection_in_progress=false;
	mouse_over_map=false;
	
	buttcount=0;
	initButtons();
	
	myplayernum=game->getMyPlayerNum();
}


GameGUI::~GameGUI(){
	int i;
	msgOut( MSG_UNINIT, "Destroying GUI" );
	
	renderer->freeTexture( "/files/images/executor/move.png" );	
	
	for( i=0; i<MAX_GUIBUTTONS; i++ )
		if( buttons[i]!=NULL ) delete buttons[i];
		
	renderer->freeTGLObject( game->dataRoot()+MARKER_MODEL );
}


void GameGUI::render()
{
	char str[256];
	
	// update panel position
	if( panelpos.x() != 0 ) //if not left side panel
		panelpos.set(game->screenW()-PANEL_W,0,0);
	
	// render lookat marker
	if( game->input.key(SDLK_z) || game->input.key(SDLK_a) ) {
		renderer->setRenderingMode( RENDER_TGL_OBJECT );
		Vector look=renderer->getCameraLookAt();
		marker->move( look.x(), look.y(), look.z() );
		marker->scale( 10, 10,10 );
		marker->rotate( 0, 0, 1, marker_angle );
		marker->draw();
		
		sprintf( str, "Alt %.0f meters", renderer->getCameraLookAt().z()-game->ground->getElevation(renderer->getCameraLookAt().x(), renderer->getCameraLookAt().y()) );
		renderer->setFont( game->defaultFont );
		renderer->renderText( renderer->getScreenW()/2+20, renderer->getScreenH()/2+20, str );
	}

	// destination marker
	if( execute_action.getActionID()!=A_NONE )
	{
		Vector dest;
		
		if( !mouse_over_map )
			dest=game->getMouse3DPosition();
		else
			dest=mouseMapToWorld();
		
		renderer->setRenderingMode( RENDER_TGL_OBJECT );
		marker->move( dest.x(), dest.y(), dest.z() );
		marker->scale( 10, 10,10 );
		marker->rotate( 0, 0, 1, marker_angle );
		marker->draw();
	}

	renderHUD();
	renderButtons();
	
	/// selection box
	if( selection_in_progress )
		renderer->renderSelectionBox( select_p1, select_p2 );
	
	
	game->input.renderMouseCursor();
}

void GameGUI::renderHUD()
{
	int i, p;
	UnitID it;
	char str[80];
	Vector mappos, pointpos;
	Color pointcol;
	
	renderer->renderBox( panelpos+Vector(0, PANEL_W), panelpos+Vector(PANEL_W,game->screenH()), buttonpanelcol );

	/* map  */
	mappos= panelpos;
	renderer->renderBox( mappos+Vector(0,PANEL_W), mappos+Vector(PANEL_W,0), 1, 1, 1, 0.7, game->minimap->getTextureID() ); //flip upside down
////	renderer->renderBox( mappos, mappos+Vector(256,256), 1, 1, 1, 0.7, game->minimap->getTextureID() );

	//units on map
	for( it=game->firstUnitID(); !game->iterationEnd(it); game->nextUnitID(it) ) //loop through all units
	{
		Unit *u;
		u=game->getUnit(it);
		
		//calculate point position
		pointpos = u->getUnitPosition() / game->ground->getTerrainWidth();
		pointpos.setY( 1-pointpos.y() ); // flip y axis
		pointpos.scale(PANEL_W);
		pointpos += mappos;
		pointpos.setZ(0);
		
		//select color
		switch( game->getMyPlayer()->getPlayerAllianceMode(u->getPlayer()) ) {
			case PLAYER_SELF: pointcol.set( 0,1,0,1 ); break;
			case PLAYER_ALLY: pointcol.set( 1,1,0,1 ); break;
			case PLAYER_ENEMY: pointcol.set( 1,0,0,1 ); break;
			case PLAYER_NEUTRAL: pointcol.set( 0,0,1,1 ); break;
			default: pointcol.set( .5,.5,.5,1 ); break; //unknown
		}

		// check unit type and render
		if( u->getUnitType()==BUILDING ) {
			pointpos += Vector(1,1); // center point
			renderer->renderPoint( pointpos, pointcol, 2 ); //bigger point
		}
		else
			renderer->renderPoint( pointpos, pointcol, 1 );
	}

	// camera point
/*	pointpos = renderer->getCameraLookAt() / game->ground->getTerrainWidth();
	pointpos.setY( 1-pointpos.y() ); // flip y axis
	pointpos.scale(PANEL_W);
	pointpos += (mappos+Vector(1,1));
	pointpos.setZ(0);
	renderer->renderPoint( pointpos, Color(1,1,1,1), 2 );*/
	
	//camera arrow
	float camangle, campitch;
	renderer->getCameraAngle( camangle, campitch );
	pointpos = renderer->getCameraLookAt() / game->ground->getTerrainWidth();
	pointpos.setY( 1-pointpos.y() ); // flip y axis
	pointpos.scale(PANEL_W);
	pointpos += (mappos+Vector(1,1));
	pointpos.setZ(0);
	Vector pointpos2;//endpoint
	pointpos2.set( 0,10,0 );
	pointpos2.rotateZ( -camangle+M_PI );
	pointpos2+=pointpos;
	renderer->renderInterfaceLine( pointpos, pointpos2, Color( .8,.8,.8,1 ), FUNC_SOLID );
	renderer->renderPoint( pointpos, Color(1,1,1,1), 3 );
	
	sprintf( str, "Executor 0.1, FPS %.1f, Alt %0.0f meters, Map width %.0f m", game->fps, renderer->getCameraPosition().z()-game->ground->getElevation(renderer->getCameraPosition().x(), renderer->getCameraPosition().y()), game->ground->getTerrainWidth() );
	renderer->setFont( game->defaultFont );
	renderer->renderText( 0, 0, str );
//	renderer->setRenderingMode(RENDER_TGL_OBJECT);
}

void GameGUI::renderButtons()
{
	unsigned int i;
	
	buttonbase= panelpos+Vector(8,game->minimap->height()+8);
	
	for( i=0; i<MAX_GUIBUTTONS; i++ )
		if( buttons[i]!=NULL ) buttons[i]->render();
		
	for( i=0; i<actions.size(); i++) //render from last to first in order to keep tooltips on top
	{
		actionbuttons[actions.size()-i-1]->render();
	}
}

void GameGUI::initButtons()
{
	int i;
	
	renderer->loadTexture( "/files/images/executor/move.png" );
	
	buttonbase.set( game->screenW()-PANEL_W+8,PANEL_W+8, 0 );
	
//	buttons[buttcount]=new GUIButton( game, renderer->getTexture( "/files/images/executor/move.png" ), "Move &(&A&l&t&+&M&)", 0, 0, buttonbase, SDLK_m, KMOD_ALT );
//	buttcount++;
	
//	buttons[buttcount]=new GUIButton( game, "&Text button", 0, 32+8, buttonbase, SDLK_t );
//	buttcount++;
	
	for( i=0; i<MAX_GUIBUTTONS; i++ )
		if( buttons[i]!=NULL ) buttons[i]->setAutoRefresh(false); // refresh in updateFrame
}

void GameGUI::refreshGUI()
{
	Vector pos, dest;
	Vector campos, lookat;
	float altitude;
	
	// rotate marker model
	marker_angle+=0.7;
	
	// camera altitude from ground
	lookat = renderer->getCameraLookAt();
	campos = renderer->getCameraPosition();
	altitude = campos.z()-game->ground->getElevation( campos.x(), campos.y() );
	
	// calculate camera movement speed
	float speed = 0.7*altitude+0.3;
	float zoomspeed = 1*altitude+50;
	speed*=(cameraspeed*2.5);
	zoomspeed*=(cameraspeed);

	/// move forward/backward
//	if( game->input.key(SDLK_UP) ) renderer->moveCameraForward( MAINLOOP_PER_FRAME(zoomspeed) );
//	if( game->input.key(SDLK_DOWN) ) renderer->moveCameraForward( MAINLOOP_PER_FRAME(-zoomspeed) );

	/// move up/down
	if(  game->input.key(SDLK_LALT) && game->input.key(SDLK_a) ) renderer->moveCameraXYZ( 0, 0, MAINLOOP_PER_FRAME(zoomspeed) );
	if(  game->input.key(SDLK_LALT) && game->input.key(SDLK_z) ) renderer->moveCameraXYZ( 0, 0, MAINLOOP_PER_FRAME(-zoomspeed) );
	
	//if mouse is in border area, pan screen
//	if( !game->input.mouseButton( M_LEFT ) && !game->input.mouseButton( M_MIDDLE ) ) {
	if( !game->input.mouseButton( M_LEFT ) ) {
		float alt1 = game->ground->getElevation( lookat.x(), lookat.y() );
		
		if( game->input.mouseX()<SCREEN_BORDER_SIZE || game->input.key(SDLK_LEFT) )
			renderer->moveCamera2DRight( MAINLOOP_PER_FRAME(-speed) );
		if( game->input.mouseY()<SCREEN_BORDER_SIZE || game->input.key(SDLK_UP) )
			renderer->moveCamera2DUp( MAINLOOP_PER_FRAME(speed) );
		if( (renderer->getScreenW()-game->input.mouseX())<SCREEN_BORDER_SIZE || game->input.key(SDLK_RIGHT) )
			renderer->moveCamera2DRight( MAINLOOP_PER_FRAME(speed) );
		if( (renderer->getScreenH()-game->input.mouseY())<SCREEN_BORDER_SIZE || game->input.key(SDLK_DOWN) )
			renderer->moveCamera2DUp( MAINLOOP_PER_FRAME(-speed) );
		
		// don't let camera move over map borders
		campos = renderer->getCameraLookAt();
		game->ground->clampToMapArea( campos );
		renderer->setCameraLookAt( campos );
			
		//preserve altitude
		campos = renderer->getCameraLookAt();
		float alt2 = game->ground->getElevation( campos.x(), campos.y() );
		renderer->moveCameraXYZ( 0,0, alt2-alt1 );
	}

	// zoom in by keyboard
	if( game->input.key(SDLK_KP_PLUS) || game->input.key(SDLK_PAGEUP) )
	{
		float zoomspeed=renderer->getCameraLookAtDistance()/100.0+0.5;
		renderer->adjustCameraLookAtDistance( -4*zoomspeed*cameraspeed );
	}
	// zoom out by keyboard
	if( game->input.key(SDLK_KP_MINUS)  || game->input.key(SDLK_INSERT) )
	{
		float zoomspeed=renderer->getCameraLookAtDistance()/100.0+0.5;
		renderer->adjustCameraLookAtDistance( 4*zoomspeed*cameraspeed );
	}

	// limit camera distance
	if( renderer->getCameraLookAtDistance() > 3000.0 ) renderer->setCameraLookAtDistance( 3000 );
	if( renderer->getCameraLookAtDistance() < 5.0 ) renderer->setCameraLookAtDistance( 5 );

	// rotate camera by keyboard
	if( game->input.key(SDLK_DELETE) )
		renderer->rotateCameraAngle( -cameraspeed/30.0, 0 );
	if( game->input.key(SDLK_PAGEDOWN) )
		renderer->rotateCameraAngle( cameraspeed/30.0, 0 );
	if( game->input.key(SDLK_HOME) )
		renderer->rotateCameraAngle( 0, -cameraspeed/30.0 );
	if( game->input.key(SDLK_END) )
		renderer->rotateCameraAngle( 0, cameraspeed/30.0 );
	
	
	// don't let camera go under ground
//	campos = renderer->getCameraPosition();
//	if( campos.z() < game->ground->getElevation( campos.x(), campos.y() )+10 )
//		campos.setZ( game->ground->getElevation( campos.x(), campos.y() )+10 );
	
	// limit max altitude to 5km
//	if( campos.z() > game->ground->getElevation( campos.x(), campos.y() )+5000 )
//		campos.setZ( game->ground->getElevation( campos.x(), campos.y() )+5000 );

//	renderer->setCameraPosition( campos, true );

	// don't let lookat go under ground
	campos = renderer->getCameraLookAt();
	if( campos.z() < game->ground->getElevation( campos.x(), campos.y() )+10 )
		campos.setZ( game->ground->getElevation( campos.x(), campos.y() )+10 );
	
	// limit max altitude to 5km
	if( campos.z() > game->ground->getElevation( campos.x(), campos.y() )+5000 )
		campos.setZ( game->ground->getElevation( campos.x(), campos.y() )+5000 );
		
	renderer->setCameraLookAt( campos );
}

void GameGUI::updateFrame()
{
	int i;
	int mouse_x=game->input.mouseX(), mouse_y=game->input.mouseY();
	Vector dest;
	
	if( game->input.key(SDLK_F8) )
		renderer->setCameraMode( CAMERA_ORBIT );
	if( game->input.key(SDLK_F7) )
		renderer->setCameraMode( CAMERA_FREE );
	
	
	// execute uint orders
	if( game->input.mouseClicked( M_RIGHT ) )
	{
/*			UnitID i;
			for( i=game->firstUnitID(); !game->iterationEnd(i);game->nextUnitID(i) )
			{
				printf( ":p%d id %d\n", i.getPlayernum(), i.getID() );
			}*/
	
		UnitCommand cmd;
		// create new action:
		
		if( !mouse_over_map )
			dest=game->getMouse3DPosition();
		else
			dest=mouseMapToWorld();
			
		cmd.setPosition( dest );
		players[myplayernum]->primaryUnitAction( cmd );
	}

	/// begin selection
	if( game->input.mouseClicked( M_LEFT ) && execute_action.getActionID()==A_NONE ) //if no action execution in progress
	{
		if( mouse_x < buttonbase.x() || mouse_x > buttonbase.x()+PANEL_W )
		{
			select_p1.set( mouse_x, mouse_y );
			selection_in_progress=true;
		}
	}
	if( game->input.mouseClicked( M_LEFT ) && execute_action.getActionID()!=A_NONE ) //select target for action & execute
	{
		if( !mouse_over_map )
			dest=game->getMouse3DPosition();
		else
			dest=mouseMapToWorld();
			
		execute_action.setPosition( dest );
		players[myplayernum]->unitAction( execute_action );
		execute_action.setActionID(A_NONE);
	}

	//pan view using minimap and left button
	if(game->input.mouseButton( M_LEFT ) && mouse_over_map )
	{
		renderer->setCameraLookAt( mouseMapToWorld(), true );
	}
	
	// selection endpoint
	if( game->input.mouseButton( M_LEFT ) )
	{
		select_p2.set( mouse_x, mouse_y  );
	}
	
	//select
	if( game->input.mouseReleased( M_LEFT ) && selection_in_progress )
	{
		players[myplayernum]->selectUnits( select_p1, select_p2 );
		selection_changed=true;
		selection_in_progress=false;
	} else selection_changed=false;
	
	// rotate camera
	if( game->input.mouseButton( M_MIDDLE ) && !game->input.key(SDLK_LALT) && !mouse_over_map)
	{
		game->input.showMouseCursor( false );
		renderer->rotateCameraAngle( cameraspeed*float(-game->input.relativeMouseX())/150.0, cameraspeed*float(game->input.relativeMouseY())/150.0 );
	}

	// zoom camera
	if( game->input.mouseButton( M_MIDDLE ) && game->input.key(SDLK_LALT) )
	{
		float zoomspeed=renderer->getCameraLookAtDistance()/100.0+0.5;
		game->input.showMouseCursor( false );
		renderer->adjustCameraLookAtDistance( float(game->input.relativeMouseY())*zoomspeed );
		
		// limit distance
		if( renderer->getCameraLookAtDistance() > 3000.0 ) renderer->setCameraLookAtDistance( 3000 );
		if( renderer->getCameraLookAtDistance() < 5.0 ) renderer->setCameraLookAtDistance( 5 );
	}
		
	if( !game->input.mouseButton( M_MIDDLE ) )
		game->input.showMouseCursor( true );
		
	//refresh buttons
	for( i=0; i<MAX_GUIBUTTONS; i++ )
		if( buttons[i]!=NULL ) buttons[i]->refresh();
		
	// don't let camera go under ground
	Vector campos = renderer->getCameraPosition();
	if( campos.z() < game->ground->getElevation( campos.x(), campos.y() )+10 )
		campos.setZ( game->ground->getElevation( campos.x(), campos.y() )+10 );
	
	// limit max altitude to 5km
//	if( campos.z() > game->ground->getElevation( campos.x(), campos.y() )+5000 )
//		campos.setZ( game->ground->getElevation( campos.x(), campos.y() )+5000 );

	renderer->setCameraPosition( campos, false );
	
	// check if action buttons pressed
	for( i=0; i<(int)actionbuttons.size(); i++ )
	{
		if( actionbuttons[i]->clicked() )
			if( actions[i].needTarget() ) //if target needed, set execute_action to get target...
				execute_action=actions[i];
			else // ...else execute action now
				players[myplayernum]->unitAction( actions[i] );
	}
	
/*	if( execute_action.getActionID()!=A_NONE )
	{
		if(
	}*/
	
	if( selection_changed )
	{
		gatherActions();
		makeActionButtons();
	}
	
	// handle map actions
 	
 	//if mouse is over minimap:
	if( !game->input.mouseButton( M_ANY ) ) //don't change status if button is pressed
	 	if( mouse_x > panelpos.x() && mouse_x < panelpos.x()+game->minimap->width() &&
			mouse_y > panelpos.y() && mouse_y < panelpos.y()+game->minimap->height() )
			mouse_over_map=true;
		else
			mouse_over_map=false;
}


void GameGUI::gatherActions()
{
	unsigned int i, a, v;
	UnitCommand cmd;
	
	actions.clear();
	
	for( UnitID it=game->firstUnitID(); !game->iterationEnd(it); game->nextUnitID(it) ) //loop through all units
	{
		Unit *u;
		u=game->getUnit(it);
		unsigned int numcmds;
		
		if( u->isSelected() )
			numcmds=u->getNumberOfActions();//get number of actiontypes
		else continue; //not selected, skip
		
		//gather all actions of this unit
		for( a=0; a<numcmds; a++ )
		{
			cmd=u->getActionType(a);
			
			// check if action type already listed in actions-array
			for( v=0; v<actions.size(); v++ )
				if( actions[v].getActionID()==cmd.getActionID() ) break;
			if( v==actions.size() ) // if action not listed, add
				actions.push_back( cmd );
		}
	}
	
	// sort action ID's
	sort( actions.begin(), actions.end() );
}

void GameGUI::makeActionButtons()
{
	unsigned int i;
	int x=0, y=0;
	int id;
	int buttonstage=0;
	
	
	//delete old items
	for( i=0; i<actions.size(); i++ )
	{
//		delete( actionbuttons.pop_back() );
		actionbuttons.pop_back(); // this deletes items??
	}
	
	actionbuttons.clear();// just to be sure...
		
	for( i=0; i<actions.size(); i++ )
	{
		id=actions[i].getActionID();
	
		// put action/build etc. buttons in different rows
		if( x && buttonstage<4096 && id>=4096 ) {
			buttonstage=4096;
			y+=(32+16);
			x=0;
		}
		if( x && buttonstage<2048 && id>=2049 ) {
			buttonstage=2048;
			y+=(32+16);
			x=0;
		}
		if( x && buttonstage<1024 && id>=1024 ) {
			buttonstage=1024;
			y+=(32+16);
			x=0;
		}
	
		actionbuttons.push_back( new GUIButton( game, actions[i].getIcon(), actions[i].getButtonText().c_str(), x, y, buttonbase ) );
		actionbuttons[i]->setID( id );
		
		// calculate position for next button
		x+=32+4;
		if( x > (PANEL_W-32-4-4) )
		{
			 x=0;
			 y+=(32+4);
		}
	}
}

Vector GameGUI::mapToWorld( Vector minimappos )
{
	Vector pos;
	
	minimappos.setY( -minimappos.y()+game->minimap->height() ); //flip y?
		
	//SQUAREMAP
	pos = minimappos/game->minimap->width()*game->ground->getTerrainWidth();

	//limit x,y
	if( pos.x()<0 ) pos.setX(0);
	if( pos.y()<0 ) pos.setY(0);
	if( pos.x()>game->ground->getTerrainWidth() ) pos.setX(game->ground->getTerrainWidth());
	if( pos.y()>game->ground->getTerrainHeight() ) pos.setY(game->ground->getTerrainHeight());
	
	//set altitude
	pos.setZ( game->ground->getElevation( pos ) );
	return pos;
}

Vector GameGUI::mouseMapToWorld()
{
	Vector mappos;
	mappos.set( panelpos.x(), 0 );
	return mapToWorld( game->input.mousePos()-mappos );
}



