/***************************************************************************
                          gamegui.h  -  description
                             -------------------
    begin                : Sun Sep 9 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GAMEGUI_H
#define GAMEGUI_H



#include "game.h"
#include "font.h"
#include "unitcommand.h"
#include "unitid.h"

#include <vector>
using namespace std;	

class Game;


/**Gui for game
  *@author T Kontkanen
  */

class GameGUI {
public:
	/// Constructor
	GameGUI( Game *parent );
	~GameGUI();

	/// Refresh and render current GUI
	void render();
	/// Update things like camera, must be called inside time syncronized mainloop.
	void refreshGUI();
	/// Update thigs like camera, must be called only once per frame.
	void updateFrame();
	
private:
	/// Transform minimap 2d-position to 3d-world position on the ground
	Vector mapToWorld( Vector minimappos );
	/// Returns a 3d-world position where mouse is currently pointing on minimap. Returns (-1,-1,-1) if mouse is outside of minimap.
	Vector mouseMapToWorld();
	void renderHUD();
	void renderButtons();
	void initButtons();
	/// Collect available actions into actions-array.
      void gatherActions();
	/// Make action buttons for actions-array
	void makeActionButtons();
	
	Game *game;
	Renderer *renderer;
	Player **players;
	Input *input; //shortcut to game->input
	
	//button stuff
	GUIButton *buttons[MAX_GUIBUTTONS];
	int buttcount; //count of buttons created
	Vector panelpos;
	Vector buttonbase; //base position
	Color buttonpanelcol;
	
	/// actions
	vector<GUIButton*> actionbuttons;
	vector<UnitCommand> actions;
	UnitCommand execute_action;
//	GUIButton *actionbuttons[MAX_GUIBUTTONS];
//	UnitCommand actions[MAX_GUIBUTTONS]; //holds information about available commands
	
	// selection
	Vector select_p1, select_p2;
	bool selection_changed, selection_in_progress, mouse_over_map;
	
	//camera stuff
	float cameraspeed; // multiplier for camera speed, default 1.0
	tgl_object *marker;
	float marker_angle;
	
	// vars
	int myplayernum;
};

#endif
