/***************************************************************************
                          ground.cpp  -  description
                             -------------------
    begin                : Sat Jul 21 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "ground.h"

using namespace msg;

//Ground::Ground( Renderer *myRenderer ){
Ground::Ground( Game *parentGame ){
	setFuncName( "Ground::Ground" ); // helps debugging

	game=parentGame;
	renderer=game->renderer;
	terrain=NULL;
	
	settings = Demeter::Settings::GetInstance();
	settings->SetScreenWidth( renderer->getScreenW() );
	settings->SetScreenHeight( renderer->getScreenH() );
	terrain=NULL;
	surrounding=NULL;
	terrainrefreshing=true;
	
	//init grid
	groundGrid=NULL;
	grid_w=grid_h=0;
	
	///load settings
	game->config.setGroup( "ground" );
	settings->SetVerbose( game->config.getBool("DemeterVerbose",true) );
	
	exitFunc(); // if error occurs after this func, don't blame this...
}

Ground::~Ground(){
	msgOut( MSG_UNINIT, "Unloading Ground..." );
	setFuncName( "Ground::~Ground" ); // helps debugging

	unloadMap();
	
	exitFunc(); // if error occurs after this func, don't blame this...
	msgOut( MSG_DEBUG_3, "Ground destroyed" );
}

//int Ground::loadMap( const char *fname ) {
int Ground::loadMap( Config &lev ) {
	setFuncName( "Ground::loadMap" ); // helps debugging
	lev.setGroup( "level" );

	string elevf, texf, detailf;
	float vertexspacing, elevscale;
	int i;
	
	
	settings->SetMediaPath( (char*)string(game->dataRoot()+MAPPATH).c_str() );
	elevf=lev.getString("mapElevation");
	texf=lev.getString("mapTexture");
	detailf=lev.getString("mapDetailTex");
	vertexspacing=lev.getFloat("mapVertexSpacing");
	elevscale=lev.getFloat("mapElevScale");
	
	try //to load map
	{
		game->config.setGroup( "ground" );
		
		terrain = new Demeter::Terrain( (char*)elevf.c_str(), (char*)texf.c_str(), (char*)detailf.c_str(),
			vertexspacing, elevscale,
			game->config.getInt( "maxVisibleTriangles", 40000 ),
			game->config.getBool( "useBorders", false ) );
	}
	catch(DemeterException* pEx)
	{
		msgOut( MSG_FATAL, "Unexpected exception while creating terrain (file %s): %s", elevf.c_str(), pEx->GetErrorMessage() );
	}
	catch(...)
	{
		msgOut( MSG_FATAL, "Unexpected exception while creating terrain (file %s). Unknown reason.", elevf.c_str() );
	}
	
	terrain->SetMaximumVisibleBlockSize( game->config.getInt("maxVisibleBlockSize", game->config.getInt("maxVisibleBlockSize",64) ) );
	terrain->SetCommonTextureRepeats(lev.getFloat("mapDetailTexRepeat",30.0f) );
	terrain->SetDetailThreshold(game->config.getFloat("detailThreshold",15)*lev.getFloat("LODScale",1) );

	//allocate ground grid
	grid_w=int(getTerrainWidth()/GROUND_GRID_RESOLUTION)+1; //add one to make sure it's big enough
	grid_h=int(getTerrainHeight()/GROUND_GRID_RESOLUTION)+1;
	if( grid_w%8 ) grid_w_bytes = grid_w/8 +1;
	else grid_w_bytes = grid_w/8;
	
	groundGrid=(unsigned char*)malloc( grid_w_bytes*grid_h ); //allocate
	if( groundGrid==NULL ) msgOut( MSG_FATAL, "Out of memory" );
	bzero( groundGrid, grid_w_bytes*grid_h ); // clear

	exitFunc(); // if error occurs after this func, don't blame this...
	return 0;
}

void Ground::unloadMap(){
	setFuncName( "Ground::unloadMap" ); // helps debugging

	//FIXME if this code is located before "delete terrain", program locks up when exiting. Really, really strange...
	if( surrounding!=NULL ) {
		renderer->freeTGLObject( surrounding );
		surrounding=NULL;
	}
	
	if( terrain !=NULL ) {
		 delete terrain;
		 free( groundGrid );
	}
	terrain=NULL;
	
/*	if( surrounding!=NULL ) {
		renderer->freeTGLObject( surrounding );
		surrounding=NULL;
	}*/
	
		
	exitFunc(); // if error occurs after this func, don't blame this...
}

void Ground::cameraMoved()
{
	setFuncName( "Ground::cameraMoved" ); // helps debugging

	if( terrainrefreshing ) {
		renderer->setRenderingMode( RENDER_TERRAIN ); //set 3d projection
		terrain->ModelViewMatrixChanged();
	}
	
	exitFunc(); // if error occurs after this func, don't blame this...
}

void Ground::renderTerrain()
{
	setFuncName( "Ground::renderTerrain" ); // helps debugging

	renderer->setRenderingMode( RENDER_TERRAIN );
	terrain->Render();
	
	if( surrounding!=NULL ) {
		surrounding->move( terrain->GetWidth()/2.0,terrain->GetHeight()/2.0,surrounding_alt );
		surrounding->scale( surrounding_scale, surrounding_scale, surrounding_scale );
		surrounding->draw();
	}
	
	exitFunc(); // if error occurs after this func, don't blame this...
}

float Ground::getElevation( float x, float y ) const
{
	setFuncName( "Ground::getElevation" ); // helps debugging

	if( terrain == NULL ) return 0;
	
	// clamp
//	if( x<0 ) x=0;
//	if( y<0 ) y=0;
//	if( x>getTerrainWidth()-2) x=getTerrainWidth()-2; // 2 is not enough
//	if( y>getTerrainHeight()-2 ) y=getTerrainHeight()-2;
	
	exitFunc(); // if error occurs after this func, don't blame this...
	return terrain->GetElevation( x, y );
	
}

float Ground::getTerrainWidth() const
{
	if( terrain == NULL ) return 0;
	return terrain->GetWidth();
}

float Ground::getTerrainHeight() const
{
	if( terrain == NULL ) return 0;
	return terrain->GetHeight();
}

void Ground::setBit( unsigned char *buffer, unsigned int bitnum, bool state )
{
	if( bitnum >= 8 ) return; //no more bits in one byte
		
	if( state==true )
		buffer[0]=buffer[0]|(0x1<<bitnum);
	else
		buffer[0]=buffer[0]&(~(0x1<<bitnum));
}

bool Ground::getBit( unsigned char *buffer, unsigned int bitnum ) const
{
	if( buffer[0]&(0x1<<bitnum) )
		return true;
	else
		return false;
}

void Ground::setMapReservationPoint( float x, float y, bool state ) // place a marker to area to prevent other units colliding
{
	if( x < 0 || x>=getTerrainWidth() || y < 0 || y>=getTerrainHeight() ) return; //out of map area

	int getx, gety, bit;
	bit=(int(x)/GROUND_GRID_RESOLUTION)%8;
	getx=(int(x)/8)/GROUND_GRID_RESOLUTION;
	gety=int(y)/GROUND_GRID_RESOLUTION;

	setBit( groundGrid+getx+grid_w_bytes*gety, bit, state );
}

bool Ground::getMapReservationPoint( float x, float y ) const // get state of area reservation
{
	if( x < 0.1 || x>=(getTerrainWidth()-0.1) || y < 0.1 || y>=(getTerrainHeight()-0.1) ) return true; //out of map area

	int getx, gety, bit;
	bit=(int(x)/GROUND_GRID_RESOLUTION)%8;
	getx=(int(x)/8)/GROUND_GRID_RESOLUTION;
	gety=int(y)/GROUND_GRID_RESOLUTION;
	
	return getBit( groundGrid+getx+grid_w_bytes*gety, bit );
}

bool Ground::getMapReservationLine( float x1, float y1, float x2, float y2 ) const // return true if way from (x1,y1) to (x2,y2) is blocked, else return false
{
	int ix, iy, cx, cy, delta_x, delta_y, i, s_delta_x, s_delta_y, ix1, iy1, ix2, iy2;

	if( isSameReservationPoint( x1, y1, x2, y2 ) ) return getMapReservationPoint( x1, y1 ); //no need to draw line
	
	ix1=int(x1); // assign to ints
	iy1=int(y1);
	ix2=int(x2);
	iy2=int(y2);
	
	delta_x = abs( ix1-ix2 );
	delta_y = abs( iy1-iy2 );


	s_delta_x = ix1-ix2; // signed
	s_delta_y = iy1-iy2;

	if( delta_x >= delta_y ) // think how to draw line
	{
		if( ix1>ix2 ) // sort ints
		{
			swapInts( ix1, ix2 );
			swapInts( iy1, iy2 );
		}

		iy = iy1*65536;  // use fixed point to save some fpu cycles
		cy = (s_delta_y*65536/s_delta_x)*GROUND_GRID_RESOLUTION;

		for( i=0; i<delta_x; i+=GROUND_GRID_RESOLUTION ) //interpolate the line
		{
			if( getMapReservationPoint( i+ix1, iy/65536 ) ) return true;
			iy += cy;
		}
	} else
	{
		if( iy1>iy2 ) //sort ints
		{
			swapInts( ix1, ix2 );
			swapInts( iy1, iy2 );
		}

		ix = ix1*65536;
		cx = (s_delta_x*65536/s_delta_y)*GROUND_GRID_RESOLUTION;

		for( i=0; i<delta_y; i+=GROUND_GRID_RESOLUTION ) //interpolate
		{
			if( getMapReservationPoint( ix/65536, i+iy1 ) ) return true;
			ix += cx;
		}
	}
	
	return false; //the way is clear
}


void Ground::setMapReservationLine( float x1, float y1, float x2, float y2, bool state ) // set reservation for line (x1,y2)->(x2,y2)
{
	int ix, iy, cx, cy, delta_x, delta_y, i, s_delta_x, s_delta_y, ix1, iy1, ix2, iy2;

	if( isSameReservationPoint( x1, y1, x2, y2 ) ) setMapReservationPoint( x1, y1, state ); //no need to draw line
	
	ix1=int(x1); // assign to ints
	iy1=int(y1);
	ix2=int(x2);
	iy2=int(y2);
	
	delta_x = abs( ix1-ix2 );
	delta_y = abs( iy1-iy2 );


	s_delta_x = ix1-ix2; // signed
	s_delta_y = iy1-iy2;

	if( delta_x >= delta_y ) // think how to draw line
	{
		if( ix1>ix2 ) // sort ints
		{
			swapInts( ix1, ix2 );
			swapInts( iy1, iy2 );
		}

		iy = iy1*65536;  // use fixed point to save some fpu cycles
		cy = (s_delta_y*65536/s_delta_x)*GROUND_GRID_RESOLUTION;

		for( i=0; i<delta_x; i+=GROUND_GRID_RESOLUTION ) //interpolate the line
		{
			setMapReservationPoint( i+ix1, iy/65536, state );
			iy += cy;
		}
	} else
	{
		if( iy1>iy2 ) //sort ints
		{
			swapInts( ix1, ix2 );
			swapInts( iy1, iy2 );
		}

		ix = ix1*65536;
		cx = (s_delta_x*65536/s_delta_y)*GROUND_GRID_RESOLUTION;

		for( i=0; i<delta_y; i+=GROUND_GRID_RESOLUTION ) //interpolate
		{
			setMapReservationPoint( ix/65536, i+iy1, state );
			ix += cx;
		}
	}
}


bool Ground::isSameReservationPoint( float x, float y, float x2, float y2 ) const
{
	int ix, iy, ix2, iy2;
	
	ix=int(int(x)/int(GROUND_GRID_RESOLUTION)); //ugly typecasting just to be sure that it works...
	ix2=int(int(x2)/int(GROUND_GRID_RESOLUTION));
	iy=int(int(y)/int(GROUND_GRID_RESOLUTION));
	iy2=int(int(y2)/int(GROUND_GRID_RESOLUTION));
	
	if( ix==ix2 && iy==iy2 )
		return true;
	else
		return false;
}

void Ground::toggleTesslation()
{
	if( terrainrefreshing )
		terrainrefreshing=false;
	else
		terrainrefreshing=true;
}

void Ground::clampToMapArea( Vector &pos )
{
	if( pos.x() < 0 ) pos.setX(0);
	if( pos.y() < 0 ) pos.setY(0);
	if( pos.x() >= getTerrainWidth()-0.1 ) pos.setX(getTerrainWidth()-0.1);
	if( pos.y() >= getTerrainHeight()-0.1 ) pos.setY(getTerrainHeight()-0.1 );
}

int Ground::loadSurrounding( string surobjfile, float scale, float altitude )
{
	//unload old surrounding
	if( surrounding!=NULL ) {
		renderer->freeTGLObject( surrounding );
		surrounding=NULL;
	}
	
	if( surobjfile==string("none") ) return 0;
		
	surrounding=renderer->loadTGLObject( surobjfile/*, scale*/ );
	surrounding_scale=scale;
	surrounding_alt=altitude;

	return 0;
}



