/***************************************************************************
                          ground.h  -  description
                             -------------------
    begin                : Sat Jul 21 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GROUND_H
#define GROUND_H

#include "Terrain.h" //demeter header
#include "game.h"
#include "allincludes.h"
#include "renderer.h"

/**Terrain data & renderer
  *@author T Kontkanen
  */
//using namespace Demeter;  //conficts with Vector?

//class Ground : public Renderer {
class Ground  {
public:
	Ground( Game *parentGame );
	~Ground();
	
	/// Load demeter map file.
	int loadMap( Config &lev );
	
/*	int loadMap( const char *fname );
	/// Load demeter map file.
	
	int loadMap( string fname ) { return loadMap( fname.c_str() ); }*/
	/// Load surrounding TGL-object.
	int loadSurrounding( string surobjfile, float scale, float altitude );
	/// Free map data.
	void unloadMap();
	
	//rendering
	/// Call this if modelview matrices have changed
	void cameraMoved();
	/// Render ground
	void renderTerrain();
	/// Toggles ground tesslation, default on. For debugging purposes only.
	void toggleTesslation();
	
	///Returns the elevation (z-coordinate) in real units of the specified terrain vertex. (comment ripped from demeter's reference)
	float getElevation( float x, float y ) const;
	///Returns the elevation (z-coordinate) in real units of the specified terrain vertex. (comment ripped from demeter's reference)
	float getElevation( Vector pos2d ) const { return getElevation( pos2d.x(), pos2d.y() ); }
	///Returns the width of the terrain in real units (this is the length of the terrain along the world's x-axis.) (comment ripped from demeter's reference)
	float getTerrainWidth() const;
	///Returns the height of the terrain in real units (this is the length of the terrain along the world's z-axis.) (comment ripped from demeter's reference)
	float getTerrainHeight() const;
	
	/// Limits position on ground area. Modifies only X and Y values of pos.
      void clampToMapArea( Vector &pos );
	
	/// Place a marker to area to prevent other units colliding	
	void setMapReservationPoint( float x, float y, bool state );
	/// Get state of area reservation
	bool getMapReservationPoint( float x, float y ) const;
	/// Return true if way from (x1,y1) to (x2,y2) is blocked, else return false
	bool getMapReservationLine( float x1, float y1, float x2, float y2 ) const;
	/// Set reservation for line (x1,y2)->(x2,y2)
	void setMapReservationLine( float x1, float y1, float x2, float y2, bool state );
	/// Return true if (x,y) is in same reservation area with (x2,y2)
	bool isSameReservationPoint( float x, float y, float x2, float y2 ) const;
	
private:
	void setBit( unsigned char *buffer, unsigned int bitnum, bool state );
	bool getBit( unsigned char *buffer, unsigned int bitnum ) const;
	/// Swap values of two integers
	void swapInts( int &i1, int &i2 ) const {
		int i=i1;
		i1=i2;
		i2=i;
	}
	
	Game *game;
	Renderer *renderer;
	
	/// terrain data
	Demeter::Settings *settings; //demeter settings
	Demeter::Terrain *terrain; //demeter terrain
	unsigned char *groundGrid; // reservation map=1-bit grid that determines where units are able to move etc. resolution can be changed in gamedefs.h
	int grid_h, grid_w, grid_w_bytes; //allocated grid dimensions
	float sealevel;
	bool terrainrefreshing; // if true, tesslate ground
	tgl_object *surrounding;
	float surrounding_scale, surrounding_alt; //altitude
};

#endif
