/***************************************************************************
                          groundmovement.cpp  -  description
                             -------------------
    begin                : Sat Jul 28 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "groundmovement.h"

GroundMovement::GroundMovement( Game *pgame )
{
	mGame=pgame;
	mGround=mGame->ground;
	mRenderer=mGame->renderer;

	topspeed=1; turnrate=1; acceleration=1; //default values, should be overwritten by unit
	turnrange_far=4;  //must be at least sqrt(2)*grid resolution
	turnrange_near=3;  //must be at least sqrt(2)*grid resolution
	radarresolution=0.1; //radians, needed in dodging other objects
	angle=pitch=tilt=velocity=0;
	positionTolerance=1; //meters (radius)
	position.set( 50, 50, mGround->getElevation( 50, 50 ) );
	destPosition.set( 50, 50, mGround->getElevation( 50, 50 ) );
	stopped=true;
	originheight=0;
}

GroundMovement::~GroundMovement()
{
	mGround->setMapReservationPoint( position.x(), position.y(), false ); // unit dead, clear reservation area
}

void GroundMovement::moveStep() // make a move (one call per frame)
{
	float diffangle, diffangle2, diffangle3, distance, deacceltime;
	Vector destangle, myangle;
	destangle = destPosition-position;
	destangle.setZ(0); //not needed
	distance = destangle.length(); //distance to destination (m)

/*	if( stopped ) { //just decrease speed
		velocity -= PER_FRAME(acceleration);
		// clamp speeds
		if( velocity < 0 ) velocity=0;
		return;
	}*/
	
	if( distance < positionTolerance || stopped ) //if near enough skip turning and acceleration or stop command recieved, slow down
	{
		velocity -= PER_FRAME(acceleration);
		// clamp speeds
		if( velocity < 0 ) velocity=0;
	} else
	{
		//calculate velocity:
		deacceltime = velocity/acceleration; // how long it takes to stop unit
		if( distance < (0.5*acceleration*deacceltime*deacceltime) ) //if distance < (1/2)at start braking
			velocity -= PER_FRAME(acceleration);
		else //accelerate
			velocity += PER_FRAME(acceleration);
		// clamp speeds
		if( velocity < 0 ) velocity=0;
		if( velocity > topspeed ) velocity=topspeed;

		//turn if necessary:
		myangle.set( 1,0,0 );
		myangle.rotateZ( angle );
		
		if( isSomethingOnMyWay(myangle) == false )
		{
			diffangle=myangle.angle( destangle ); // angle before rotation
			// find out the shortest direction to turn
			angle+=PER_FRAME(turnrate);
			myangle.rotateZ( PER_FRAME(turnrate) );
			diffangle2=myangle.angle( destangle ); //angle after first rotation

			if( diffangle2 > diffangle ) // if we are turning to wrong direction
			{
				angle -= 2*PER_FRAME(turnrate); // undo first turn and turn to another direction
				myangle.rotateZ( -2*PER_FRAME(turnrate) );
				diffangle3=myangle.angle( destangle ); //angle after last rotation
				if( diffangle3 > diffangle ) // if we turned too much, cancel all turning
					angle += PER_FRAME(turnrate);
			}
		} else //we have to dodge blocking object
		{
			float i=1, add/*, oldangle=angle*/;
			Vector newangle;
			
			for( i=0; i<M_PI; i+=radarresolution ) // find the best way to turn
			{
				add=i;
				newangle.set( 1,0,0 );
				newangle.rotateZ( angle+add ); //try turning left
				if( isSomethingOnMyWay(newangle) == false )
				{
					angle+=PER_FRAME(turnrate);
					break;
				}
				newangle.rotateZ( -2*add ); // try turning right. -2*i to undo previous rotation
				if( isSomethingOnMyWay(newangle) == false )
				{
					angle-=PER_FRAME(turnrate);
					break;
				}
			}
//			if( angle==oldangle ) angle=
		}
	}
	// now move towards
	if( velocity != 0 ) //if moving
	{
		myangle.set( 1,0,0 ); //reset
		myangle.rotateZ( angle );
		myangle.scale( PER_FRAME(velocity) );
		
		//set new reservation if we are crossing ground grid borders
		if( !mGround->isSameReservationPoint( position.x(), position.y(), position.x()+myangle.x(), position.y()+myangle.y() ) )
		{
			mGround->setMapReservationPoint( position.x(), position.y(), false ); // unit is moving out of current point, clear area
		
			if( mGround->getMapReservationPoint( position.x()+myangle.x(), position.y()+myangle.y() ) ) // we must dodge if we are going to collide, simply clip positions
				if( !mGround->getMapReservationPoint( position.x(), position.y()+myangle.y() ) ) //test if clipping x-movement avoids collision
					myangle.setX( 0 ); //clip it
				else if ( !mGround->getMapReservationPoint( position.x()+myangle.x(), position.y() ) ) //test if clipping y-movement avoids collision
					myangle.setY( 0 ); //clip it
				else { // no way to go
					myangle.setX( 0 ); //clip it
					myangle.setY( 0 ); //clip it
				}
			
			mGround->setMapReservationPoint( position.x()+myangle.x(), position.y()+myangle.y(), true ); // reserve next point
		}
		
		//  update position
		position += myangle;
		position.setZ( mGround->getElevation( position.x(), position.y() ) +originheight);
	}

//	printf( "m %f %f %f\n", myangle.x(),myangle.y(),myangle.z() );
//	printf( "d %f %f %f\n", destangle.x(),destangle.y(),destangle.z() );
}

bool GroundMovement::isSomethingOnMyWay( Vector direction )
{
	Vector far;
	bool state;
	
	direction.scale( turnrange_far );
	far=position+direction;
	
	mGround->setMapReservationPoint( position.x(), position.y(), false ); //avoid seeing own point by clearin it temporary
	state = mGround->getMapReservationLine( position.x(), position.y(), far.x(), far.y() );
	mGround->setMapReservationPoint( position.x(), position.y(), true ); //restore curren point
//	printf( "    wall %f %f->%f %f \n", position.x(), position.y(), look.x(), look.y() );
	return state;
}

void GroundMovement::stopMoving()
{
	stopped=true;
}


