/***************************************************************************
                          groundmovement.h  -  description
                             -------------------
    begin                : Sat Jul 28 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GROUNDMOVEMENT_H
#define GROUNDMOVEMENT_H

#include "renderer.h"
#include "ground.h"
#include "game.h"
#include "allincludes.h"
#include "gamedefs.h"
#include "vector.h"

/**Movement for ground units
  *@author T Kontkanen
  */


class GroundMovement {
public: 
//	GroundMovement( Renderer *prenderer, Ground *pground );
	GroundMovement( Game *pgame );
	~GroundMovement();

	Vector getPosition() { return position; }
	float getSpeed() { return velocity; }
	float getAngle() { return angle; }
	float getPitch() { return pitch; }
	float getTilt() { return tilt; }

	/// Set destination where to go to.
	void moveSetDestination( Vector destpos ) {
		stopped=false;
		destPosition=destpos;
	}
	/// Get destination where we are going.
	Vector moveGetDestination() { return destPosition; }
	/// Make a move (one call per frame).
	void moveStep();
	/// Set position.
	void setPosition( Vector &pos ) {
		position=pos;
		position.setZ(mGround->getElevation( pos.x(), pos.y() )+originheight );//set elevation
		moveSetDestination( pos );
	}
	/// Halt movement, clears destination.
	void stopMoving();
	
	void setTurnRate( float rate ) { turnrate=rate; }
	void setTopSpeed( float speed ) { topspeed=speed; }
	void setAcceleration( float a ) { acceleration=a; }
	void setOriginHeight( float z ){ originheight=z; }
protected:
	float velocity; /* m/s */
	bool stopped;
	float originheight;
	
private:
	// own data
	bool isSomethingOnMyWay( Vector direction );
	
	Vector position, destPosition;
	float angle, pitch, tilt; //angle=z-axis rotation, pitch=x-axis and tilt=y-axis
	float turnrate /* rad/s */, acceleration /* m/s */, topspeed /* m/s */;
	float positionTolerance; // destination position tolerance in meters (radius)
	float turnrange_near, turnrange_far, radarresolution; // distance when unit starts turnign if wall or other unit is detected ahead
	
	//needed information, don't use same names with Unit to avoid conficts (may be unnecessary?)
	Renderer *mRenderer;
	Ground *mGround;
	Game *mGame;
};

#endif
