/***************************************************************************
                          guibutton.cpp  -  description
                             -------------------
    begin                : Fri Sep 7 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "guibutton.h"

GUIButton::GUIButton( Game *parentgame, Texture *icon, const char *tooltip, int x, int y, Vector &baseposition, SDLKey hotkey, int modifier  )
{
	pos.set( x, y, 0 );
	basepos=&baseposition;
	tip=tooltip;
	game=parentgame;
	font=parentgame->tooltipFont;
	ismouseover=0;
	buttonid=-1;
	
	if( icon==NULL )
		tex=game->renderer->getTexture( game->dataRoot()+ICONPATH+"noaction.png" );
	else
		tex=icon;
	size.set( tex->width(), tex->height(), 0 );

		
	//hotkey
	hotk=hotkey;
	modk=modifier;
	
	//colors
	normalcolor[0]=normalcolor[1]=normalcolor[2]=1.0;
	normalcolor[3]=0.7;
	
	hovercolor[0]=1.0;
	hovercolor[1]=1.0;
	hovercolor[2]=1.0;
	hovercolor[3]=1.0;
	
	activecolor[0]=1.0;
	activecolor[1]=1.0;
	activecolor[2]=0.4;
	activecolor[3]=1.0;
	
	buttontype=0;
	buttonCallback=NULL;
	click=false;
	autorefresh=true;
}

GUIButton::GUIButton( Game *parentgame, const char *text, int x, int y, Vector &baseposition, SDLKey hotkey, int modifier )
{
	game=parentgame;
	
	font=game->buttonFont;
	game->renderer->setFont( font );
	size.set( game->renderer->textLength(text), font->getLineHeight(), 0 );
	buttontext=text;
	pos.set( x, y, 0 );
	basepos=&baseposition;
	ismouseover=0;
	buttonid=-1;
	
	//hotkey
	hotk=hotkey;
	modk=modifier;
	
	//colors
	normalcolor[0]=normalcolor[1]=normalcolor[2]=1.0;
	normalcolor[3]=0.7;
	
	hovercolor[0]=1.0;
	hovercolor[1]=1.0;
	hovercolor[2]=1.0;
	hovercolor[3]=1.0;
	
	activecolor[0]=1.0;
	activecolor[1]=1.0;
	activecolor[2]=0.4;
	activecolor[3]=1.0;
	
	buttontype=1;
	buttonCallback=NULL;
	click=false;
	autorefresh=true;
}

GUIButton::~GUIButton()
{

}

void GUIButton::render()
{
	Vector shift;
	shift.set(0,0,0);
	BFont *myfont=new BFont( *font ); // make a copy
	
	/// refresh if auto mode enabled
	if( autorefresh ) refresh();
	
	//colors etc
	switch( ismouseover )
	{
		case 0: memcpy( currentcolor, normalcolor, sizeof(float)*4 ); break;
		case 1: memcpy( currentcolor, hovercolor, sizeof(float)*4 ); break;
		case 2:
			memcpy( currentcolor, activecolor, sizeof(float)*4 );
			shift.set(1,1,0);
			break;
		default: memcpy( currentcolor, normalcolor, sizeof(float)*4 ); break;
	}

      if( buttontype==0 ) // icon
      {
		Vector s=size*0.75;
	
		game->renderer->renderBox( pos+shift+*basepos, pos+size+shift+*basepos, currentcolor[0], currentcolor[1], currentcolor[2], currentcolor[3], tex->getTextureID() );
		game->renderer->setFont( font );
		if( ismouseover==1 ) game->renderer->renderText( (pos+s+*basepos).x(), (pos+s+*basepos).y(), tip ); ///tooltip
	} else //text
	{
		myfont->setRGBA( currentcolor[0],currentcolor[1],currentcolor[2],currentcolor[3] );
		game->renderer->setFont( myfont );
		game->renderer->renderText( pos.x()+shift.x()+basepos->x(), pos.y()+shift.y()+basepos->y(), buttontext );
	}
}

void GUIButton::refresh()
{
	Vector m=game->input.mousePos();
	bool modok=false;

	//clear	
	click=false;
	ismouseover=0;
	
 	if( (pos+*basepos) <= m && (pos+size+*basepos) >= m && game->input.isMouseVisible() )
 	{
		if( game->input.mouseButton( M_LEFT ) ) ismouseover=2;
		else ismouseover=1;
	}
	
	//if modifier status correct
	if( modk==KMOD_NONE && game->input.keyModifiers()==KMOD_NONE )
		modok=true;
	if( modk!=KMOD_NONE && (modk&game->input.keyModifiers()) )
		modok=true;
	
	//mouse click
	if( ismouseover && game->input.mouseReleased( M_LEFT ) )
		click=true;
	//hotkey
	if( game->input.key( hotk )  && modok )
		ismouseover=2; // button pressed
	if( game->input.keyClicked( hotk ) && modok )
		click=true;
	
	/// call a callback
	if( click==true && buttonCallback!=NULL ) buttonCallback();
}
