/***************************************************************************
                          guibutton.h  -  description
                             -------------------
    begin                : Fri Sep 7 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef GUIBUTTON_H
#define GUIBUTTON_H

#include "game.h"
#include "font.h"

class Game;

/**A simple push button with icon and tooltip
  *@author T Kontkanen
  */

class GUIButton {
public:
	/// Create new button with icon and tooltip at specific position. Render() adds baseposition to x,y and it can be changed at any time. Optionally hotkey and key modifier (KMOD_ALT, KMOD_CTRL etc) can be set.
	GUIButton( Game *parentgame, Texture *icon, const char *tooltip, int x, int y, Vector &baseposition, SDLKey hotkey=SDLK_UNKNOWN, int modifier=KMOD_NONE );
	/// Create new button with text at specific position. Other options are same with icon button.
	GUIButton( Game *parentgame, const char *text, int x, int y, Vector &baseposition, SDLKey hotkey=SDLK_UNKNOWN, int modifier=KMOD_NONE );
	/// Destructor
	~GUIButton();
	
	/// Set autorefresh mode, If enable is true refresh() will be called in render(), otherwise refresh() must be called separately. By default autorefresh is enabled.
	void setAutoRefresh( bool enable ) { autorefresh=enable; }
	/// Set a callback fuction for button press. Callback can't be a member of class, must be C-function like: void test()
	void setCallback( void (*callback)() ) { buttonCallback=callback; }
	
	/// Set button identifier
	void setID( int id ) { buttonid=id; }
	/// Returns button identifier number
	int getID() { return buttonid; }
	
	/// Return true if button is clicked. Cliked status will be cleared in next refresh()
	bool clicked() { return click; }
	/// Render a button
	void render();
	/// Check input and update state, call this every frame when button is visible
	void refresh();
	/// Set tooltip font
	void setFont( BFont *tipfont ) { font=tipfont; }
private:
	int ismouseover; //0=no mouse focus, 1=mouse hovering, 2=mouse howering & pressed
	Texture *tex;
	const char *tip, *buttontext;
	Vector pos, size, *basepos;
	Game *game;
	BFont *font;
	int buttontype; //0=icon+tip, 1=text only
	float normalcolor[4], hovercolor[4], activecolor[4], currentcolor[4];
	
	/// Call this function when pressed
	void (*buttonCallback)();
	bool click, autorefresh;
	SDLKey hotk;
	int modk;
	int buttonid;
};

#endif
