/***************************************************************************
                          input.cpp  -  description
                             -------------------
    begin                : Tue Jul 24 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "input.h"

Input::Input(){
	mouse_visible=true; //system default
	mouse_sys_cursor=false; //use custom cursor
	mouse_x=0;
	mouse_y=0;
	mouse_buttons[0]=mouse_buttons[1]=mouse_buttons[2]=false;
	time_now=time_prev=0;
	renderer=NULL;
}

Input::~Input(){
}

void Input::setRenderer( Renderer *prenderer )
{
	renderer=prenderer;
}

int Input::pollEvents()
{
	int count=0, i;
	
	//get time
	time_prev=time_now;
	time_now=SDL_GetTicks();
//	if (!diffTime() ) SDL_Delay(10);
	
	rel_mouse_x=rel_mouse_y=0;
	
	for( i=0; i<512; i++ ) key_clicks[i]=false;
	
	for( i=0; i<3; i++ )
	{
		mouse_clicked[i]=false; //reset click status
		mouse_released[i]=false; //reset click status
	}
	
	while ( SDL_PollEvent ( &event ) )
	{
		handleEvent( &event );
		count++;
	}
	
	return count;
}

int Input::mouseX()
{
	return mouse_x;
}

int Input::mouseY()
{
	return mouse_y;
}

/// Return mouse position as Vector.
Vector Input::mousePos()
{
	Vector pos;
	pos.set( mouse_x, mouse_y, 0 );
	return pos;
}


int Input::relativeMouseX()
{
	return rel_mouse_x;
}

int Input::relativeMouseY()
{
	return rel_mouse_y;
}

bool Input::mouseButton( int button )
{
	int i;
	
	if( button==M_ANY ) // M_ANY
		for( i=0;i<3;i++ )
			if( mouse_buttons[i] ) return true;

	if( button < 0 || button >= 3 ) return false;
	return mouse_buttons[button];
}

bool Input::mouseClicked( int button )
{
	if( button < 0 || button >= 3 ) return false;
	return mouse_clicked[button];
}

bool Input::mouseReleased( int button )
{
	if( button < 0 || button >= 3 ) return false;
	return mouse_released[button];
}

bool Input::key( int keycode )
{
	if( keycode < 0 || keycode >= 512 ) return false;
	return keys[keycode];
}
	
bool Input::keyClicked( int keycode )
{
	if( keycode < 0 || keycode >= 512 ) return false;
	return key_clicks[keycode];
}

void Input::handleEvent ( SDL_Event *event )
{
	switch ( event->type )
	{
		case SDL_QUIT:
			break;

		case SDL_KEYDOWN:
			keys[event->key.keysym.sym] = true;
			key_clicks[event->key.keysym.sym] = true;
			modifiers=event->key.keysym.mod;
			break;

		case SDL_KEYUP:
			keys[event->key.keysym.sym] = false;
			modifiers=event->key.keysym.mod;
			break;

		case SDL_MOUSEMOTION:
			mouse_x = event->motion.x;
			mouse_y = event->motion.y;
			rel_mouse_x = event->motion.xrel;
			rel_mouse_y = event->motion.yrel;
			break;

		case SDL_MOUSEBUTTONDOWN:
			mouse_buttons[event->button.button-1] = true;
			mouse_clicked[event->button.button-1] = true;
			break;

		case SDL_MOUSEBUTTONUP:
			mouse_buttons[event->button.button-1] = false;
			mouse_released[event->button.button-1] = true;
			break;

		case SDL_VIDEORESIZE:
			 /* handle resize event */
			if( renderer!=NULL ) renderer->setVideoMode( event->resize.w, event->resize.h, 0, renderer->getFullScreen() );
			
		case SDL_ACTIVEEVENT:
			if( !event->active.gain )
				if( event->active.state & SDL_APPMOUSEFOCUS )
				{
					mouse_buttons[0]=mouse_buttons[1]=mouse_buttons[2]=false;
				}
			break;
	}
}

void Input::setInputGrapping( bool onoff )
{
	if( onoff==true )
		SDL_WM_GrabInput(SDL_GRAB_ON);
	else
		SDL_WM_GrabInput(SDL_GRAB_OFF);
}

void Input::showMouseCursor( bool onoff )
{
	mouse_visible=onoff;
	if( mouse_sys_cursor )
	{
		if( onoff==true )
			SDL_ShowCursor( 1 );
		else
			SDL_ShowCursor( 0 );
	} else
		SDL_ShowCursor( 0 );
}

void Input::renderMouseCursor( void )
{
	Vector pos;
	pos.set( mouseX(), mouseY(), 0.5 );
	if( renderer!=NULL && isMouseVisible() && !mouse_sys_cursor ) renderer->renderMouseCursor( pos );
}

int Input::timerTicks()
{
	return time_now;
}

int Input::diffTime()
{
	return (time_now-time_prev);
}

int Input::frameDelay()
{
	while( (SDL_GetTicks()-time_now)<10 )
	{
		SDL_Delay( 10 );
	}
	return 0;
}
