/***************************************************************************
                          input.h  -  mouse, keyboard, joystick, timer etc. input methods
                             -------------------
    begin                : Tue Jul 24 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef INPUT_H
#define INPUT_H

#include "allincludes.h"
#include "gamedefs.h"
#include "renderer.h"

/**Basic keyboard/mouse/joystic input methods
  *@author T Kontkanen
  */

class Input {
public: 
	Input();
	~Input();
	/// Supply a renderer pointer to be able to resize window
	void setRenderer( Renderer *prenderer );
	int pollEvents();	
	int mouseX();
	int mouseY();
	/// Return mouse position as Vector.
	Vector mousePos();
	int relativeMouseX();
	int relativeMouseY();
	/// Return elapsed time in ms since app start
	int timerTicks();
	/// Return time difference in ms between two pollEvents() calls
	int diffTime();
	/// Wait for new input events, in linux timers etc gets jerky if system load is too high so delays is needed :-(
	int frameDelay();
	bool mouseButton( int button );
	bool mouseClicked( int button );
	bool mouseReleased( int button );
	bool key( int keycode );
	/// Return currently pressed key modifiers (e.g. KMOD_CTRL, KMOD_ALT, KMOD_SHIFT etc.).
	SDLMod keyModifiers() { return modifiers; }
	/// Return true if key is clicked between two pollEvents. This doesn't repeat key press.
	bool keyClicked( int keycode );
	void setInputGrapping( bool onoff );
	void showMouseCursor( bool onoff );
	bool isMouseVisible( void ) { return mouse_visible; }
	void renderMouseCursor( void );
	/// Choose between system or software mouse cursor.
	void setSysCursor( bool onoff ) { mouse_sys_cursor=onoff; }
private:
	bool mouse_visible, mouse_sys_cursor;
	void handleEvent ( SDL_Event *event );
	SDL_Event event;
	bool keys[512], key_clicks[512];
	SDLMod modifiers;
	int mouse_x, mouse_y, rel_mouse_x, rel_mouse_y;
	int time_now, time_prev;
	bool mouse_buttons[3], mouse_clicked[3], mouse_released[3]; //buttons state, buttons clicked before poll
	Renderer *renderer;
};

#endif
