/***************************************************************************
                          msgout.cpp  -  description
                             -------------------
    begin                : Fri Aug 3 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "msgout.h"

namespace msg {

FILE *msg_output=stdout;
int msg_debuglevel=5;
const char *msg_lastfunc="unknown";
int msg_last_glerror=GL_NO_ERROR;
bool msg_colors=false;
bool msg_gl_ready=false, msg_func_exit=true;;

void setDebugLevel( int level )
{
	if( level < 0 ) level=0;
	if( level > 5 ) level=5;
	msg_debuglevel=level;
}

////colors:
////ESC[30m black
////ESC[31m dred
////ESC[32m dgreen
////ESC[33m dbrown
////ESC[34m dblue
////ESC[35m dmagenta
////ESC[36m dcyan
////ESC[37m gray
////ESC[90m dgray
////ESC[91m red
////ESC[92m green
////ESC[93m yellow
////ESC[94m blue
////ESC[95m magenta
////ESC[96m cyan
////ESC[97m white

void msgSetColor( int color )
{
	if( msg_colors ) {
		fprintf( msg_output, "\e[%dm", color );
		fflush( msg_output );
	}
}

void msgOut( enum MsgType messagetype, const char *format, ... ) //func from CrstalSpace
{
	char buf[1024];
	va_list arg;

	va_start (arg, format);
	vsprintf (buf, format, arg);
	va_end (arg);

	switch (messagetype)
	{
		case MSG_INTERNAL: //errors
		case MSG_FATAL:
			msgSetColor( 91 );
			fprintf( msg_output, "Fatal/internal Error: %s\n", buf );
			exit( 0 );
			break;

		case MSG_WARN:
			msgSetColor( 91 );
			fprintf( msg_output, "Warning: %s\n", buf );
			break;

		case MSG_INIT:
			msgSetColor( 96 );
			fprintf( msg_output, "Init: %s\n", buf );
			break;

		case MSG_UNINIT:
			msgSetColor( 96 );
			fprintf( msg_output, "UnInit: %s\n", buf );
			break;

		case MSG_DEBUG_1:
			msgSetColor( 32 );
			if( msg_debuglevel >= 1 )
				fprintf( msg_output, "Debug(1): %s\n", buf );
			break;

		case MSG_DEBUG_2:
			msgSetColor( 32 );
			if( msg_debuglevel >= 2 )
				fprintf( msg_output, "Debug(2): %s\n", buf );
			break;

		case MSG_DEBUG_3:
			msgSetColor( 32 );
			if( msg_debuglevel >= 3 )
				fprintf( msg_output, "Debug(3): %s\n", buf );
			break;

		case MSG_DEBUG_4:
			msgSetColor( 32 );
			if( msg_debuglevel >= 4 )
				fprintf( msg_output, "Debug(4): %s\n", buf );
			break;
			
		case MSG_DEBUG_5:
			msgSetColor( 32 );
			if( msg_debuglevel >= 5 )
				fprintf( msg_output, "Debug(5): %s\n", buf );
			break;
			
		case MSG_INFO:
			msgSetColor( 93 );
			fprintf( msg_output, "Info: %s\n", buf );
			break;
		
		case MSG_FAILURE:
			msgSetColor( 91 );
			fprintf( msg_output, "Failure: %s\n", buf );
			break;
			
		case MSG_ERROR:
			msgSetColor( 91 );
			fprintf( msg_output, "Error: %s\n", buf );
			break;
						
		case MSG_NOTE:
			msgSetColor( 93 );
			fprintf( msg_output, "Note: %s\n", buf );
			break;
		default: break;
	}

	msgSetColor( 37 ); //set default color
}

void setFuncName( const char *name )
{
	//if error detected, dont overwrite this name and error
	if( !msg_last_glerror )
	{
		msg_func_exit=false;
		
		if( msg_gl_ready )
			msg_last_glerror=glGetError();

		msg_lastfunc=name;
	}
}

void setColoring( bool enable )
{
	msg_colors=enable;
}


void exitFunc()
{
//	SDL_Delay(10);
	msg_func_exit=true;
}

void glReady( bool enabled )
{
	msg_gl_ready=enabled;
}

const char* getLastFuncName()
{
	return msg_lastfunc;
}

int getLastGlError()
{
	int tmp=msg_last_glerror;
	msg_last_glerror=0;
	return tmp;
}

}

