/***************************************************************************
                          msgout.h  -  Basic output system for debuggin and other messages
                             -------------------
    begin                : Fri Aug 3 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef MSGOUT_H
#define MSGOUT_H

#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <GL/gl.h>

namespace msg {

extern FILE *msg_output;
extern int msg_debuglevel;
extern bool msg_colors;
extern const char *msg_lastfunc;
extern int msg_last_gl_error;
extern bool msg_gl_ready, msg_func_exit;

enum MsgType { MSG_INTERNAL, MSG_FATAL, MSG_FAILURE, MSG_WARN, MSG_INIT, MSG_UNINIT, MSG_DEBUG_1, MSG_DEBUG_2, MSG_DEBUG_3, MSG_DEBUG_4, MSG_DEBUG_5, MSG_INFO, MSG_NOTE, MSG_ERROR };

/// Level 0-5 determines how much to output debugging information, 0 disables debug output. Level 1 is for initialization and 5 for huge amounts of continuous output
extern void setDebugLevel( int level );
/// Set output coloring on and off. Default off.
extern void setColoring( bool enable );
/// Output messages to stdout, log or screen. MSG_INTERNA and MSG_FATAL aborts execution, others just prints output.
extern void msgOut( enum MsgType messagetype, const char *format, ... ); //func from CrstalSpace
/// Set current function/method name to help debugging. Every func/method that does something with GL or SDL should use this.
extern void setFuncName( const char *name );
/// Can be called at end of function if setFuncName is called at begin of function
void exitFunc();
/// Returns name of last called function/method
extern const char* getLastFuncName();
/// Returns last occurred GL error number or zero if no errors. getLastGlError returns error only once.
extern int getLastGlError();
/// Renderer should call glReady(true) when OpenGL is functional and glReady(false) befor uninit.
extern void glReady( bool enabled );

} //EO namespace

#endif
