/***************************************************************************
                          player.cpp  -  description
                             -------------------
    begin                : Wed Jul 25 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "player.h"

#include "renderer.h"
#include "ground.h"
#include "game.h"
#include "unit.h"
#include "unitmarine.h"
#include "unitdestroyer.h"
#include "unitcommandcenter.h"


//Player::Player( Renderer *prenderer, Ground *pground, Game *pgame, int myplayernum )
Player::Player( Game *pgame, int myplayernum )
{
	int i;

	//init players
//	for( i=0; i<MAX_PLAYERS; i++ )
//		players[i]=NULL;
//	players=pplayers;
//	player[myplayernum]=this;
		
	game=pgame;
	playerNum=myplayernum;
	ground=game->ground;
	renderer=game->renderer;
	
	for( i=0; i<MAX_PLAYERS; i++ )
		setPlayerAllianceMode( i, PLAYER_ENEMY );
	setPlayerAllianceMode( playerNum, PLAYER_SELF );
	
	//init units
	for(i=0; i<MAX_UNITS;i++) units[i]=NULL;
	firstunit=lastunit=-1;
	unitcount=0;
}

void Player::createInitialUnits()
{
	int i;
	Unit *u;
	u=new UnitCommandCenter( game );
	insertNewUnit( u );
	u->setUnitPosition( Vector(140+500*playerNum,140,0) );
	u->setUnitSide( playerNum );

	
	//create fisrt units
	for(i=0; i<150; i++ )
	{
		u=new UnitMarine( game );
		insertNewUnit( u );
		u->setUnitPosition( Vector(10+500*playerNum,5*i+10) );
		u->setUnitSide( playerNum );
	}
	
	for(i=0; i<5; i++ )
	{
		insertNewUnit( u=new UnitDestroyer( game ) );
		u->setUnitPosition( Vector(50+500*playerNum,100*i+10) );
		u->setUnitSide( playerNum );
	}
}

Player::~Player()
{
	int i;
	
	for(i=0; i<MAX_UNITS;i++) if( units[i]!=NULL ) delete units[i];
}


int Player::findFreeUnit()
{
	int i;
	
	for(i=0; i<MAX_UNITS; i++ )
		if( units[i]==NULL ) return i;
	
	//no free units
	msg::msgOut( msg::MSG_WARN, "Unit limit exceeded, can't create new unit" );
	return -1;
}

void Player::renderUnits()
{
	int i;
	for(i=0; i<MAX_UNITS; i++ )
		if( units[i]!=NULL )
		{
			units[i]->render();
		}
		
	removeDeadUnits();
}

//void Player::unitAction( int actionnum, Vector destpos ) //execute action for selected units
void Player::unitAction( UnitCommand action ) //execute action for selected units
{
	int i;
	for(i=0; i<MAX_UNITS; i++ )
		if( units[i]!=NULL && units[i]->isSelected()==true )
		{
			units[i]->action( action );
		}
}

//void Player::primaryUnitAction( Vector destpos ) //execute primary action for selected units
void Player::primaryUnitAction( UnitCommand target ) //execute primary action for selected units
{
	int i;
	for(i=0; i<MAX_UNITS; i++ )
		if( units[i]!=NULL && units[i]->isSelected()==true )
		{
			units[i]->primaryAction( target );
		}
}

void Player::unitStep() //execute units's step() method
{
	int i;
	for(i=0; i<MAX_UNITS; i++ )
		if( units[i]!=NULL )
		{
			units[i]->step();
		}
}

void Player::selectUnits( Vector &sel1, Vector &sel2 )
{
	int i;
	renderer->setRenderingMode( RENDER_TGL_OBJECT ); //set 3d projection
	
	for(i=0; i<MAX_UNITS; i++ )
		if( units[i]!=NULL )
		{
			units[i]->screenSelect( sel1, sel2 );
		}
}

void Player::removeDeadUnits()
{
	int i;
	
	for( i=0; i<MAX_UNITS; i++ )
		if( units[i]!=NULL )
			if( !units[i]->isAlive() )
			{
				deleteUnit( units[i] );
			}
}

inline int Player::getUnitNumber( Unit *u )
{
	int i;
	
	for( i=0; i<MAX_UNITS; i++ )
	{
		if( units[i]==u ) return i;
	}
		
	return -1; //not found
}


/// Inserts a new Unit into units array. Returns true if succesfull, otherwise maximum number of units reached. findFreeUnit is now obsolete.
bool Player::insertNewUnit( Unit *upointer )
{
	int slot=Player::findFreeUnit(), i;
	UnitID newid;
	
	// if failed...
	if( slot==-1 ) {
		delete upointer; // useless unit
		return false;
	}
	
	//...otherwise insert unit
	newid=upointer->getID(); // update ID
	newid.setPlayernum( playerNum );
	newid.setUnitnum( slot );
	upointer->setUnitID( newid );

//printf( "p%d u%d->p%d u%d\n", newid.getPlayernum(), newid.getUnitnum(), upointer->getID().getPlayernum(), upointer->getID().getUnitnum());
		
	units[slot]=upointer; //insert
	unitcount++;
	
	// update lastunit and firstunit if necessary
//	if( slot <= firstunit )
		for( i=0; i<MAX_UNITS; i++ ) if( units[i]!=NULL ) {
			firstunit=i;
			break;
		}
//	if( slot >= lastunit )
		for( i=MAX_UNITS-1; i>=0; i-- ) if( units[i]!=NULL ) {
			lastunit=i;
			break;
		}
	return true;
}

UnitID Player::firstUnitID()
{
	static UnitID first;
	
	if( firstunit!=-1 && unitcount!=0 ){
		first=units[firstunit]->getID();
		first.setPlayernum( playerNum );
		first.setUnitnum( firstunit );
	} else { //if there is no units at all
		first.setID( -1 );
		first.setUnitnum( firstunit );
		first.setPlayernum( playerNum );
	}
	
	return first;
}

void Player::nextUnitID( UnitID &id )
{
	int i;

	for( i=id.getUnitnum()+1; i<=lastunit; i++ )
		if( units[i]!=NULL )
		{
//debug			printf("pl %d  last %d\n", i, lastunit);
			id=units[i]->getID();
			id.setPlayernum( playerNum );
			id.setUnitnum(i);
			return;
		}
		
	// no more units, cause iteration end
	id.setUnitnum(lastunit+1);
	return;
}

bool Player::lastUnitID( UnitID &id ) const
{
	if( unitcount==0 ) return true;
	if( id.unitnum >= lastunit ) return true;
//	if( id.getUnitnum() >= lastunit ) return true;
	return false;
}

bool Player::iterationEnd( UnitID &id ) const
{
	if( unitcount==0 ) return true;
	if( id.unitnum > lastunit ) return true;
//	if( id.getUnitnum() > lastunit ) return true;
	return false;
}

void Player::deleteUnit( Unit *upointer )
{
	int slot=getUnitNumber( upointer ), i;
	if( slot==-1 ) {
//		msgOut( MSG_ERROR, "Attempt to delete non existing unit\n" );
		return;
	}
	
	// delete
	delete units[slot];
	units[slot]=NULL;
	unitcount--;
	
	// update lastunit and firstunit if necessary
//	if( slot <= firstunit )
		for( i=0; i<MAX_UNITS; i++ ) if( units[i]!=NULL ) {
			firstunit=i;
			break;
		}
//	if( slot >= lastunit )
		for( i=MAX_UNITS-1; i>=0; i-- ) if( units[i]!=NULL ) {
			lastunit=i;
			break;
		}
	if( unitcount==0 ) firstunit=lastunit=-1;
}

void Player::deleteUnit( UnitID id )
{
	Unit *u=game->getUnit( id );
	
	deleteUnit( u );
}
