/***************************************************************************
                          player.h  -  description
                             -------------------
    begin                : Wed Jul 25 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef PLAYER_H
#define PLAYER_H

#include "allincludes.h"
#include "gamedefs.h"
#include "unitid.h"

#include "unitcommand.h"

/**Contains player data and units etc...
  *@author T Kontkanen
  */

class Unit;
class UnitMarine;
class UnitCommandCenter;
class UnitDestroyer;

class Player;
class Game;
class Ground;
class Renderer;
class Vector;

enum AllianceModes { PLAYER_ALLY, PLAYER_ENEMY, PLAYER_NEUTRAL, PLAYER_SELF };

class Player {
public:
	
	Player( Game *pgame, int myplayernum );
	~Player();
	
	/// Name tells everything...
	void createInitialUnits();
	
	void renderUnits();
	/// Execute action for selected units
//	void unitAction( int actionnum, Vector destpos );
	void unitAction( UnitCommand action );
	/// Execute primary action for selected units
//	void primaryUnitAction( Vector destpos );
	void primaryUnitAction( UnitCommand target );
	/// Execute every units's step() method
	void unitStep();
	/// Select units on screen, sel1 and sel2 defines 2d selection rectangle on screen.
	void selectUnits( Vector &sel1, Vector &sel2 );
	
	/// Returns alliance mode for specified player
	enum AllianceModes getPlayerAllianceMode( int player ) { return alliancemodes[player]; }
	/// Sets alliance mode for specific player.
	void setPlayerAllianceMode( int player, enum AllianceModes mode ) { alliancemodes[player]=mode; }
	/// Refresh player, should be called only once per frame.
	void refresh() { removeDeadUnits(); }
	/// Returns unit slot numver for specified Unit. Returns -1 if unit not found.
	int getUnitNumber( Unit *u );
	
	
	/// Find a unused Unt slot. Returns empty slot number or -1 if no slots left. Note: it's very important to check if return value is -1 because amount of units is limited to MAX_UNITS.
	int findFreeUnit();
	/// Inserts a new Unit into units array. Returns true if succesfull, otherwise maximum number of units reached. findFreeUnit is now obsolete.
	bool insertNewUnit( Unit *upointer );
	/// Returns number of units.
	int unitCount() { return unitcount; }
	/// Deletes unit.
	void deleteUnit( Unit *upointer );
	/// Deletes unit.
	void deleteUnit( UnitID id );
	/// Returns the first UnitID.
	UnitID firstUnitID();
	/// Returns next UnitID after 'id'. Useful when iterating through all ID's.
	void nextUnitID( UnitID &id );
	/// Returns true if 'id' is last available UnitID.
	bool lastUnitID( UnitID &id ) const;
	/// Returns true if 'id' is a UnitID after the last available UnitID.
	bool iterationEnd( UnitID &id ) const;
	
//	UnitID getLastUnitID();
protected:
	Unit *units[MAX_UNITS];

private:
	/// Removes all dead units.
	void removeDeadUnits();
	
	int playerNum; // number of this player
	int firstunit, lastunit, unitcount; // lookups for iteration
	
	Renderer *renderer;
	Ground *ground;
	Game *game;
	enum AllianceModes alliancemodes[MAX_PLAYERS];
	
	friend Unit; friend UnitMarine; friend UnitCommandCenter;
	friend Game;
};

#endif
