/***************************************************************************
                          texture.cpp  -  description
                             -------------------
    begin                : Fri Sep 7 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "texture.h"

Texture::Texture( const char *texturefile, enum AlphaModes alpha, bool mipmap, int minfilter, int magfilter, int wrapmode )
{
	setFuncName( "Texture::Texture" );
	
	pngInfo info;
	int glpng_alpha;
	int mipmapping, ret;
	
	msgOut( MSG_INIT, "Loading texture %s", texturefile );
	clients=1;
	
	switch( alpha ) {
		case BLEND_ALPHA: glpng_alpha=PNG_ALPHA; break;
		case BLEND_SOLID: glpng_alpha=PNG_SOLID; break;
		case BLEND_STENCIL: glpng_alpha=PNG_STENCIL; break;
		case BLEND_RGB: glpng_alpha=PNG_BLEND3; break;
		case BLEND_RGBDIST: glpng_alpha=PNG_BLEND8; break;
		default: glpng_alpha=PNG_SOLID; break;
	}
	if( mipmap==true ) mipmapping=PNG_BUILDMIPMAPS;
	else mipmapping=PNG_NOMIPMAPS;

	pngSetStencil( 255, 255, 255 );
	
	glPixelStorei( GL_UNPACK_ROW_LENGTH, 0 ); //GL defaults
	glPixelStorei( GL_UNPACK_ALIGNMENT,4 );
	
	ret=pngBind( texturefile, mipmapping, glpng_alpha, &info, wrapmode, minfilter, magfilter );
	
	if( ret ){
		//make new texture
		twidth=info.Width;
		theight=info.Height;
		filename = texturefile;
		texture_id=ret;
		msgOut( MSG_INIT, "  Texture id %d, size %dx%d", ret, twidth, theight );
	} else
	{
		texture_id=0;
		filename = "";
		twidth=theight=0;
		msgOut( MSG_ERROR, "Unable to load texture %s", texturefile );
	}
}

Texture::Texture( string texturefilename, enum AlphaModes alpha, bool mipmap, int minfilter, int magfilter, int wrapmode )
{
	Texture( texturefilename.c_str(), alpha, mipmap, minfilter, magfilter, wrapmode );
}

Texture::~Texture() {
	setFuncName( "Texture::~Texture" );

	if( clients >0 ) {
		msgOut( MSG_WARN, "Deleting texture with %d clients", clients );
	}
	
	GLuint ids[2];
	
	if( !texture_id ) return;
	msgOut( MSG_UNINIT, "Deleting texture %s (Id %d)", filename.c_str(), texture_id );
	
	
	ids[0]=texture_id;
	ids[1]=0;
	filename="";
	
	glDeleteTextures( 1, ids );
}


bool Texture::isSameFile( const char *fname )
{
	if( !strcmp( fname, filename.c_str() ) ) return true;
	else return false;
}


