/***************************************************************************
                          texture.h  -  description
                             -------------------
    begin                : Fri Sep 7 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TEXTURE_H
#define TEXTURE_H

#include "allincludes.h"
#include "GL/gl.h"
#include "GL/glpng.h"

class Renderer;

/**Textures for Renderer
  *@author T Kontkanen
  */

enum AlphaModes { BLEND_SOLID, BLEND_ALPHA, BLEND_STENCIL, BLEND_RGBDIST, BLEND_RGB };

class Texture {
public:
	/// Create new texture from bitmap file. Currently supports only PNG. It's better to use Renderer's texture management than this class.
	Texture( const char *texturefile, enum AlphaModes alpha, bool mipmap, int minfilter, int magfilter, int wrapmode );
	Texture( string texturefilename, enum AlphaModes alpha, bool mipmap, int minfilter, int magfilter, int wrapmode );
	~Texture();
	
	/// Return texture width in pixels
	int width() { return twidth; }
	/// Return texture height in pixels
	int height() { return theight; }
	/// Return GL texture ID or zero if no texture loaded
	GLint getTextureID() { return texture_id; }
	/// Return texture filename
	const char *getFilename() { return filename.c_str(); }
	/// Compares fname with current texture filename and return true if names are poining at same file.
	bool isSameFile( const char *fname );
protected:
	/// Determines how many classes are using this texture.
	int clients;
	friend Renderer;
private:
	GLint texture_id;
	int twidth, theight;
//	const char *filename;
	string filename;
};

#endif
