/***************************************************************************
                          unitcommand.h  -  description
                             -------------------
    begin                : Mon Oct 8 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef UNITCOMMAND_H
#define UNITCOMMAND_H

#include "allincludes.h"
#include "vector.h"
#include "texture.h"
#include "unitid.h"

/**Holds information about unit actions like move, attack etc. UnitCommands are used in communication between Units and GUI.
  *@author T Kontkanen
  */

typedef enum TargetType { TARGET_UNIT, TARGET_POSITION };

class UnitCommand {
public: 
	UnitCommand( int actionid=A_NONE );
	~UnitCommand();

	/// Set target position of action
	void setPosition( Vector pos ) {
		targetpos=pos;
		targettype=TARGET_POSITION;
	}
	/// Return target position of action
	Vector getPosition() { return targetpos; }
	/// Set target UnitID
	void setTargetUnit( UnitID id ) {
		targetunitid=id;
		targettype=TARGET_UNIT;
	}
	/// Returns target UnitID
	UnitID getTargetUnit() { return targetunitid; }
/*	void setTargetUnit( int player, int unit ) {
		targetplayer=player;
		targetunit=targetunit;
		targettype=TARGET_UNIT;
	}*/
	/// Get target type enumerant, can be TARGET_UNIT or TARGET_POSITION
	TargetType getTargetType() { return targettype; }
	/// Return a c++ string containing button icon filename
	Texture* getIcon() { return icon; }
	/// Set butotn icon filename
	void setIcon( Texture *icontexture ) { icon=icontexture; }
	/// Return a c++ string containing button text
	string getButtonText() { return buttontext; }
	/// Set a button text
	void setButtonText( string text ) { buttontext=text; }
	/// Set action identifier number, see gamedefs.h for common action id's.
	void setActionID( int num ) { id=num; }
	/// Returns action identifier number.
	int getActionID() { return id; }
	/// Returns true if target is required
	bool needTarget() { return needtarget; }
	/// Set value that needTarget() returns. Default is false.
	void setTargetRequirement( bool needed ) { needtarget=needed; }
	
	/// Compares getActionID() values, needed in sorting action GUIBbuttons.
	bool operator<(const UnitCommand& compare) const;

private:
	Vector targetpos;
	Texture *icon;
	string buttontext;
	int id;
	bool needtarget;
	TargetType targettype;
	int targetplayer, targetunit;
	UnitID targetunitid;
};

#endif
