/***************************************************************************
                          unitdestroyer.cpp  -  description
                             -------------------
    begin                : Wed Oct 17 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "unitdestroyer.h"


//UnitDestroyer::UnitDestroyer( Game *pgame, int myplayernum, int myunitnum ): Unit( pgame ), AirMovement( pgame ), BulletWeapon( pgame, this )
UnitDestroyer::UnitDestroyer( Game *pgame ): Unit( pgame ), AirMovement( pgame ), BulletWeapon( pgame, this ), TargetFinder( pgame, this )
{
	game=pgame;
	renderer=pgame->renderer;
	ground=pgame->ground;
	players=pgame->players;
	
	//load model
	model=renderer->loadTGLObject( game->dataRoot()+DESTROYER_MODEL, 10 );
//	model=renderer->loadTGLObject( "/files/3d/spherel.t3d" );
	// load textures
	particle=renderer->loadTexture( game->dataRoot()+TEXTUREPATH+"explosion2.png", BLEND_ALPHA, true, GL_LINEAR, GL_LINEAR );
	
	//misc
	getting_damage=false;
	death=0;
	
	//parameters
	setPrimaryAction( A_MOVE );
	setName( "Destroyer" );
	setMaxHitpoints( 750 );
	setHitpoints( 1.0 );
	setUnitType( AIR_UNIT );
	setSeekDistance( 110 );
	setFinderParams( true, true );
	setSize( 70 );
	setMaxSize( 100 );
	
	//init movement
	setTopSpeed(75);
	setTurnRate(0.5);
	setAcceleration(10);
	
	//weapon
	setAverageDamage( 55*1 );
	setFireRate( 1 );
	setSightRange( 150 ); //if you edit these, modify also findTarget
	setMinRange( 0 );
	setDamage( 55 );
	setAccuracy( 1/100.0 );
}


UnitDestroyer::~UnitDestroyer(){
	renderer->freeTexture( particle );
	renderer->freeTGLObject( game->dataRoot()+DESTROYER_MODEL );
}

int UnitDestroyer::render() // render this unit
{
	msg::setFuncName( "UnitDestroyer::render" );
	renderer->setRenderingMode( RENDER_TGL_OBJECT );
	model->move( getPosition().x(), getPosition().y(), getPosition().z()+1 );
	model->rotate( 0, 0, 1, RAD_TO_DEG(getAngle()) ); //angle
//	model->rotate( 0, 0, 1, 90 ); //angle
	model->rotate( 1, 0, 0, 90 ); //angle
//	model->scale(10,10,10);
	
	// morph according to damage
	model->set_morph( 0, 1 );
	model->morph( 1-(getHitpoints()/getMaxHitpoints()) );

	model->draw();

	if( getting_damage ) { //add blood particle
		static float particlepos;
		renderer->setParticleMode( FUNC_ALPHA, 10 );
		renderer->addParticleEffect( getPosition(), Vector(10,10,10), Vector(50,50,1), Color(1,1,1,1), Color(0,0,0,0), 1, particle );
		getting_damage=false;
		particlepos+=0.2;
		if( particlepos > 0.6 ) particlepos=-0.6;
	}

	//line
	if( shootingNow() ) {
		linep1=getUnitPosition()+Vector(20,00,0);
		linep2=shootingAtPos();
		linecol.set(0.1,1,0.2,1);
	}
	Color sub;
	renderer->renderLine( linep1, linep2, linecol, FUNC_ADD );

	return 0;
}

int UnitDestroyer::action( UnitCommand action )
{
	int actionnum=action.getActionID();
	Vector destpos=action.getPosition();
	switch( actionnum ) {
		case A_MOVE:
			moveSetDestination( destpos );
			break;
		default: break;
	}
	
	return 0;
}


int UnitDestroyer::step()
{
	moveStep();
	setTargetUnit( findTarget().getTargetUnit() );
	weaponThink();
	
	// process rendering
//	linecol-=Color(.05,0.05,.05,.02); //Func_add mode
	linecol-=Color(.03, .03, .03, .005);
	
	//process death animation
	if( hitpoints <= 0 )
		death+=PER_FRAME(1);

	return 0;
}

