/***************************************************************************
                          unitmarine.cpp  -  description
                             -------------------
    begin                : Fri Jul 27 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "unitmarine.h"

//UnitMarine::UnitMarine( Game *pgame, int myplayernum, int myunitnum ): Unit( pgame ), GroundMovement( pgame ), BulletWeapon( pgame, this )
UnitMarine::UnitMarine( Game *pgame ): Unit( pgame ), GroundMovement( pgame ), BulletWeapon( pgame, this ), TargetFinder( pgame, this )
{
	game=pgame;
	renderer=pgame->renderer;
	ground=pgame->ground;
	players=pgame->players;
	
//	playerNum=myid.getPlayernum();
//	playerNum= getUnitSide();
//	playerNum=myplayernum;
//	unitNum=myunitnum;
	
	//load model
	model_hq=renderer->loadTGLObject( game->dataRoot()+MODELPATH+"marine.t3d" );
	model_lq=renderer->loadTGLObject( game->dataRoot()+MODELPATH+"marinelq.t3d" );
	
	// load textures
	particle=renderer->loadTexture( game->dataRoot()+TEXTUREPATH+"lightparticle.png", BLEND_STENCIL, true, GL_LINEAR, GL_LINEAR );
	
	//parameters
	setName( "Marine" );
	setMaxHitpoints( 50 );
	setHitpoints( 1.0 );
	setUnitType( GROUND_UNIT );
	setSeekDistance( 110 );
	setFinderParams( true, true );
	setPrimaryAction( A_MOVE );
	setSize( 0.5 );
	setMaxSize( 1 );
	bleeding=false;
	death=0;
	
	//init movement
	setTopSpeed(3.0);
	setTurnRate(2.5);
	setAcceleration(2);
	setOriginHeight( 1 );
	
	//weapon
	setAverageDamage( 8*3 );
	setFireRate( 3 );
	setSightRange( 100 ); //if you edit these, modify also setSeekDistance
	setMinRange( 0 );
	setDamage( 8 );
	setAccuracy( 0.5/100.0 );
	
	loadSequences();
}


UnitMarine::~UnitMarine(){
	renderer->freeTGLObject( game->dataRoot()+MODELPATH+MARINE_MODEL );
	renderer->freeTexture( particle );
}

int UnitMarine::render() // render this unit
{
	float dist;
	msg::setFuncName( "UnitMarine::render" );
	
	dist=renderer->isVisible( getPosition(), getMaxSize() );
	if( dist>0 && dist<800 ) // if distance over 800, don't render, it looks like pixel
	{
		if( dist > 100 ) // use low-quality model
			model=model_lq;
		else model=model_hq;
		
		renderer->setRenderingMode( RENDER_TGL_OBJECT );
		model->move( getPosition().x(), getPosition().y(), getPosition().z()-originheight );
		model->rotate( 0, 0, 1, RAD_TO_DEG(getAngle()) ); //angle
	
/*		if( hitpoints<=0 )//dying
		{
			//death animation
			model->move( 0,0,-death );
			model->rotate( 0, 1, 0, -90*death ); //angle
		}*/
	
		model->rotate( 0, 0, 1, 90 ); //angle
		model->rotate( 1, 0, 0, 90 ); //angle
		model->set_morph( prevframe, currframe );
		model->morph( currframe_pos );
		model->draw();
	}
	renderer->setParticleMode( FUNC_ADD, 1 );
//	renderer->setParticleMode( FUNC_ADD );
//	renderer->renderParticle( getPosition()+Vector(0,0,5), Vector(1,1,1), particle, Color(1,1,1,1) );
//	renderer->addParticleEffect( getPosition()+Vector(0,0,6), Vector(1,1,1), Vector(2,2,2), Color(1,1,1,1), Color(0,0,0,0), 1, particle );
	
	if( bleeding ) { //add blood particle
//		static float bloodpos;
		renderer->setParticleMode( FUNC_ADD, 10 );
//		renderer->addParticleEffect( getPosition()+Vector(0+bloodpos,0+bloodpos,2+bloodpos), Vector(1,1,1), Vector(4,4,1), Color(1,0,0,1), Color(0,0,0,0), 1, particle );
		renderer->addParticleEffect( hitpos, Vector(1,1,1), Vector(4,4,1), Color(0.6,0,0,1), Color(0,0,0,0), 1, particle );
		bleeding=false;
//		bloodpos+=0.2;
//		if( bloodpos > 0.6 ) bloodpos=-0.6;
	}
	
	//line
	if( shootingNow() ) {
		linep1=getUnitPosition();
		linep2=shootingAtPos();
//		linecol.set(0.3,0.2,0,0.5);
		linecol.set(1,0.7,0,0.3);
	}
	Color sub;
	renderer->renderLine( linep1, linep2, linecol, FUNC_ALPHA );
	
	return 0;
}

//int UnitMarine::action( int actionnum, Vector destpos )
int UnitMarine::action( UnitCommand action )
{
	int actionnum=action.getActionID();
	Vector destpos=action.getPosition();
	UnitID i;

	job=action;
	
	switch( action.getActionID() ) {
		case A_MOVE:
//			moveSetDestination( destpos );
			break;
		case A_TEST:
//			printf( "unum %d, pnum %d\n", myID.getUnitnum(), myID.getPlayernum() );
/*			for( i=players[getUnitSide()]->firstUnitID(); !players[getUnitSide()]->iterationEnd(i);players[getUnitSide()]->nextUnitID(i) )
			{
				printf( "p id %d\n", i.getID() );
			}
			for( i=players[1]->firstUnitID(); !players[1]->iterationEnd(i);players[1]->nextUnitID(i) )
			{
				printf( "p2 id %d\n", i.getID() );
			}*/
/*			for( i=game->firstUnitID(); !game->iterationEnd(i);game->nextUnitID(i) )
			{
				printf( "id %d\n", i.getID() );
			}*/
			break;
/*		case 17:
			int unit=players[playerNum]->findFreeUnit();
			if( unit == -1 ) return 0;
			msgOut( MSG_DEBUG_4, "Unit #%d, action %d", unitNum, actionnum );
//			players[playerNum]->units[unit]=new UnitMarine( renderer, ground, players, playerNum, unitNum );
			players[playerNum]->units[unit]=new UnitMarine( game, playerNum, unit ); //testgame
			players[playerNum]->units[unit]->setUnitPosition( destpos );
			break;*/
		default: break;
	}
	
	return 0;
}


int UnitMarine::step()
{
	moveStep();
	
	if( hitpoints>0 ) switch( job.getActionID() ) //if alive, do the job
	{
		case A_MOVE:
			processAnimation(walk_sequence, walk_fps, A_MOVE );
			moveSetDestination( job.getPosition() );
			break;
		case A_ATTACK:
			moveSetDestination( job.getPosition() );
			setTargetUnit( findTarget().getTargetUnit() );
			weaponThink();
			break;
		case A_CANCEL:
			processAnimation(stand_sequence, stand_fps, A_STOP);
			stopMoving();
			setTargetUnit( findTarget().getTargetUnit() );
			weaponThink();
			break;
		case A_STOP:
			processAnimation(stand_sequence, stand_fps, A_STOP);
			stopMoving();
			setTargetUnit( findTarget().getTargetUnit() );
			weaponThink();
			break;
		case A_NONE:
			processAnimation(stand_sequence, stand_fps, A_STOP);
			stopMoving();
			setTargetUnit( findTarget().getTargetUnit() );
			weaponThink();
			break;
		default:
			break;
	}
	else { //death animation
		stopMoving();
		processAnimation( death_sequence, death_fps, 58043 );
	}
	
	// process rendering
//	linecol-=Color(.05,0.05,.05,.02); //Func_add mode
	linecol-=Color(.00,0,-0.06,.02); //shot
	
//	if( hitpoints<=0 )//dying
//		death+=PER_FRAME(1);
	
	return 0;
}

int UnitMarine::getNumberOfActions()
{
	return 4;
}

UnitCommand UnitMarine::getActionType( int number )
{
	UnitCommand cmd;
	
	switch( number )
	{
		case 0:
			cmd.setActionID(A_MOVE);
			cmd.setButtonText("Move");
			cmd.setIcon( game->iconMove );
			cmd.setTargetRequirement(true);
			break;
		case 1:
			cmd.setActionID(A_STOP);
			cmd.setButtonText("Stop");
			cmd.setIcon( game->iconNone );
			cmd.setTargetRequirement(false);
			break;
		case 2:
			cmd.setActionID(A_ATTACK);
			cmd.setButtonText("Attack");
			cmd.setIcon( game->iconAttack );
			cmd.setTargetRequirement(true);
			break;
		case 3:
			cmd.setActionID(A_CANCEL);
			cmd.setButtonText("Cancel");
			cmd.setIcon( game->iconCancel );
			cmd.setTargetRequirement(false);
			break;
		default:
			cmd.setActionID(A_NONE);
			cmd.setButtonText("None");
			cmd.setIcon( game->iconNone );
			cmd.setTargetRequirement(false);
			break;
	}
		
	return cmd;
}

void UnitMarine::loadSequences()
{
	int i, len;
	Config cfg;
	char key[128];
	string frame;
	
	prevframe=currframe=0;
	currframe_pos=0;
	sequenceframe=0;
	currsequence_num=0;
	
	cfg.setVerbosity(false); //quiet
	cfg.readFile( game->dataRoot()+UNITPATH+"marine.unit" );
	cfg.setGroup( "um" );
	
	//move
	len=cfg.getInt( "moveSequenceLen", 1 );
	for( i=0; i<len; i++ ) //read sequence
	{
		sprintf( key, "moveSequence_%d", i+1 );
		frame=key;
		walk_sequence.push_back( cfg.getInt( frame, 0 ) );
	}
	walk_fps=cfg.getFloat( "moveSequenceFPS", 1 );

	//stand
	len=cfg.getInt( "standSequenceLen", 1 );
	for( i=0; i<len; i++ ) //read sequence
	{
		sprintf( key, "standSequence_%d", i+1 );
		frame=key;
		stand_sequence.push_back( cfg.getInt( frame, 0 ) );
	}
	stand_fps=cfg.getFloat( "standSequenceFPS", 1 );

	//die
	len=cfg.getInt( "deathSequenceLen", 1 );
	for( i=0; i<len; i++ ) //read sequence
	{
		sprintf( key, "deathSequence_%d", i+1 );
		frame=key;
		death_sequence.push_back( cfg.getInt( frame, 0 ) );
	}
	death_fps=cfg.getFloat( "deathSequenceFPS", 1 );
}

void UnitMarine::processAnimation( vector <unsigned int>sequence, float fps, unsigned int sequence_num )
{
	currframe_pos+=PER_FRAME(fps);
	if( currframe_pos > 1 ) //advance to next frame
	{
		currframe_pos=0;
//		prevframe=sequence[sequenceframe];
		prevframe=currframe;
		
			
		sequenceframe++;
		if( sequenceframe >= sequence.size() ) //restart sequence
			sequenceframe=0;

		if( currsequence_num!=sequence_num )
			sequenceframe=0; //start new sequence
		currsequence_num=sequence_num;
			
		currframe=sequence[sequenceframe];
	}
}

