/***************************************************************************
                          unit.h  -  description
                             -------------------
    begin                : Wed Jul 25 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef UNITMARINE_H
#define UNITMARINE_H

#include "renderer.h"
#include "player.h"
#include "ground.h"
#include "allincludes.h"
#include "gamedefs.h"
#include "vector.h"
#include "groundmovement.h"
#include "bulletweapon.h"
#include "unit.h"
#include "targetfinder.h"

/**Marine unit
  *@author T Kontkanen
  */

class Player;
class Unit;

using namespace TGL;
using namespace std;	

class Game;

class UnitMarine : public Unit, GroundMovement, BulletWeapon, TargetFinder {
public:
//	UnitMarine::UnitMarine( Game *pgame, int myplayernum, int myunitnum );
	UnitMarine::UnitMarine( Game *pgame );
	virtual ~UnitMarine();

	//controls
	virtual int action( UnitCommand action );
//	virtual int action( int actionnum, Vector destpos );
	
	//render
	virtual int render(); // render this unit
	
	virtual int step();
	
	virtual void setUnitPosition( Vector pos ) { setPosition( pos ); }
	virtual Vector getUnitPosition( void ) { return getPosition(); }
	
	/// Returns a number of possible actions of this unit
	virtual int getNumberOfActions();
	/// Returns a action type where "number" is something between 0 and getNumberOfActions().
	virtual UnitCommand getActionType( int number );
	
	/// Needed for death animation.
	virtual bool isAlive() {
//		if( hitpoints>0 || (currframe!=death_sequence[death_sequence.size()-1] && currframe_pos>=.99 ) )return true;
		if( hitpoints <= 0 && currframe==death_sequence[death_sequence.size()-1] && currframe_pos>=.99 ) return false;
//		if( hitpoints>0 )
//			if(currframe!=death_sequence[death_sequence.size()-1] && currframe_pos>=.99 ) return true;
		return true;
//		return false;
//		if( hitpoints>0 || death<1 ) return true; else return false;
	}
	/// Needed for bleeding.
	virtual void decreaseHitpoints( float amount, Vector hitposition ) {
		hitpoints-=amount;
		bleeding=true;
		hitpos=hitposition;
	}
	virtual bool isMoving() { if( velocity != 0 ) return true; else return false; } //DIRECT
private:
	/// Loads unit animation sequences
	void loadSequences();
	/// Processes model animation.
	void processAnimation( vector <unsigned int>sequence, float fps, unsigned int sequence_num );
	
	//needed information
//	int playerNum; // number of this player
	tgl_object *model, *model_hq, *model_lq;
	Texture *particle;
	Vector linep1, linep2; //line pos
	Color linecol; //line color
	bool bleeding;
	float death; //0=alive,  1>=dead
	Vector hitpos;
	UnitCommand job;
	//anim
	vector <unsigned int> walk_sequence;
	vector <unsigned int> stand_sequence;
	vector <unsigned int> death_sequence;
	float walk_fps, stand_fps, death_fps, currframe_pos;
	unsigned int prevframe, currframe, sequenceframe, currsequence_num;
	
	/// Try to find a new target
//	void findTarget();
	
	Renderer *renderer;
	Game *game;
	Ground *ground;
	Player **players; //the other players including this

	//unit information
	int unitNum; // number of this unit
};

#endif
