/***************************************************************************
                          vector.cpp  -  description
                             -------------------
    begin                : Sun Jul 22 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "vector.h"

using namespace msg;

Vector::~Vector(){
}

void Vector::rotateX( float x )
{
	//rotate j,k
/*   X2 = X1 * cos(k) - Y1 * sin(k)
     Y2 = X1 * sin(k) + Y1 * cos(k). */
	float j2=vj*cos(x)-vk*sin(x); //no reason to use matrices here
	float k2=vj*sin(x)+vk*cos(x);
	vj=j2;
	vk=k2;
}

void Vector::rotateY( float x )
{
	//rotate i,k
/*   X2 = X1 * cos(k) - Y1 * sin(k)
     Y2 = X1 * sin(k) + Y1 * cos(k). */
	float i2=vi*cos(x)-vk*sin(x);
	float k2=vi*sin(x)+vk*cos(x);
	vi=i2;
	vk=k2;
}
void Vector::rotateZ( float x )
{
	//rotate i,j
/*   X2 = X1 * cos(k) - Y1 * sin(k)
     Y2 = X1 * sin(k) + Y1 * cos(k). */
	float i2=vi*cos(x)-vj*sin(x);
	float j2=vi*sin(x)+vj*cos(x);
	vi=i2;
	vj=j2;
}

void Vector::normalize()
{
	float len = length();
	if( len ) {
		vi/=len;
		vj/=len;
		vk/=len;
	}
}

void Vector::scale( float x )
{
	vi*=x;
	vj*=x;
	vk*=x;
}

Vector Vector::operator=(const Vector& vector)
{
	vi = vector.vi;
	vj = vector.vj;
	vk = vector.vk;
	return *this;
}


Vector Vector::operator*(const float mul)
{
	Vector res;
	res = *this;
	res.scale(mul);
	return res;
}

Vector Vector::operator/(const float div)
{
	if( div<0.0000001 ) msgOut(  MSG_ERROR, "Division by zero (Vector)" );
	Vector res;
	res = *this;
	res.scale( 1.0/div );
	return res;
}

Vector Vector::operator+(const Vector& vector)
{
	Vector res;
	
	res.vi=vi+vector.vi;
	res.vj=vj+vector.vj;
	res.vk=vk+vector.vk;
	
	return res;
}

Vector Vector::operator-(const Vector& vector)
{
	Vector res;
	
	res.vi=-vector.vi+vi;
	res.vj=-vector.vj+vj;
	res.vk=-vector.vk+vk;
	
	return res;
}

Vector Vector::operator+=(const Vector& vector)
{
	vi+=vector.vi;
	vj+=vector.vj;
	vk+=vector.vk;
	
	return *this;
}

Vector Vector::operator-=(const Vector& vector)
{
	vi-=vector.vi;
	vj-=vector.vj;
	vk-=vector.vk;
	
	return *this;
}

bool Vector::operator>=(const Vector& vector)
{
	if( vi>=vector.vi && vj>=vector.vj && vk>=vector.vk ) return true;
	return false;
}

bool Vector::operator<=(const Vector& vector)
{
	if( vi<=vector.vi && vj<=vector.vj && vk<=vector.vk ) return true;
	return false;
}

bool Vector::operator>(const Vector& vector)
{
	if( vi>vector.vi && vj>vector.vj && vk>vector.vk ) return true;
	return false;
}

bool Vector::operator<(const Vector& vector)
{
	if( vi<vector.vi && vj<vector.vj && vk<vector.vk ) return true;
	return false;
}

bool Vector::operator==(const Vector& vector)
{
	if( vi==vector.vi && vj==vector.vj && vk==vector.vk ) return true;
	return false;
}

float Vector::dot( const Vector &vector ) const
{
	return( vi*vector.vi+ vj*vector.vj + vk*vector.vk );
}

float Vector::dot( const Vector &vector, const Vector &vector2 ) const
{
	return( vector.dot(vector2) );
}

float* Vector::ijk()
{
//	float fv[3];
	varray[0]=vi;
	varray[1]=vj;
	varray[2]=vk;
	return varray;
}

