/***************************************************************************
                          vector.h  -  description
                             -------------------
    begin                : Sun Jul 22 2001
    copyright            : (C) 2001 by T Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef VECTOR_H
#define VECTOR_H

#include <math.h>
#include <msgout.h>

/**Simple vector class
  *@author T Kontkanen
  */


// Note: Vector class currently not tested, may be buggy
class Vector {
public:
	/// Vector with defineable i,j,k values
	Vector( float i1=0, float j1=0, float k1=0 ) { vi=i1; vj=j1; vk=k1; }
	~Vector();
	/// Set vector i,j,k, passing -1 leaves original value untouched
	void set( float i1=-1, float j1=-1, float k1=-1 ) { if(i1!=-1)vi=i1; if(j1!=-1)vj=j1; if(k1!=-1)vk=k1; }
	/// Set I value
	void setI( float val ) { vi=val; }
	/// Set J value
	void setJ( float val ) { vj=val; }
	/// Set K value
	void setK( float val ) { vk=val; }
	/// Set X value, X value is similar to I.
	void setX( float val ) { vi=val; }
	/// Set Y value. Y value is similar to J.
	void setY( float val ) { vj=val; }
	/// Set Z value. Z value is similar to K.
	void setZ( float val ) { vk=val; }
	/// Returns i-component (same as x)
	float i() { return vi; }
	/// Returns j-component (same as u)
	float j() { return vj; }
	/// Returns k-component (same as z)
	float k() { return vk; }
	/// Returns x-component (same as i)
	float x() { return vi; }
	/// Returns y-component (same as j)
	float y() { return vj; }
	/// Returns z-component (same as k)
	float z() { return vk; }
	/// Returns a pointer to array of 3 floats whick contains current i, j, k values.
	float* ijk();
	/// Returns a pointer to array of 3 floats whick contains current x, y, z values.
	float* xyz() { return ijk(); }
	/// Rotate vector around X axis by "vx" radians.
	void rotateX( float vx );
	/// Rotate vector around Y axis by "vx" radians.
	void rotateY( float vx );
	/// Rotate vector around Z axis by "vx" radians.
	void rotateZ( float vx );
	/// Normalize vector.
	void normalize();
	/// Multiply i,j,k values by x.
	void scale( float x );
	/// Returns true if vector is valid
	bool isValid() const { if( length()>0.0000001 ) return true; else return false; }
	/// Returns lenth of Vector.
	float length() const { return sqrt( vi*vi + vj*vj + vk*vk ); }	
	/// Calculate and return dot product with another Vector.
	float dot( const Vector &vector ) const; //dot product
	/// Calculate and return dot product of two Vectors.
	float dot( const Vector &vector, const Vector &vector2 ) const;
	/// Return angle between two Vectors
	float angle( const Vector &v ) const {
		if( isValid() && v.isValid() )
			return acos( dot(v)/(v.length()*length()) );
		else return 0;
	}
	
	Vector operator*(const float mul);
	Vector operator/(const float div);
	Vector operator=(const Vector& vector);
	Vector operator+(const Vector& vector);
	Vector operator-(const Vector& vector);
	Vector operator+=(const Vector& vector);
	Vector operator-=(const Vector& vector);
	/// Compares i,j and k values not length.
	bool operator==(const Vector& vector);
	bool operator>=(const Vector& vector);
	bool operator>(const Vector& vector);
	bool operator<=(const Vector& vector);
	bool operator<(const Vector& vector);
	
private:
	float vi, vj, vk;
	float varray[3]; // stores i,j,k values for ijk() and xyz()
};

#endif
