// Demeter Terrain Visualization Library by Clay Fowler
// Copyright (C) 2001 Clay Fowler

/*
This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Library General Public License for more details.

You should have received a copy of the GNU Library General Public
License along with this library; if not, write to the
Free Software Foundation, Inc., 59 Temple Place - Suite 330,
Boston, MA  02111-1307, USA.
*/

/*
This is a utility application that slices large input elevation
and texture images into smaller parts to be used with Demeter's
terrain paging feature.
*/

#include "Terrain.h"
#include "SDL_image.h"
#include <iostream>

using namespace Demeter;

void ShowUsage();
void ChopImage(char* szFilename,char* szBaseName,int widthSubTerrains,int subWidth);
void CheckSuperImage(char* szFilename,int subWidth,int numSubTerrains);
void FindExtension(char* szInput,char** szExtension);

#undef main
void main( int argc,char** argv )
{
    char* szMasterFilename;
    
	Settings* pSettings = Settings::GetInstance();

    if (argc < 5)
        ShowUsage();
    else
    {
        szMasterFilename = argv[1];

        pSettings->SetCompilerOnly(true);
        pSettings->SetVerbose(true);

        // Initialize rendering target so that textures can be properly handled.
        if (SDL_Init( SDL_INIT_VIDEO ) < 0)
        {
            printf("MAPC: ERROR: Unable to initialize SDL: %s\n",SDL_GetError());
            exit(1);
        }
		
        bool bSuccess = false;

		char* szBaseName = argv[2];
		int widthSubTerrains = atoi(argv[3]);
		int subWidth = atoi(argv[4]);

		CheckSuperImage(szMasterFilename,subWidth,widthSubTerrains);
		ChopImage(szMasterFilename,szBaseName,widthSubTerrains,subWidth);
		
		bSuccess = true;

        exit( 0 );
    }
}

void CheckSuperImage(char* szFilename,int subWidth,int numSubTerrains)
{
    SDL_Surface* pImage = IMG_Load(szFilename);
    if (pImage == NULL)
    {
        cout << "MAPC: ERROR: Failed to load image " << szFilename << endl;
        exit(-1);
    }
    else
    {
        if (pImage->w != pImage->h)
        {
            cout << "MAPC: ERROR: Input image " << szFilename << " is not square" << endl;
            exit(-2);
        }
        else
        {
            if (pImage->w != numSubTerrains * subWidth - (numSubTerrains - 1))
            {
                cout << "MAPC: ERROR: Input image " << szFilename << " width and height must be equal to NumSubTerrains * SubWidth - (NumSubTerrains - 1)\nSee the programmer's guide for details" << endl;
                exit(-3);
            }
        }
    }
}

void ChopImage(char* szFilename,char* szBaseName,int widthSubTerrains,int subWidth)
{
    SDL_Surface* pImageTotal = IMG_Load(szFilename);
	
    int y = 0;
	int k = 0;
	int total = widthSubTerrains * widthSubTerrains;
    for (int i = 0; i < widthSubTerrains; i++,y += subWidth - 1)
    {
        for (int j = 0,x = 0; j < widthSubTerrains; j++,x += subWidth - 1,k++)
        {
			cout << "MAPC: Processing sub terrain " << k << " of " << (total - 1) << endl;
            SDL_Surface* pSubSurface = SDL_CreateRGBSurface(SDL_SWSURFACE,subWidth,subWidth,pImageTotal->format->BitsPerPixel,pImageTotal->format->Rmask,pImageTotal->format->Gmask,pImageTotal->format->Bmask,pImageTotal->format->Amask);
            SDL_Rect sourceRect={x,y,subWidth,subWidth};
            SDL_BlitSurface(pImageTotal,&sourceRect,pSubSurface,NULL);
            char szOutputFilename[1024];
            sprintf(szOutputFilename,"%s.bmp.%ld-%ld",szBaseName,j,i);
            SDL_SaveBMP(pSubSurface,szOutputFilename);
            SDL_FreeSurface(pSubSurface);
        }
    }

    SDL_FreeSurface(pImageTotal);
}

void ShowUsage()
{
	cout << "This is a utility application that slices large input elevation" << endl;
	cout << "and texture images into smaller parts to be used with Demeter's" << endl;
	cout << "terrain paging feature." << endl;
	cout << "Binary map files are no longer used" << endl;
    cout << "USAGE: mapc <MasterFile> <BaseName> <NumSubTerrains> <SubSize> [OutputPath]" << endl;
}

void FindExtension(char* szInput,char** szExtension)
{
	*szExtension = NULL;
	char* szFoundExtension = strstr(szInput,".");
	if (szFoundExtension != NULL)
	{
		szFoundExtension++;
		char* szChars = new char[strlen(szFoundExtension) + 1];
		sprintf(szChars,szFoundExtension);
		*szExtension = szChars;
	}
}

