#include <stdio.h>
#include <iostream>
#include "Demeter/Terrain.h"
#include <jni.h>

using namespace Demeter;

Terrain** pTerrains;
int capacityTerrains = 5;
float MAX_VIEW_DISTANCE = 5000.0f;

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_init(JNIEnv* pEnv,jobject object)
{
	pTerrains = new Terrain*[capacityTerrains];
	
	for (int i = 0; i < capacityTerrains; i++)
		pTerrains[i] = NULL;	
}

extern "C" JNIEXPORT jint JNICALL Java_demeter_Terrain_createNativeTerrain(JNIEnv* pEnv,jobject object,jstring mapFilename, jint maxTriangles)
{
    const char* szTempFilename = pEnv->GetStringUTFChars(mapFilename,0);
    char* szFilename = new char[strlen(szTempFilename)+1];
    sprintf(szFilename, szTempFilename);
    
    int emptySlot = -1;
    
	for (int i = 0; i < capacityTerrains && emptySlot == -1; i++)	    
	{
		if(pTerrains[i] == NULL)
			emptySlot = i;
	}
	if (emptySlot == -1)
	{
		Terrain** pTemp = new Terrain*[capacityTerrains * 2];
		
		for (int i = 0; i < capacityTerrains * 2; i++)
		{
			if(i < capacityTerrains)
				pTemp[i] = pTerrains[i];
			else
				pTemp[i] = NULL;
		}
		
		emptySlot = capacityTerrains;
		capacityTerrains *= 2;
		
		delete[] pTerrains;
		pTerrains = pTemp;
	}
	pTerrains[emptySlot] = new Terrain(szFilename, maxTriangles);
	
	delete[] szFilename;	  
    pEnv->ReleaseStringUTFChars(mapFilename, szTempFilename);
    
    return emptySlot;
}

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_createRenderWindow(JNIEnv* pEnv, jobject object, jint width, jint height, jint colorDepth, jboolean fullscreen)
{
	Settings::GetInstance()->SetScreenWidth(width);
	Settings::GetInstance()->SetScreenHeight(height);
	
    if (SDL_Init(SDL_INIT_VIDEO) < 0)
        cout << "Unable to initialize SDL" << endl;
    else
    { 
		atexit(SDL_Quit);
		SDL_GL_SetAttribute(SDL_GL_BUFFER_SIZE,32);
	    SDL_GL_SetAttribute(SDL_GL_DOUBLEBUFFER,1);

	    SDL_EnableUNICODE(1);
	    
	    Uint32 screenFlags = SDL_OPENGL;
	    
	    if (fullscreen)
	        screenFlags |= SDL_FULLSCREEN;
	    
	    SDL_Surface* pScreen = SDL_SetVideoMode(Settings::GetInstance()->GetScreenWidth(),Settings::GetInstance()->GetScreenHeight(), colorDepth, screenFlags);
	    
	    if (pScreen == NULL)
	        cout << "Unable to set screen resolution" << endl;
		else
		{
			SDL_WM_SetCaption("Demeter","testgl");
		    
		    glMatrixMode(GL_PROJECTION);
		    glLoadIdentity();
		    glViewport(0,0,Settings::GetInstance()->GetScreenWidth(), Settings::GetInstance()->GetScreenHeight());
		    glFrustum(-0.5f, 0.5f, -0.5f, 0.5f, 0.65f, MAX_VIEW_DISTANCE);
		    glClearColor(0.5f,0.75f,1.0f,0.0f);
		    glDisable(GL_NORMALIZE);
			//SET CAMERA
		    glMatrixMode( GL_MODELVIEW );
			glLoadIdentity();
			gluLookAt(2000, 2000, 300, 0, 0, 0, 0, 0, 1);
		}
    }
}

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_setMediaPath(JNIEnv* pEnv, jobject object, jstring mediaPath)
{
    const char* szTempMediaPath = pEnv->GetStringUTFChars(mediaPath,0);
    char* szMediaPath = new char[strlen(szTempMediaPath)+1];
    sprintf(szMediaPath, szTempMediaPath);
    
    Settings::GetInstance()->SetMediaPath(szMediaPath);

    delete[] szMediaPath;
    pEnv->ReleaseStringUTFChars(mediaPath,szTempMediaPath);    
}    

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_clearBuffers(JNIEnv* pEnv, jobject object)
{
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
}    

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_swapBuffers(JNIEnv* pEnv, jobject object)
{
    SDL_GL_SwapBuffers();
}

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_render(JNIEnv* pEnv, jobject object, jint terrainIndex)
{
	pTerrains[terrainIndex]->Render();    
}    

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_setCameraPosition(JNIEnv* pEnv,jobject object, jfloat posX, jfloat posY, jfloat posZ, jfloat lookAtX, jfloat lookAtY, jfloat lookAtZ, jfloat lookUpX, jfloat lookUpY, jfloat lookUpZ, jfloat viewPlaneDistance)
{
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();
	gluLookAt(posX, posY, posZ, lookAtX, lookAtY, lookAtZ, lookUpX, lookUpY, lookUpZ);	
	
	for (int i = 0; i < capacityTerrains; i++)
		if (pTerrains[i] != NULL)
			pTerrains[i]->ModelViewMatrixChanged();
}

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_setDetailThreshold(JNIEnv* pEnv, jobject object, jint terrainIndex, jfloat threshold)
{
	pTerrains[terrainIndex]->SetDetailThreshold(threshold);
}

extern "C" JNIEXPORT jfloat JNICALL Java_demeter_Terrain_getElevation(JNIEnv* pEnv, jobject object, jint terrainIndex, jfloat x,jfloat y)
{
	return pTerrains[terrainIndex]->GetElevation(x,y);
}

extern "C" JNIEXPORT void JNICALL Java_demeter_Terrain_getNormal(JNIEnv* pEnv,jobject object,jint terrainIndex,jfloat x,jfloat y,jfloatArray vector3D)
{
	float* pVector3D = pEnv->GetFloatArrayElements(vector3D,0);
	pTerrains[terrainIndex]->GetNormal(x,y,pVector3D[0],pVector3D[1],pVector3D[2]);
	pEnv->ReleaseFloatArrayElements(vector3D,pVector3D,0);
}
