/***************************************************************************
                          cubic.cpp  -  cubic keyframe interpolation
                             -------------------
    begin                : Fri Oct 27 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tgl.h"

namespace TGL{

float tgl_object::bezvalue( float mu, float *coor )
{
	int i;
	int j;

	for (j=0; j<3; j++)
		for (i=0; i<(3-j); i++)
			coor[i] = (1.0-mu)*coor[i] + mu*coor[i+1]; // weighted average

	return coor[0];
}

/* Unoptimized testing code */
float tgl_object::cubic_morph( int component, int from, int to, int ver, float phase )
{
	float c[4]; // current component's position values
	float dist, tanc[3], coord[4], k;
	int iprev, ito, ifrom, inext;
	
	// init
	if( from <= loop_begin ) //from -1 < 0
		iprev = loop_end*numvertices +ver; //last frame+ver
	else
		iprev = numvertices*(from-1) +ver; //from+ver
		
	ifrom = numvertices*from+ver;
		
	if( to > loop_end )
		ito = loop_begin*numvertices +ver; // first frame+ver
	else
		ito = numvertices*to +ver; //to+ver

	if( (to +1) > loop_end )
		inext = loop_begin*numvertices +ver; // firstframe+ver
	else
		inext = numvertices*(to+1)+ver; //to+1+ver
	
	c[0] = vertices[iprev].tex[component]; // prev node	
	c[1] = vertices[ifrom].tex[component]; // from node
	c[2] = vertices[ito].tex[component]; // to node
	c[3] = vertices[inext].tex[component]; // next node

	// calculate pos
	k = c[2]-c[0];
	coord[1] = c[1]+k*cubic_strength;
	k = c[1]-c[3];
	coord[2] = c[2]+k*cubic_strength;
	
	coord[0] = c[1];
	coord[3] = c[2];
	
	return( bezvalue( phase, coord ) );
}
	
}