/***************************************************************************
                          pcx.cpp  -  pcx loader
                             -------------------
    begin                : Tue Jul 18 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tgl.h"

//using namespace TGL;

namespace TGL{

unsigned char* pcx_load( const char *filename, int *pw, int *ph )
{
	FILE *pcx;
	PCX_HEADER pcxh;
	int width, height, bpl, x, y, i;
	unsigned char byte1, byte2, *buf, *pal;

	pcx = fopen( filename, "rb" );
	if( pcx == NULL )
	{
//		tgl_file_not_found( "pcx_load", filename );

		tgl_msg( "pcx_load: file %s not found", TGL_WARNING, (void*)filename );
		return NULL;
	}

	// Verify header
	fread( &pcxh, sizeof( PCX_HEADER ), 1, pcx );
	if( pcxh.manufacturer!=10 || pcxh.version!=5 || pcxh.encoding!=1 ||
		pcxh.bits!=8 || pcxh.color_planes!=1 )
	{
		tgl_msg( "pcx_load: unsupported format, only 256 color pcx supported (%s)", TGL_WARNING, (void*)filename );
		return NULL;
	}

	width = pcxh.width - pcxh.x + 1;
	height = pcxh.height - pcxh.y + 1;
	bpl = pcxh.bytes_per_line;

	if( pw != NULL )
	{
		*pw = width;
		*ph = height;
	}

	buf = (unsigned char*)malloc( width*height*3 );
	if( buf == NULL )
	{
		tgl_msg( "pcx_load: out of mem", TGL_WARNING );
		return NULL;
	}
	memset( (unsigned char*)buf, 0, width*height*3 );
	pal = (unsigned char*)malloc( 768 );
	if( pal == NULL )
	{
		tgl_msg( "pcx_load: out of mem", TGL_WARNING );
 		tgl_free( "pcx_load", buf );
		return NULL;
	}


	// read palette
	fseek( pcx, -768, SEEK_END );
	for( i=0; i<768; i++ )
		pal[i] = fgetc( pcx );


	// Unpack data
	fseek( pcx, 128, SEEK_SET );
	for( y=0; y<height; y++ )
	{
		x = 0;
		while( x<bpl )
		{
			byte1 = fgetc( pcx );
			if( byte1>192 ) // RLE
			{
				byte2 = fgetc( pcx );
				for( ; byte1>192; byte1--)
				{
					// convert to RGB
					buf[y*width*3+x*3] = pal[byte2*3];
					buf[y*width*3+x*3+1] = pal[byte2*3+1];
					buf[y*width*3+x*3+2] = pal[byte2*3+2];
					x++;
				}
			}
			else // unpacked
			{
				buf[y*width*3+x*3] = pal[byte1*3];
				buf[y*width*3+x*3+1] = pal[byte1*3+1];
				buf[y*width*3+x*3+2] = pal[byte1*3+2];
				x++;
			}
		}
	}

	free( pal );
	fclose( pcx );

	return buf;
}

}