/***************************************************************************
                          renderer.cpp  -  rendering code
                             -------------------
    begin                : Sun Feb 11 2001
    copyright            : (C) 2001 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tgl.h"
#include "tgl_object.h"

//using namespace TGL;

namespace TGL{

// render object
GLint tgl_object::draw()
{
	switch( tgl_object_rendermode )
	{
		case TGL_ELEMENTS: draw_elements(); break;
		case TGL_ELEMENT:  draw_element(); break;
		case TGL_TRIANGLES: draw_triangles(); break;
		default: draw_triangles(); break;
	}
}

// elements method
GLint tgl_object::draw_elements()
{
	int srcbl, dstbl, locked;
	int lit;
	
	// disable lighting if necessary
	lit=lighting;
	if( tgl_disable_lighting ) lit=0;
	
	// these must be set BEFORE glInterleavedArrays
	if( texturing )
	{
		glBindTexture( GL_TEXTURE_2D, tgl_texturelist[(textures[currframenum])].texturenum );
		glEnable( GL_TEXTURE_2D );
	} else glDisable( GL_TEXTURE_2D );

	// coloring flag
	if( coloring )
	{
		if( lit || texturing );
			glEnable( GL_COLOR_MATERIAL );
	}

	// lighting flag handler
	if( lit == 1 || lit == 2 )
		glShadeModel( GL_FLAT );
	else
		glShadeModel( GL_SMOOTH );

	if( lit == 0 || lit == 1 || lit == 3 )
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 0 );
	else
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 1 );

	if( lit && !tgl_disable_lighting )
		glEnable( GL_LIGHTING );
	else
		glDisable( GL_LIGHTING );



	//hidden face removal, back face culling etc..
	if( hfre )
	{
		glEnable( GL_CULL_FACE );
	}	else
	{
		glDisable( GL_CULL_FACE );
	}

	// blending
	if( src_blendfunc && dest_blendfunc )
	{
		switch( src_blendfunc ) {
			case 1: srcbl = GL_ZERO; break;
			case 2: srcbl = GL_ONE; break;
			case 3: srcbl = GL_DST_COLOR; break;
			case 4: srcbl = GL_DST_ALPHA; break;
			case 5: srcbl = GL_SRC_ALPHA; break;
			case 6: srcbl = GL_ONE_MINUS_DST_COLOR; break;
			case 7: srcbl = GL_ONE_MINUS_DST_ALPHA; break;
			case 8: srcbl = GL_ONE_MINUS_SRC_ALPHA; break;
			default: srcbl = GL_ONE; // unsupported
		}

		switch( dest_blendfunc ) {
			case 1: dstbl = GL_ZERO; break;
			case 2: dstbl = GL_ONE; break;
			case 3: dstbl = GL_SRC_COLOR; break;
			case 4: dstbl = GL_SRC_ALPHA; break;
			case 5: dstbl = GL_DST_ALPHA; break;
			case 6: dstbl = GL_ONE_MINUS_SRC_COLOR; break;
			case 7: dstbl = GL_ONE_MINUS_SRC_ALPHA; break;
			case 8: dstbl = GL_ONE_MINUS_DST_ALPHA; break;
			default: dstbl = GL_ONE; // unsupported
		}

		glBlendFunc( srcbl, dstbl );
		glEnable( GL_BLEND );
	} else
		glDisable( GL_BLEND );

	// set material
	glMaterialfv( GL_FRONT_AND_BACK, GL_SHININESS, &material.shiness );
	glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, material.specular );
	glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, material.diffuse );
	glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, material.ambient );
	glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, material.emission );

	// morph
	if( do_morph ) calc_currframe();
	do_morph = 0;

	// set vertexpointer
	glInterleavedArrays( GL_T2F_C4F_N3F_V3F, 0, currframe );


	// these must be disabled AFTER glInterleavedArrays:

	if( !texturing )
	{
		#ifndef DRI_BUG_WORKAROUND
		glDisable( GL_TEXTURE_2D );
		glDisableClientState( GL_TEXTURE_COORD_ARRAY );
		#else
		glBindTexture( GL_TEXTURE_2D, tgl_texturelist[(textures[currframenum])].texturenum );
		#endif
	} else
	{
		glEnable( GL_TEXTURE_2D );
		glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	}

	// lighting flag handler
	#ifndef DRI_BUG_WORKAROUND
	if( !lit && !hfre )
		glDisableClientState( GL_NORMAL_ARRAY );
	else
		glEnableClientState( GL_NORMAL_ARRAY );
	#endif


	// coloring flag
	if( !coloring )
	{
		#ifndef DRI_BUG_WORKAROUND
		glDisableClientState( GL_COLOR_ARRAY );
		#endif
		glDisable( GL_COLOR_MATERIAL );
		glColor4f( 1.0, 1.0, 1.0, 1.0 );
	} else
	{
		glEnableClientState( GL_COLOR_ARRAY );
		glEnable( GL_COLOR_MATERIAL );
	}

	// render:
	// lock arrays
	if( tgl_lockarrays_ext_available==true && numvertices < tgl_max_lock_count ) // don't try to lock if numvertices exceeds maximum lock count
	{
		glLockArraysEXT_p( 0, numvertices );

		#ifdef GL_ARRAY_ELEMENT_LOCK_COUNT_EXT
		glGetIntegerv( GL_ARRAY_ELEMENT_LOCK_COUNT_EXT, &locked ); // how many locked vertices
		if( locked != numvertices )
		{
			tgl_msg( "Can't lock %d vertices, setting new lock count limit", TGL_WARNING, (void*)numvertices );
			tgl_max_lock_count = numvertices; // set new limit
		}
		#else
		locked=numvertices;
		#endif
	}

	// draw it!
	glDrawElements( GL_TRIANGLES, numindices, GL_UNSIGNED_INT, indices ); // draw object

	// unclock arrays
	if( tgl_lockarrays_ext_available==true && locked==numvertices ) // don't try to unlock if its not locked
		glUnlockArraysEXT_p();


	// undo transformations
	if( matpushed && matpushing )
	{
		glPopMatrix();
		matpushed = 0;
	}

	return 0;
}


// render element
GLint tgl_object::draw_element()
{
	int srcbl, dstbl, i, ii, locked;
	int lit;
	
	// disable lighting if necessary
	lit=lighting;
	if( tgl_disable_lighting ) lit=0;

	// these must be set BEFORE glInterleavedArrays
	if( texturing )
	{
		glBindTexture( GL_TEXTURE_2D, tgl_texturelist[(textures[currframenum])].texturenum );
		glEnable( GL_TEXTURE_2D );
	}

	// coloring flag
	if( coloring )
	{
		if( lit || texturing );
			glEnable( GL_COLOR_MATERIAL );
	}

	// lighting flag handler
	if( lit == 1 || lit == 2 )
		glShadeModel( GL_FLAT );
	else
		glShadeModel( GL_SMOOTH );

	if( lit == 0 || lit == 1 || lit == 3 )
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 0 );
	else
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 1 );
		
	if( lit )
		glEnable( GL_LIGHTING );
	else
		glDisable( GL_LIGHTING );



	//hidden face removal, back face culling etc..
	if( hfre )
	{
		glEnable( GL_CULL_FACE );	
	}	else
	{
		glDisable( GL_CULL_FACE );
	}

	// blending
	if( src_blendfunc && dest_blendfunc )
	{
		switch( src_blendfunc ) {
			case 1: srcbl = GL_ZERO; break;
			case 2: srcbl = GL_ONE; break;
			case 3: srcbl = GL_DST_COLOR; break;
			case 4: srcbl = GL_DST_ALPHA; break;
			case 5: srcbl = GL_SRC_ALPHA; break;
			case 6: srcbl = GL_ONE_MINUS_DST_COLOR; break;
			case 7: srcbl = GL_ONE_MINUS_DST_ALPHA; break;
			case 8: srcbl = GL_ONE_MINUS_SRC_ALPHA; break;
			default: srcbl = GL_ONE; // unsupported
		}

		switch( dest_blendfunc ) {
			case 1: dstbl = GL_ZERO; break;
			case 2: dstbl = GL_ONE; break;
			case 3: dstbl = GL_SRC_COLOR; break;
			case 4: dstbl = GL_SRC_ALPHA; break;
			case 5: dstbl = GL_DST_ALPHA; break;
			case 6: dstbl = GL_ONE_MINUS_SRC_COLOR; break;
			case 7: dstbl = GL_ONE_MINUS_SRC_ALPHA; break;
			case 8: dstbl = GL_ONE_MINUS_DST_ALPHA; break;
			default: dstbl = GL_ONE; // unsupported
		}
				
		glBlendFunc( srcbl, dstbl );
		glEnable( GL_BLEND );
	} else
		glDisable( GL_BLEND );

	// set material		
	glMaterialfv( GL_FRONT_AND_BACK, GL_SHININESS, &material.shiness );
	glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, material.specular );
	glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, material.diffuse );
	glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, material.ambient );
	glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, material.emission );

	// morph
	if( do_morph ) calc_currframe();
	do_morph = 0;
	
	// set vertexpointer
	glVertexPointer( 3, GL_FLOAT, TVERTEX_SIZE, (char*)((char*)currframe+POS_OFFSET) );
//	glNormalPointer( GL_FLOAT, TVERTEX_SIZE, (char*)((char*)currframe+VEC_OFFSET) );
//	glColorPointer( 4, GL_FLOAT, TVERTEX_SIZE, (char*)((char*)currframe+COL_OFFSET) );
//	glTexCoordPointer( 2, GL_FLOAT, TVERTEX_SIZE, (char*)((char*)currframe+TEX_OFFSET) );
	glEnableClientState( GL_VERTEX_ARRAY );
//	glEnableClientState( GL_COLOR_ARRAY );
//	glEnableClientState( GL_NORMAL_ARRAY );
//	glEnableClientState( GL_TEXTURE_COORD_ARRAY );
	glDisableClientState( GL_NORMAL_ARRAY );
	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
	glDisableClientState( GL_COLOR_ARRAY );

//	glInterleavedArrays( GL_T2F_C4F_N3F_V3F, 0, currframe );


	// these must be disabled AFTER glInterleavedArrays:
	
	if( !texturing )
	{
		glDisable( GL_TEXTURE_2D );
	} else
	{
		glEnable( GL_TEXTURE_2D );
	}

	// coloring flag
	if( !coloring )
	{
		glDisable( GL_COLOR_MATERIAL );
		glColor4f( 1.0, 1.0, 1.0, 1.0 );
	} else
	{
		glEnable( GL_COLOR_MATERIAL );
	}

	// render
	// lock arrays
	if( tgl_lockarrays_ext_available==true && numvertices < tgl_max_lock_count ) // don't try to lock if numvertices exceeds maximum lock count
	{
		glLockArraysEXT_p( 0, numvertices );

		#ifdef GL_ARRAY_ELEMENT_LOCK_COUNT_EXT
		glGetIntegerv( GL_ARRAY_ELEMENT_LOCK_COUNT_EXT, &locked ); // how many locked vertices
		if( locked != numvertices )
		{
			tgl_msg( "Can't lock %d vertices, setting new lock count limit", TGL_WARNING, (void*)numvertices );
			tgl_max_lock_count = numvertices; // set new limit
		}
		#else
		locked=numvertices;
		#endif
	}
	
  for( i=0; i<numindices; i+=3 )
	{
		int ci; // currindex
		float a,b,c,d;
		
		glBegin( GL_TRIANGLES );
		
		for( ii=0; ii<3; ii++ ) // 3 vertices/poly
		{
			ci = indices[i+ii];

			a = currframe[ci].tex[0];
			b = currframe[ci].tex[1];
			glTexCoord2f( a, b );

			a = currframe[ci].col[0];
			b = currframe[ci].col[1];
			c = currframe[ci].col[2];
			d = currframe[ci].col[3];
			glColor4f( a, b, c, d );
						
			a = currframe[ci].vec[0];
			b = currframe[ci].vec[1];
			c = currframe[ci].vec[2];
			glNormal3f( a, b, c );
			
			glArrayElement( ci );
		}
						
		glEnd();
	}
	
	
	if( tgl_lockarrays_ext_available==true && locked==numvertices ) // don't try to unlock if its not locked
		glUnlockArraysEXT_p();
	

	// undo transformations
	if( matpushed && matpushing )
	{
		glPopMatrix();
		matpushed = 0;
	}

	return 0;
}

// render triangles
GLint tgl_object::draw_triangles()
{
	int srcbl, dstbl, i, ii;
	int lit;
	
	// disable lighting if necessary
	lit=lighting;
	if( tgl_disable_lighting ) lit=0;

	// these must be set BEFORE glInterleavedArrays
	if( texturing )
	{
		glBindTexture( GL_TEXTURE_2D, tgl_texturelist[(textures[currframenum])].texturenum );
		glEnable( GL_TEXTURE_2D );
	}

	// coloring flag
	if( coloring )
	{
		if( lit || texturing );
			glEnable( GL_COLOR_MATERIAL );
	}

	// lighting flag handler
	if( lit == 1 || lit == 2 )
		glShadeModel( GL_FLAT );
	else
		glShadeModel( GL_SMOOTH );

	if( lit == 0 || lit == 1 || lit == 3 )
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 0 );
	else
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 1 );
		
	if( lit )
		glEnable( GL_LIGHTING );
	else
		glDisable( GL_LIGHTING );



	//hidden face removal, back face culling etc..
	if( hfre )
	{
		glEnable( GL_CULL_FACE );	
	}	else
	{
		glDisable( GL_CULL_FACE );
	}

	// blending
	if( src_blendfunc && dest_blendfunc )
	{
		switch( src_blendfunc ) {
			case 1: srcbl = GL_ZERO; break;
			case 2: srcbl = GL_ONE; break;
			case 3: srcbl = GL_DST_COLOR; break;
			case 4: srcbl = GL_DST_ALPHA; break;
			case 5: srcbl = GL_SRC_ALPHA; break;
			case 6: srcbl = GL_ONE_MINUS_DST_COLOR; break;
			case 7: srcbl = GL_ONE_MINUS_DST_ALPHA; break;
			case 8: srcbl = GL_ONE_MINUS_SRC_ALPHA; break;
			default: srcbl = GL_ONE; // unsupported
		}

		switch( dest_blendfunc ) {
			case 1: dstbl = GL_ZERO; break;
			case 2: dstbl = GL_ONE; break;
			case 3: dstbl = GL_SRC_COLOR; break;
			case 4: dstbl = GL_SRC_ALPHA; break;
			case 5: dstbl = GL_DST_ALPHA; break;
			case 6: dstbl = GL_ONE_MINUS_SRC_COLOR; break;
			case 7: dstbl = GL_ONE_MINUS_SRC_ALPHA; break;
			case 8: dstbl = GL_ONE_MINUS_DST_ALPHA; break;
			default: dstbl = GL_ONE; // unsupported
		}

		glBlendFunc( srcbl, dstbl );
		glEnable( GL_BLEND );
	} else
		glDisable( GL_BLEND );

	// set material		
	glMaterialfv( GL_FRONT_AND_BACK, GL_SHININESS, &material.shiness );
	glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, material.specular );
	glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, material.diffuse );
	glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, material.ambient );
	glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, material.emission );

	// morph
	if( do_morph ) calc_currframe();
	do_morph = 0;
	
	// set vertexpointer
/*	glVertexPointer( 3, GL_FLOAT, TVERTEX_SIZE, (char*)((char*)currframe+POS_OFFSET) );
	glNormalPointer( GL_FLOAT, TVERTEX_SIZE, (char*)((char*)currframe+VEC_OFFSET) );
	glColorPointer( 4, GL_FLOAT, TVERTEX_SIZE, (char*)((char*)currframe+COL_OFFSET) );
	glTexCoordPointer( 2, GL_FLOAT, TVERTEX_SIZE, (char*)((char*)currframe+TEX_OFFSET) );
	glEnableClientState( GL_VERTEX_ARRAY );
	glEnableClientState( GL_COLOR_ARRAY );
	glEnableClientState( GL_NORMAL_ARRAY );
	glEnableClientState( GL_TEXTURE_COORD_ARRAY );*/
//	glDisableClientState( GL_NORMAL_ARRAY );
//	glDisableClientState( GL_TEXTURE_COORD_ARRAY );
//	glDisableClientState( GL_COLOR_ARRAY );

//	glInterleavedArrays( GL_T2F_C4F_N3F_V3F, 0, currframe );


	// these must be disabled AFTER glInterleavedArrays:
	
	if( !texturing )
	{
		glDisable( GL_TEXTURE_2D );
	} else
	{
		glEnable( GL_TEXTURE_2D );
	}

	// coloring flag
	if( !coloring )
	{
		glDisable( GL_COLOR_MATERIAL );
		glColor4f( 1.0, 1.0, 1.0, 1.0 );
	} else
	{
		glEnable( GL_COLOR_MATERIAL );
	}

	// render
  for( i=0; i<numindices; i+=3 )
	{
		int ci; // currindex
		float a,b,c,d;

		glBegin( GL_TRIANGLES );

		for( ii=0; ii<3; ii++ ) // 3 vertices/poly
		{
			ci = indices[i+ii];

			a = currframe[ci].tex[0];
			b = currframe[ci].tex[1];
			glTexCoord2f( a, b );
			
			a = currframe[ci].col[0];
			b = currframe[ci].col[1];
			c = currframe[ci].col[2];
			d = currframe[ci].col[3];
			glColor4f( a, b, c, d );
						
			a = currframe[ci].vec[0];
			b = currframe[ci].vec[1];
			c = currframe[ci].vec[2];
			glNormal3f( a, b, c );
			
			a = currframe[ci].pos[0];
			b = currframe[ci].pos[1];
			c = currframe[ci].pos[2];
			glVertex3f( a, b, c );
		}
						
		glEnd();
	}

	// undo transformations
	if( matpushed && matpushing )
	{
		glPopMatrix();
		matpushed = 0;
	}

	return 0;
}

} //namespace

