/***************************************************************************
                          t3dmodel101.cpp  -  t3dmodel 1.01 load/save code
                             -------------------
    begin                : Sun Oct 29 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

//#include "t3dmodel101.h"
#include "tgl.h"

namespace TGL{

// clamp value to [0...1]

float clamp_min = 0.0, clamp_max = 1.0;

float clampf( float x )
{
	if( x < clamp_min ) return clamp_min;
	if( x > clamp_max ) return clamp_max;
	return x;
}


/* loader for T3Dmodel ver 1.01 */
GLint tgl_object::load_t3dmodel_101( FILE *f, const char *fname )
{
	char idstr[10], mname[64], texname[64], str[160];
	int ver, verts, tris, frames;
	int i;

	// begin of file
	fseek( f, 0, SEEK_SET );


	//parse header
	if( fscanf( f, "%s %d %d %d %d %s", &idstr, &ver, &verts, &frames, &tris, &mname ) !=6 )
		tgl_msg( "load_t3dmodel_101: corrupted file (%s)", TGL_ERROR, (void*)fname );


	//parse texture names
	textures = (GLuint*)tgl_alloc( "load_t3dmodel_101", frames*sizeof(GLuint) );
	texturing = 0;
	for( i=0; i<frames; i++ )
	{
		if( fscanf( f, "%s", &texname ) !=1 )
			tgl_msg( "load_t3dmodel_101: corrupted file (%s)", TGL_ERROR, (void*)fname );

		if( strcasecmp( (const char*)texname, "notexture" ) )
			textures[i] = load_texture( tgl_parse_path(fname)+string(texname), texname ); // load texture
//			textures[i] = load_texture( string(tgl_parse_path(fname)+string(texname)).c_str() ); // load texture
		else
			textures[i] = load_texture( "notexture", "notexture" ); // "load" notexture
//			textures[i] = load_texture( (char*)texname ); // "load" notexture

		if( strcasecmp( (const char*)texname, "notexture" ) )
			texturing = 1; // enable texturing

		if( tgl_debug>=1 )
			fprintf( stderr, "texname=%s textures[%d]=%d\n", texname, i, textures[i] );
	}


	//parse flags
	if( fscanf( f, "%d %d %d %d %d  %d %d %d %d %f", &coloring, &hfre, &src_blendfunc, &dest_blendfunc, &lighting, &pos_morph, &vec_morph, &col_morph, &tex_morph, &cubic_strength ) !=10 )
		tgl_msg( "load_t3dmodel_101: corrupted file (%s)", TGL_ERROR, (void*)fname );

	//read material
	if( fscanf( f, "%f  %f %f %f %f  %f %f %f %f  %f %f %f %f  %f %f %f %f",
		&material.shiness,
		&material.specular[0], &material.specular[1], &material.specular[2], &material.specular[3],
		&material.diffuse[0], &material.diffuse[1], &material.diffuse[2], &material.diffuse[3],
		&material.ambient[0], &material.ambient[1], &material.ambient[2], &material.ambient[3],
		&material.emission[0], &material.emission[1], &material.emission[2], &material.emission[3] ) != 17 )
	{
		tgl_msg( "load_t3dmodel_101: error reading file (%s)", TGL_ERROR, (void*)fname );
		fclose( f );
		return -2;
	}

		
	// update object data
	if( ver != 101 ) tgl_msg( "load_t3dmodel_101: unsupported file version (%s)", TGL_ERROR, (void*)fname );
	numindices = 3*tris;
	numvertices = verts;
	numframes = frames;
	vertices = (TVERTEX*)tgl_alloc( "load_t3dmodel_101", numvertices*sizeof(TVERTEX)*numframes );
	indices = (GLuint*)tgl_alloc( "load_t3dmodel_101", numindices*sizeof(GLuint) );
//	currframe = vertices;
	strcpy( modelname, (const char*)mname );


	// print debug information
	if( tgl_debug>=1 )	fprintf( stderr, "type=%s ver=%d verts=%d tris=%d frames=%d name=%s\n"
		"%d %d %d %d %d\n", idstr, ver, verts, tris, frames, mname,
		 coloring, hfre, src_blendfunc, dest_blendfunc, lighting );


	// read vertices	
	for( i=0; i<(numvertices*frames); i++ )
	{
		if( fscanf( f, "%f %f %f  %f %f %f %f  %f %f",
			&vertices[i].pos[0],
			&vertices[i].pos[1],
			&vertices[i].pos[2],
			&vertices[i].col[0],
			&vertices[i].col[1],
			&vertices[i].col[2],
			&vertices[i].col[3],
			&vertices[i].tex[0],
			&vertices[i].tex[1] ) !=9 )
				tgl_msg( "load_t3dmodel_101: corrupted file '%s', can't read vertices ", TGL_ERROR, (void*)fname );

			//print debug information
			if( tgl_debug>=2 )	fprintf( stderr, "vert %d: x=%.3f y=%.3f z=%.3f  r=%.2f g=%.2f b=%.2f a=%.2f  u=%.2f v=%.2f\n", i,
				vertices[i].pos[0],
				vertices[i].pos[1],
				vertices[i].pos[2],
				vertices[i].col[0],
				vertices[i].col[1],
				vertices[i].col[2],
				vertices[i].col[3],
				vertices[i].tex[0],
				vertices[i].tex[1] );
	}

	// read triangle indices
	for( i=0; i<(tris); i++ )
	{
		if( fscanf( f, "%d %d %d",
			&indices[i*3],
			&indices[i*3+1],
			&indices[i*3+2] ) !=3 )
				tgl_msg( "load_t3dmodel_101: corrupted file '%s', can't read triangles", TGL_ERROR, (void*)fname );

			//print debug information
			if( tgl_debug>=2 )	fprintf( stderr, "triangle %d: %d %d %d\n", i,
				indices[i*3],
				indices[i*3+1],
				indices[i*3+2] );

	}

	sprintf( str, "T3Dmodel loaded (name=%s vertices=%d faces=%d frames=%d)", modelname, numvertices, numindices/3, numframes );
	tgl_msg( str, TGL_INFO );
}



/* save T3Dmodel v 1.01 */
GLint tgl_object::save_t3dmodel_101( const char *fname )
{
	int i;
	FILE *f;
	char str[160];


	f = fopen( fname, "wb" );
	if( f == NULL )
	{
		tgl_msg( "save_t3dmodel_101: can't write to file %s", TGL_WARNING, (void*)fname );
		return -1;
	}


	//write header
	if( fprintf( f, "%s %d %d %d %d %s\n\n", "T3Dmodel", 101, numvertices, numframes, numindices/3, modelname ) == 0 )
	{
		tgl_msg( "save_t3dmodel_101: error saving file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}

	//texture names
	for( i=0; i<numframes; i++ )
	{
		if( fprintf( f, "%s\n", tgl_texturelist[textures[i]].relative_filename->c_str() ) == 0 )
		{
			tgl_msg( "save_t3dmodel_101: error saving file (%s)", TGL_WARNING, (void*)fname );
			fclose( f );
			return -2;
		}
	}
	fprintf( f, "\n" );

	//write flags
	if( fprintf( f, "%d %d %d %d %d  %d %d %d %d %.3f\n", coloring, hfre, src_blendfunc, dest_blendfunc, lighting, pos_morph, vec_morph, col_morph, tex_morph, cubic_strength ) == 0 )
	{
		tgl_msg( "save_t3dmodel_101: error saving file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}

	//write material
	if( fprintf( f, "%.3f  %.3f %.3f %.3f %.3f  %.3f %.3f %.3f %.3f  %.3f %.3f %.3f %.3f  %.3f %.3f %.3f %.3f\n",
		material.shiness,
		material.specular[0], material.specular[1], material.specular[2], material.specular[3],
		material.diffuse[0], material.diffuse[1], material.diffuse[2], material.diffuse[3],
		material.ambient[0], material.ambient[1], material.ambient[2], material.ambient[3],
		material.emission[0], material.emission[1], material.emission[2], material.emission[3] ) == 0 )
	{
		tgl_msg( "save_t3dmodel_101: error saving file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}
	
	// write vertices	
	for( i=0; i<(numvertices*numframes); i++ )
	{
		if( !(i%numvertices) ) fprintf( f, "\n" ); // separate frames
		if( fprintf( f, "%.3f %.3f %.3f  %.3f %.3f %.3f %.3f  %.3f %.3f\n",
			vertices[i].pos[0],
			vertices[i].pos[1],
			vertices[i].pos[2],
			vertices[i].col[0],
			vertices[i].col[1],
			vertices[i].col[2],
			vertices[i].col[3],
			vertices[i].tex[0],
			vertices[i].tex[1] ) == 0 )
		{
			tgl_msg( "save_t3dmodel_101: error saving file (%s)", TGL_WARNING, (void*)fname );
			fclose( f );
			return -2;
		}
	}
	fprintf( f, "\n" );

	// write triangle indices
	for( i=0; i<(numindices/3); i++ )
	{
		if( fprintf( f, "%d %d %d\n",
			indices[i*3],
			indices[i*3+1],
			indices[i*3+2] ) == 0 )
			{
				tgl_msg( "save_t3dmodel_101: error saving file (%s)", TGL_WARNING, (void*)fname );
				fclose( f );
				return -2;
			}
	}
	fprintf( f, "\n" );

	fclose( f );	

	sprintf( str, "T3Dmodel saved (name=%s vertices=%d faces=%d frames=%d)", modelname, numvertices, numindices/3, numframes );
	tgl_msg( str, TGL_INFO );
}

/* save T3DModelBin v 1.01 */
GLint tgl_object::save_t3dmodelbin_101( const char *fname )
{
	int i, n, ret;
	unsigned char morphmode=0;
	FILE *f;
	char str[160];


	f = fopen( fname, "wb" );
	if( f == NULL )
	{
		tgl_msg( "save_t3dmodelbin_101: can't write to file %s", TGL_WARNING, (void*)fname );
		return -1;
	}

	//write header
	ret = 0;
	ret += fwrite( "T3DModelBin", 11, 1, f ); // id string
	n=101; ret += fwrite( &n, 4, 1, f ); // version
	n=100; ret += fwrite( &n, 4, 1, f ); // required loader version
	ret += fwrite( &numvertices, 4, 1, f ); // amount of vertices
	ret += fwrite( &numframes, 4, 1, f ); // frames
	n=numindices/3; ret += fwrite( &n, 4, 1, f ); // triangles
	n=strlen( modelname ); ret += fwrite( &n, 4, 1, f ); // model name len
	ret += fwrite( modelname, n, 1, f ); // model name str

  if( ret != 8 )
	{
		tgl_msg( "save_t3dmodelbin_101: error saving file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}

	//write flags
	ret = 0;
	ret += fwrite( &coloring, 4, 1, f ); // coloring
	ret += fwrite( &hfre, 4, 1, f ); // back face culling
	ret += fwrite( &src_blendfunc, 4, 1, f ); // src blendf
	ret += fwrite( &dest_blendfunc, 4, 1, f ); // dst blendf
	ret += fwrite( &lighting, 4, 1, f ); // lighting

	/* use the reserved area: */	
	
	// morphmode
	morphmode = tex_morph +((col_morph<<2)&0x0c) +((vec_morph<<4)&0x30) +((pos_morph<<6)&0xc0);
	fputc( morphmode, f ); // morphing modes (2 bits each)
	ret += fwrite( &cubic_strength, 4, 1, f ); // cubic strength
	
	// materials
	clamp_min = 0.0; clamp_max = 127.0;
	fputc( int(clampf(material.shiness)*2.0), f ); // shiness
	clamp_min = -1.0; clamp_max = 1.0;
	fputc( int(clampf(material.specular[0])*127.0+127), f ); // specular
	fputc( int(clampf(material.specular[1])*127.0+127), f );
	fputc( int(clampf(material.specular[2])*127.0+127), f );
	fputc( int(clampf(material.specular[3])*127.0+127), f );
	fputc( int(clampf(material.diffuse[0])*127.0+127), f ); // diffuse
	fputc( int(clampf(material.diffuse[1])*127.0+127), f );
	fputc( int(clampf(material.diffuse[2])*127.0+127), f );
	fputc( int(clampf(material.diffuse[3])*127.0+127), f );
	fputc( int(clampf(material.ambient[0])*127.0+127), f ); // ambient
	fputc( int(clampf(material.ambient[1])*127.0+127), f );
	fputc( int(clampf(material.ambient[2])*127.0+127), f );
	fputc( int(clampf(material.ambient[3])*127.0+127), f );
	fputc( int(clampf(material.emission[0])*127.0+127), f ); // emission
	fputc( int(clampf(material.emission[1])*127.0+127), f );
	fputc( int(clampf(material.emission[2])*127.0+127), f );
	fputc( int(clampf(material.emission[3])*127.0+127), f );
	
	for( i=0; i<(64-22); i++ )	// 64-22 bytes left for future use
		fputc( 0, f );

	if( ret != 6 )
	{
		tgl_msg( "save_t3dmodelbin_101: error saving file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}

	//texture names
	for( i=0; i<numframes; i++ )
	{
		ret = 0;
//		string relname;
//		if( rel
		n=strlen( tgl_texturelist[textures[i]].relative_filename->c_str() ); ret += fwrite( &n, 4, 1, f ); //
		ret += fwrite( tgl_texturelist[textures[i]].relative_filename->c_str(), n, 1, f ); //

		if( ret != 2 )
		{
			tgl_msg( "save_t3dmodel_101bin: error saving file (%s)", TGL_WARNING, (void*)fname );
			fclose( f );
			return -2;
		}
	}


//	ret = fwrite( vertices, TVERTEXSIZE, numvertices*numframes, f ); // all vertices
	// write vertices	
	for( i=0; i<(numvertices*numframes); i++ )
	{
		ret = 0;

		// position (float
		ret += fwrite( &vertices[i].pos, sizeof(float), 3, f );
	
		// color (char, 1.0=255)
		n = vertices[i].col[0]*255; ret += fwrite( &n, sizeof(char), 1, f );
		n = vertices[i].col[1]*255; ret += fwrite( &n, sizeof(char), 1, f );
		n = vertices[i].col[2]*255; ret += fwrite( &n, sizeof(char), 1, f );
		n = vertices[i].col[3]*255; ret += fwrite( &n, sizeof(char), 1, f );

		// tex uv (float)
		ret += fwrite( &vertices[i].tex, sizeof(float), 2, f );

		if( ret != 9 )
		{
			tgl_msg( "save_t3dmodelbin_101: error saving file (%s)", TGL_WARNING, (void*)fname );
			fclose( f );
			return -2;
		}
	}



	// write triangle indices
	ret = fwrite( indices, sizeof(int), numindices, f ); // all indices
	
	if( ret != numindices )
	{
		tgl_msg( "save_t3dmodelbin_101: error saving file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}

	fclose( f );	

	sprintf( str, "T3DModelBin saved (name=%s vertices=%d faces=%d frames=%d)", modelname, numvertices, numindices/3, numframes );
	tgl_msg( str, TGL_INFO );
}


/* loader for T3DModelBin v 1.01 */
GLint tgl_object::load_t3dmodelbin_101( FILE *f, const char *fname )
{
	int i, n, ret;
	char str[300];
	unsigned char morphmode;

	//seek pos after idstr+ver
	fseek( f, 19, SEEK_SET );
	

	//read header
	ret = 0;
	ret += fread( &numvertices, 4, 1, f ); // numverts
	ret += fread( &numframes, 4, 1, f ); // numframes
	ret += fread( &n, 4, 1, f ); numindices = n*3; // numindices
	ret += fread( &n, 4, 1, f ); // chars
	ret += fread( modelname, n, 1, f ); // str
  modelname[n] = 0; // end of string

  if( ret != 5 )
	{
		tgl_msg( "load_t3dmodelbin_101: corrupted file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}

	//read flags
	ret = 0;
	ret += fread( &coloring, 4, 1, f ); // coloring
	ret += fread( &hfre, 4, 1, f ); // hfre
	ret += fread( &src_blendfunc, 4, 1, f ); // src blend
	ret += fread( &dest_blendfunc, 4, 1, f ); // dest blend
	ret += fread( &lighting, 4, 1, f ); // litmode
	
	// extract morping modes:
	morphmode = fgetc(f); // read morphing modes (2 bits each)
	pos_morph = (morphmode>>6)&0x03;
	vec_morph = (morphmode>>4)&0x03;
	col_morph = (morphmode>>2)&0x03;
	tex_morph = morphmode&0x03;
	ret += fread( &cubic_strength, 4, 1, f ); // cubic strength
	// read material
	material.shiness = float(fgetc(f)) /2.0;
	material.specular[0] = (float(fgetc(f)) -127.0) /127.0;
	material.specular[1] = (float(fgetc(f)) -127.0) /127.0;
	material.specular[2] = (float(fgetc(f)) -127.0) /127.0;
	material.specular[3] = (float(fgetc(f)) -127.0) /127.0;
	material.diffuse[0] = (float(fgetc(f)) -127.0) /127.0;
	material.diffuse[1] = (float(fgetc(f)) -127.0) /127.0;
	material.diffuse[2] = (float(fgetc(f)) -127.0) /127.0;
	material.diffuse[3] = (float(fgetc(f)) -127.0) /127.0;
	material.ambient[0] = (float(fgetc(f)) -127.0) /127.0;
	material.ambient[1] = (float(fgetc(f)) -127.0) /127.0;
	material.ambient[2] = (float(fgetc(f)) -127.0) /127.0;
	material.ambient[3] = (float(fgetc(f)) -127.0) /127.0;
	material.emission[0] = (float(fgetc(f)) -127.0) /127.0;
	material.emission[1] = (float(fgetc(f)) -127.0) /127.0;
	material.emission[2] = (float(fgetc(f)) -127.0) /127.0;
	material.emission[3] = (float(fgetc(f)) -127.0) /127.0;
	
	fseek( f, 64-22, SEEK_CUR ); // skip 64-22 reserved bytes
	
	if( ret != 6 )
	{
		tgl_msg( "load_t3dmodelbin_101: corrupted file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}

	// allocate texture table:
	texturing = 0;
	textures = (GLuint*)tgl_alloc( "load_t3dmodelbin_101", numframes*sizeof(GLuint) );

	//texture names
	for( i=0; i<numframes; i++ )
	{
		ret = 0;
		ret += fread( &n, 4, 1, f ); // strlen
		ret += fread( str, n, 1, f ); // read name, dangerous because no dynamic str allocating! /*FIXME*/
		str[n] = 0; // null terminated string

		if( ret != 2 )
		{
			tgl_msg( "load_t3dmodelbin_101: corrupted file (%s)", TGL_WARNING, (void*)fname );
			fclose( f );
			return -2;
		}

		if( strcasecmp( str, "notexture" ) )
			textures[i] = load_texture( tgl_parse_path(fname)+string(str), str ); // load texture
//			textures[i] = load_texture( string(tgl_parse_path(fname)+string(str)).c_str() ); // load texture
		else
			textures[i] = load_texture( "notexture", "notexture" ); // "load" notexture
//			textures[i] = load_texture( (char*)str ); // "load" notexture

		if( strcasecmp( str, "notexture" ) )
			texturing = 1; // enable texturing

	}

	// allocate arrays
	vertices = (TVERTEX*)tgl_alloc( "load_t3dmodel_101", numvertices*sizeof(TVERTEX)*numframes );
	indices = (GLuint*)tgl_alloc( "load_t3dmodel_101", numindices*sizeof(GLuint) );

	// read vertices	
	for( i=0; i<(numvertices*numframes); i++ )
	{
		ret = 0;

		// position (float)
		ret += fread( &vertices[i].pos, sizeof(float), 3, f );
	
		// color (char, 1.0=255)
		ret += fread( &n, sizeof(char), 1, f ); vertices[i].col[0] = (float)n / 255.0;
		ret += fread( &n, sizeof(char), 1, f ); vertices[i].col[1] = (float)n / 255.0;
		ret += fread( &n, sizeof(char), 1, f ); vertices[i].col[2] = (float)n / 255.0;
		ret += fread( &n, sizeof(char), 1, f ); vertices[i].col[3] = (float)n / 255.0;

		// tex uv (short, 1.0=65535)
		ret += fread( &vertices[i].tex, sizeof(float), 2, f );

		if( ret != 9 )
		{
			tgl_msg( "load_t3dmodelbin_101: corrupted file (%s)", TGL_WARNING, (void*)fname );
			fclose( f );
			return -2;
		}
	}

	// write triangle indices
	ret = fread( indices, sizeof(int), numindices, f ); // all indices
	
	if( ret != numindices )
	{
		tgl_msg( "load_t3dmodelbin_101: corrupted file (%s)", TGL_WARNING, (void*)fname );
		fclose( f );
		return -2;
	}

	sprintf( str, "T3DModelBin loaded (name=%s vertices=%d faces=%d frames=%d)", modelname, numvertices, numindices/3, numframes );
	tgl_msg( str, TGL_INFO );
}

}
