/***************************************************************************
                          texture.cpp  -  texture handling
                             -------------------
    begin                : Tue Jul 18 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tgl.h"

namespace TGL{

//GLuint load_texture( const char *fname )
GLuint load_texture( string fname, string relfilename )
{
	int i, slot, width, height;
	GLuint texnum;
	unsigned char *data;

	// check if texture is already loaded
	for(i=0; i<tgl_max_textures; i++ )
//		if( !strcasecmp( (char*)tgl_texturelist[i].name, (char*)fname ) )
		if( !strcasecmp( (char*)tgl_texturelist[i].name,fname.c_str() ) )
		{
			tgl_texturelist[i].numclients++;
			return( i );
		}

	// find a free texture slot
	for(i=0; i<tgl_max_textures; i++ )
		if( !strcasecmp( tgl_texturelist[i].name, "" ) )
		{
			slot = i;
			break;
		}

	// out of slots?
	if( i>=tgl_max_textures )
	{
		tgl_msg( "load_texture: maximum texture count exceeded (%d)\n    modify tgl_init( max_textures );", TGL_WARNING, (void*)tgl_max_textures );
//		tgl_texturelist[tgl_notexture_slot].numclients++;
		return( tgl_notexture_slot ); // use notexture
	}

	texnum = tgl_texturenums[slot];

	data = pcx_load( fname.c_str(), &width, &height ); // load image
	if( data == NULL )
	{
		tgl_msg( "load_texture: texture load failed, using notexture", TGL_WARNING );
		return( tgl_notexture_slot ); // use notexture
	}

	if( tgl_debug >=1 )
		fprintf( stderr, "load_texture: texture %s loaded: width=%d height=%d slot=%d texnum=%d\n", fname.c_str(), width, height, slot, texnum );

	glBindTexture( GL_TEXTURE_2D, texnum );
	glPixelStorei( GL_UNPACK_ROW_LENGTH, width );
	glPixelStorei( GL_UNPACK_ALIGNMENT,1 );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT );
	glTexParameteri( GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT );
	glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data );

	// scaling mode
	if( texturemode == 0 ) // nearest
	{
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); // scale cheaply when image bigger than texture
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); // scale cheaply when image smalled than texture
	}

	if( texturemode == 1 ) // nearest+mipmap
	{
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, 3, width, height, GL_RGB, GL_UNSIGNED_BYTE, data );
	}

	if( texturemode == 2 ) // linear
	{
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR); // bilinear filtering
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR);
	}

	if( texturemode == 3 ) // linear+mipmap
	{
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_LINEAR_MIPMAP_NEAREST);
		gluBuild2DMipmaps(GL_TEXTURE_2D, GL_RGB, width, height, GL_RGB, GL_UNSIGNED_BYTE, data );
	}

	tgl_free( "load_texture", data );

	// fill up texture_list information
	strcpy( tgl_texturelist[slot].name, fname.c_str() );
//	tgl_texturelist[slot].relative_filename = relfilename;
//	tgl_texturelist[slot].relative_filename = string("");
	tgl_texturelist[slot].relative_filename = new string(relfilename);
	tgl_texturelist[slot].numclients = 1;
	tgl_texturelist[slot].texturenum = texnum;

	return( slot );
}


int delete_unused_textures( void )
{
	int count=0, slot;

	for( slot=0; slot<tgl_max_textures; slot++ )
	{
		if( tgl_texturelist[slot].numclients < 0 ) // this should newer happen
		{
			tgl_msg( "delete_texture: Invalid number of texture clients (%d)", TGL_WARNING, (void*)tgl_texturelist[slot].numclients );
			tgl_texturelist[slot].numclients = 0;
		}

		// don't delete notexture
		if( tgl_texturelist[slot].numclients == 0 && tgl_texturelist[slot].texturenum!=tgl_notexture && strcmp( tgl_texturelist[slot].name, "" ) )
		{
			glDeleteTextures( 1, tgl_texturenums+sizeof(GLuint)*slot );
	
			if( tgl_debug >=1 )
				fprintf( stderr, "delete_unused..: texture slot %d (%s) deleted\n", slot, tgl_texturelist[slot].name );

			strcpy( tgl_texturelist[slot].name, "" ); // clear name
			delete( tgl_texturelist[slot].relative_filename );
			tgl_texturelist[slot].relative_filename=NULL;
//			tgl_texturelist[slot].relative_filename="";
				
			count++;
		}
	}

	if( tgl_debug >=1 )
		fprintf( stderr, "totally %d textures unloaded\n", count );

	return 0;
}

// unload one texture
GLuint delete_texture( int slot, int unload )
{
	// don't delete notexture
	if( slot == tgl_notexture_slot ) return 0;
	
	tgl_texturelist[slot].numclients--;

	if( tgl_texturelist[slot].numclients < 0 ) // this should newer happen
	{
		tgl_msg( "delete_texture: Negative number of texture clients (slot %d)", TGL_WARNING, (void*)slot );
		tgl_texturelist[slot].numclients = 0;
	}

	// don't delete notexture
	if( unload && tgl_texturelist[slot].numclients <= 0 && tgl_texturelist[slot].texturenum!=tgl_notexture && strcmp( tgl_texturelist[slot].name, "" ) )
	{
		glDeleteTextures( 1, tgl_texturenums+sizeof(GLuint)*slot );

		if( tgl_debug >=1 )
			fprintf( stderr, "delete_texture: texture slot %d (%s) deleted\n", slot, tgl_texturelist[slot].name );
			
		strcpy( tgl_texturelist[slot].name, "" ); // clear name
	}

		
	return 0;
}

}
