/***************************************************************************
                          tgl.cpp  -  description
                             -------------------
    begin                : Mon Jun 26 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tgl.h"
#include <string.h>

//using namespace TGL;
namespace TGL {

//set global variables
int SCREEN_W=0, SCREEN_H=0, DEPTH=0, SCREENMODE=0;
bool tgl_disable_lighting=false;
int tgl_initialized=0, tgl_debug=1, tgl_max_textures, tgl_max_lock_count=9999, tgl_max_texture_size=2048; // assume some values
int matpushing=1, matpushed=0, texturemode=3, tgl_texturecache=TGL_AUTO, tgl_object_rendermode=TGL_ELEMENTS; //texmode: 0=nearest 1=nearest+mipmap 2=linear 3=linear+mipmap
int tgl_messages[3]={ TGL_DISPLAY, TGL_DISPLAY, TGL_QUIT };
TTEXTURE *tgl_texturelist=NULL;
GLuint *tgl_texturenums=NULL, tgl_notexture=0xffffffff, tgl_notexture_slot=0;
bool tgl_lockarrays_ext_available=0;
PFNGLLOCKARRAYSEXTPROC glLockArraysEXT_p=NULL;
PFNGLUNLOCKARRAYSEXTPROC glUnlockArraysEXT_p=NULL;

void create_notexture( void )
{
	unsigned char *data;
	int width, height;


	// first init notexture
	if( tgl_notexture=0xffffffff ) // uninitialized
	{
		data = (unsigned char*)tgl_alloc( "create_notexture", 32*32*3 );
		memset( data, 255, 32*32*3 ); // white texture
		width = height = 32;
			
		glBindTexture(GL_TEXTURE_2D, tgl_texturenums[0] );
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MAG_FILTER,GL_NEAREST); // scale cheaply when image bigger than texture
		glTexParameteri(GL_TEXTURE_2D,GL_TEXTURE_MIN_FILTER,GL_NEAREST); // scale cheaply when image smalled than texture
		glTexImage2D(GL_TEXTURE_2D, 0, GL_RGB, width, height, 0, GL_RGB, GL_UNSIGNED_BYTE, data );

		strcpy( tgl_texturelist[0].name, "notexture" );
		tgl_texturelist[0].numclients = 1;
		tgl_texturelist[0].relative_filename=new string( "notexture" );
		tgl_texturelist[0].texturenum = tgl_texturenums[0];
		tgl_notexture = tgl_texturenums[0];
		tgl_notexture_slot = 0;
		
		tgl_free( "create_notexture", data );
	}
}


int tgl_init( int max_textures=TGL_DEFAULT_MAX_TEXTURES )
{
	int i;
	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &tgl_max_texture_size );

	tgl_max_textures = max_textures;
	if( tgl_max_textures <= 1 ) tgl_max_textures = 1;

	tgl_texturelist = (TTEXTURE*)tgl_alloc( "tgl_init", tgl_max_textures*sizeof(TTEXTURE) );
	tgl_texturenums = (GLuint*)tgl_alloc( "tgl_init", tgl_max_textures*sizeof(GLuint) );
//	memset( tgl_texturelist, 0, tgl_max_textures*sizeof( TTEXTURE ) );
	for( i=0; i<tgl_max_textures; i++ ) // clear texturelist
	{
		strcpy( tgl_texturelist[i].name, "" );
//		tgl_texturelist[i].relative_filename=new string("");
		tgl_texturelist[i].relative_filename=NULL;
		tgl_texturelist[i].numclients=0;
		tgl_texturelist[i].texturenum=0;
	}
	memset( tgl_texturenums, 0, tgl_max_textures*sizeof( GLuint ) );

	glGenTextures( max_textures, &tgl_texturenums[0] );
	create_notexture();
	tgl_init_extensions();

	tgl_initialized=1;
	tgl_msg( "TGL version %s initialized", TGL_INFO, (void*)TGL_VERSION_STR );
	#ifdef DRI_BUG_WORKAROUND
	tgl_msg( "DRI/tdfx bug workaround enabled", TGL_INFO );
	#endif
	tgl_msg( "Compiled vertex arrays %s", TGL_INFO, (void*)((tgl_lockarrays_ext_available)?"supported":"not supported") );
	tgl_msg( "GL vendor: %s", TGL_INFO, (char*)glGetString( GL_VENDOR ) );
	tgl_msg( "GL renderer: %s", TGL_INFO, (char*)glGetString( GL_RENDERER ) );
	tgl_msg( "GL version: %s", TGL_INFO, (char*)glGetString( GL_VERSION ) );
	tgl_msg( "GL max texture resolution: %d", TGL_INFO, (void*)tgl_max_texture_size );
	

	return 0;
}

int tgl_setmode( int w, int h, int depth, int screenmode )
{
	SCREEN_W = w;
	SCREEN_H = h;
	DEPTH = depth;
	SCREENMODE = screenmode;
	return 0;
}

int remove_tgl( void )
{
	SCREEN_W = 0;
	SCREEN_H = 0;
	DEPTH = 0;
	SCREENMODE = 0;

	tgl_free( "remove_tgl", tgl_texturelist );
	
	tgl_initialized = 0;
  tgl_msg( "TGL uninitialized", TGL_NOTE );
	
	return 0;
}


int tgl_msg( const char *msg, int type, void *val=NULL )
{
	char *typestr;
	char msgstr[160];
	
	if( type == TGL_NOTE ) typestr="info";
	if( type == TGL_WARNING ) typestr="warning";
	if( type == TGL_ERROR ) typestr="error";

	sprintf( msgstr, "TGL %s: %s\n", typestr, msg );

	if( tgl_messages[type] == TGL_HIDE )
		return 0;
	if( tgl_messages[type] == TGL_DISPLAY )
	{
		fprintf( tglerr, msgstr, val );
		if( type == TGL_WARNING || type == TGL_ERROR )
			return -1;
		else
			return 0;
	}
	if( tgl_messages[type] == TGL_QUIT )
	{
		fprintf( tglerr, msgstr, val );
		exit( 1 );
	}

	return 0;
}



void tgl_file_not_found( const char *func, const char *fname )
{
	char str[160];
	sprintf( str, "%s: file '%s' not found", func, fname );
	tgl_msg( str, TGL_ERROR );
}

void tgl_test_alloc( const char *func, void *pointer )
{
	if( pointer == NULL )
	{
		tgl_msg( "%s: out of memory", TGL_ERROR, (void*)func );
	}
}

void *tgl_alloc( const char *func, unsigned int bytes, bool clear )
{
	void *alloc;

	if( tgl_debug >= 2 ) fprintf( tglerr, "%s: allocating %d bytes memory\n", func, bytes );

	alloc = (void *)malloc( bytes );
	tgl_test_alloc( func, alloc );
	if( clear == true )
		memset( alloc, 0, bytes );

	return alloc;
}

void tgl_free( const char *func, void *pointer )
{
	if( pointer==NULL )
		tgl_msg( "%s: internal error (tried to unallocate NULL pointer)", TGL_ERROR, (void*)func );
	free( pointer );
	pointer = NULL;
}


void gl_get_fullpath( const char *fname )
{


}

int tgl_init_extensions()
{
//	const GLubyte *str;
	const char *str;
	
	// get list of supported extensions
	str = (const char*)glGetString( GL_EXTENSIONS );
	
	// if supported
	if( strstr( (const char*)str, "GL_EXT_compiled_vertex_array" ) )
	{
		tgl_lockarrays_ext_available = true;

		// get funcion poitenrs
		glLockArraysEXT_p = (PFNGLLOCKARRAYSEXTPROC) glXGetProcAddressARB( (GLubyte*) "glLockArraysEXT" );
		glUnlockArraysEXT_p = (PFNGLUNLOCKARRAYSEXTPROC) glXGetProcAddressARB( (GLubyte*) "glUnlockArraysEXT" );
	}
	return 0; //ok
}



string tgl_parse_path( string filename )
{
	string path;
	int i, lastslash;
	
	// search last slash
	for( i=0; i<filename.length(); i++ )
		if( filename[i]=='/' ) lastslash=i;
	
	//coply path
	for( i=0; i<=lastslash; i++ )
		path+=filename[i];
	
	return path;
}



} //EO namespace

