/***************************************************************************
                          tgl.h  -  description
                             -------------------
    begin                : Mon Jun 26 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#ifndef TGL_H
#define TGL_H

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>
#include <GL/gl.h>
#include <GL/glu.h>
#include <GL/glext.h>
#include <SDL/SDL.h>
#include <GL/glx.h>
#include <iostream>
#include <string>
using namespace std;

//definitions
#define TGL_VERSION 0.10
#define TGL_VERSION_STR "0.10"
#define TGL_WINDOWED 0
#define TGL_FULLSCREEN 1
#define TGL_NOTE 0
#define TGL_INFO 0
#define TGL_WARNING 1
#define TGL_ERROR 2
#define TGL_HIDE 0
#define TGL_DISPLAY 1
#define TGL_QUIT 2
#define TGL_DEFAULT_MAX_TEXTURES 256
#define TGL_AUTO 2
#define TGL_MANUALUNLOAD 1
#define TGL_MANUAL 0
#define TGL_T3D 0 // file types:
#define TGL_T3DBIN 1
#define TGL_NONE 0
#define TGL_NEAREST 1
#define TGL_LINEAR 2
#define TGL_CUBIC 3
#define TGL_ELEMENTS 0
#define TGL_ELEMENT 1
#define TGL_TRIANGLES 2
#define tglerr stderr
#define tglout stdout

/* something strange happens with DRI/tdfx when disabling color/normal/texture
   arrays while using glElements. Everything works just fine with software renderer (Mesa). */
//#define DRI_BUG_WORKAROUND 1  // uncomment to enable

//TVERTEX
#define TGL_U 0
#define TGL_V 1
#define TGL_X 0
#define TGL_Y 1
#define TGL_Z 2
#define TGL_I 0
#define TGL_J 1
#define TGL_K 2
#define TGL_R 0
#define TGL_G 1
#define TGL_B 2
#define TGL_A 3

/* Structures */

namespace TGL {

typedef struct _PCX_HEADER
{
	char manufacturer;
	char version;
	char encoding;
	char bits;
	short x;
	short y;
	short width;
	short height;
	short h_res;
	short w_res;
	char ega_pal[48];
	char reserved;
	char color_planes;
	short bytes_per_line;
	short pal_type;
	char padding[58];
} PCX_HEADER;


// GL_T2F_C4F_N3F_V3F
typedef struct _TVERTEX
{
	GLfloat tex[2]; // u,v
	GLfloat col[4]; // r,g,b,a
	GLfloat vec[3]; // i,j,k
	GLfloat pos[3]; // x,y,z
} TVERTEX;

// RGBA
typedef struct _TCOLOR
{
	GLfloat r,g,b,a;
} TCOLOR;

// material
typedef struct _TMATERIAL
{
	GLfloat shiness; // percent
	GLfloat specular[4]; // r,g,b,a
	GLfloat diffuse[4]; // r,g,b,a
	GLfloat ambient[4];
	GLfloat emission[4];
} TMATERIAL;

typedef struct _TTEXTURE
{
	char name[64];
	string *relative_filename;
	GLuint texturenum;
	int numclients;
} TTEXTURE;

#define TVERTEX_SIZE 48
#define POS_OFFSET 36
#define VEC_OFFSET 24
#define COL_OFFSET 8
#define TEX_OFFSET 0

//functions
int tgl_init( int max_textures=TGL_DEFAULT_MAX_TEXTURES );
int tgl_setmode( int w, int h, int depth, int screenmode );
int remove_tgl( void );
int tgl_msg( const char *msg, int type, void *val=NULL );
void tgl_file_not_found( const char *func, const char *fname );
unsigned char* pcx_load( const char *filename, int *pw, int *ph );
void tgl_test_alloc( const char *func, void *pointer );
void *tgl_alloc( const char *func, unsigned int bytes, bool clear=false );
void tgl_free( const char *func, void *pointer );
int tgl_init_extensions();
string tgl_parse_path( string filename );
//texture utilities
int delete_unused_textures( void );
GLuint load_texture( string fname, string relfilename );
//GLuint load_texture( const char *fname );
// unload one texture (if unload=0, delete client but leave texture in memory, else unload)
GLuint delete_texture( int slot, int unload );


//global variables
extern int SCREEN_W, SCREEN_H, DEPTH, SCREENMODE; // currently unused
extern bool tgl_disable_lighting;
extern int tgl_initialized, tgl_debug, tgl_max_textures, tgl_max_lock_count, tgl_max_texture_size; // tgl engine ready, debug level 0-4, max texture count, max vertex count, max texture resolution
extern int matpushing, matpushed, texturemode, tgl_texturecache, tgl_object_rendermode; // push matrices, matpush status, textruring on/off, uload textures automatically/manually
extern int tgl_messages[3]; // message printing mode: off/on/on+halt
extern TTEXTURE *tgl_texturelist; // currently loaded textures
extern GLuint *tgl_texturenums, tgl_notexture, tgl_notexture_slot; // GL texnums, blank texturenum
extern bool tgl_lockarrays_ext_available;
extern PFNGLLOCKARRAYSEXTPROC glLockArraysEXT_p;
extern PFNGLUNLOCKARRAYSEXTPROC glUnlockArraysEXT_p;


//classes
//#include "tgl_scene.h"

} // end of namespace

#include "tgl_object.h"
#include "tgl_camera.h"

#endif
