/***************************************************************************
                          tgl_camera.cpp  -  description
                             -------------------
    begin                : Mon Jun 26 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tgl_camera.h"

namespace TGL{

tgl_camera::tgl_camera()
{
	x=y=z=0.0;
	rot_x=rot_y=rot_z=0.0;
	lookat_x=lookat_y=lookat_z=0.0;
	up[0]=up[2]=0; up[1]=1; //up vector

	rotate1st=0;
	rotate2nd=1;
	rotate3rd=2; // 0=x, 1=y, 2=z

	near_clip=0.10;
	far_clip=250.0;
	view_x=view_y=0;
	view_w=SCREEN_W;
	view_h=SCREEN_H;
}

tgl_camera::~tgl_camera(){
}


//view
int tgl_camera::set_view()
{
	lookat_x = x + sin( rot_y )*cos( rot_x );
	lookat_z = z + cos( rot_y )*cos( rot_x );
	lookat_y = y + sin( rot_x );


	// reset camera
	glViewport( view_x, view_y, view_w, view_h );
	glMatrixMode( GL_PROJECTION );
	glLoadIdentity();
	gluPerspective(60.0f,(float)SCREEN_W/(float)SCREEN_H,near_clip,far_clip);
	glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();


	gluLookAt(x, y, z,
		lookat_x, lookat_y, lookat_z,
		up[0], up[1], up[2] );



	// rotate camera
/*	switch( rotate1st )
	{
		case 0: glRotatef(rot_x, 1.0f, 0, 0); break;
		case 1: glRotatef(rot_y, 0, 1.0f, 0); break;
		case 2: glRotatef(rot_z, 0, 0, 1.0f); break;
	}

	switch( rotate2nd )
	{
		case 0: glRotatef(rot_x, 1.0f, 0, 0); break;
		case 1: glRotatef(rot_y, 0, 1.0f, 0); break;
		case 2: glRotatef(rot_z, 0, 0, 1.0f); break;
	}

	switch( rotate3rd )
	{
		case 0: glRotatef(rot_x, 1.0f, 0, 0); break;
		case 1: glRotatef(rot_y, 0, 1.0f, 0); break;
		case 2: glRotatef(rot_z, 0, 0, 1.0f); break;
	}*/

	//move camera
//	glTranslatef( x, y, z );

//	glMatrixMode( GL_MODELVIEW );

	return 0;
}
	
//rotate
int tgl_camera::rotate( float rx, float ry, float rz )
{
	rot_x += rx;
	rot_y += ry;
	rot_z += rz;

	return 0;
}

int tgl_camera::set_angle( float angle, float pitch )
{
	rot_x = pitch;
	rot_y = angle;
//	rot_x = pitch;
//	rot_z = angle;

	return 0;
}

//move
int tgl_camera::move( float mx, float my, float mz )
{
	x += mx;
	y += my;
	z += mz;

	return 0;
}

//move relatively to camera angle
int tgl_camera::move_forward( float mx, float my, float mz )
{	
	// forward
	float move_x = sin( rot_y )*cos( rot_x )*mz;
	float move_z = cos( rot_y )*cos( rot_x )*mz;
	float move_y = sin( rot_x )*mz;

	// right
	move_x -= cos( -rot_y )*mx;
	move_z -= sin( -rot_y )*mx;
	
	// up
	move_x += sin( rot_y )*sin( -rot_x )*my;
	move_z += cos( rot_y )*sin( -rot_x )*my;
	move_y += cos( -rot_x )*my;
	
  move( move_x, move_y, move_z );
	
	return 0;
}

int tgl_camera::set_position( float mx, float my, float mz )
{
	x = mx;
	y = my;
	z = mz;

	return 0;
}

}