/***************************************************************************
                          tgl_heightfield.cpp  -  description
                             -------------------
    begin                : Fri Dec 15 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tgl_heightfield.h"
#include <GL/gl.h>

namespace TGL{

tgl_heightfield::tgl_heightfield( char *fname )
{
	// default material
	material.shiness = 0.0; // shiness
	material.diffuse[0] = material.diffuse[1] = material.diffuse[2] = 0.8; material.diffuse[3] = 1.0; // diffuse
	material.specular[0] = material.specular[1] = material.specular[2] = 0.0; material.specular[3] = 1.0; // specular
  material.ambient[0] = material.ambient[1] = material.ambient[2] = 0.2; material.ambient[3] = 1.0; // ambient
  material.emission[0] = material.emission[1] = material.emission[2] = 0.0; material.emission[3] = 1.0; // emission
}

tgl_heightfield::~tgl_heightfield()
{
	int i;

	// delete textures
/*	if( tgl_texturecache == TGL_AUTO ) // auto/manual delete
	{
		for( i=0; i<numframes; i++ )
			delete_texture( textures[i], 1 );	
	}  fixme!!!!!!!!!!
	else if( tgl_texturecache == TGL_MANUALUNLOAD ) // delete client but don't unload texture
	{
		for( i=0; i<numframes; i++ )
			delete_texture( textures[i], 0 );	
	}*/

	// unallocate arrays:
	tgl_free( "~tgl_heightfield", vertices );
	tgl_free( "~tgl_heightfield", indices );
	tgl_free( "~tgl_heightfield", currindices );
	tgl_free( "~tgl_heightfield", textures );
	tgl_free( "~tgl_heightfield", currframe );
	tgl_free( "~tgl_heightfield", vertexmorph );

	numvertices = 0;
	numindices = 0;
}


GLvoid tgl_heightfield::rotate( GLfloat x, GLfloat y, GLfloat z, GLfloat amount )
{
	if( !matpushed && matpushing )
	{
		glPushMatrix();
		matpushed = 1;
	}

	glRotatef( amount, x, y, z );
}

GLvoid tgl_heightfield::move( GLfloat x, GLfloat y, GLfloat z )
{
	if( !matpushed && matpushing )
	{
		glPushMatrix();
		matpushed = 1;
	}

	glTranslatef( x, y, z );
}

GLvoid tgl_heightfield::scale( GLfloat x, GLfloat y, GLfloat z )
{
	if( !matpushed && matpushing )
	{
		glPushMatrix();
		matpushed = 1;
	}

	glScalef( x, y, z );
}

// render hf
GLint tgl_heightfield::draw()
{
	int srcbl, dstbl, locked;

	// these must be set BEFORE glInterleavedArrays
	if( texturing )
	{
		glBindTexture( GL_TEXTURE_2D, tgl_texturelist[(textures[0])].texturenum ); //fixme
		glEnable( GL_TEXTURE_2D );
	}

	// coloring flag
	if( coloring )
	{
		if( lighting || texturing );
			glEnable( GL_COLOR_MATERIAL );
	}

	// lighting flag handler
	if( lighting == 1 || lighting == 2 )
		glShadeModel( GL_FLAT );
	else
		glShadeModel( GL_SMOOTH );

	if( lighting == 0 || lighting == 1 || lighting == 3 )
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 0 );
	else
		glLightModeli( GL_LIGHT_MODEL_TWO_SIDE, 1 );
		
	if( lighting )
		glEnable( GL_LIGHTING );
	else
		glDisable( GL_LIGHTING );



	//hidden face removal, back face culling etc..
	if( hfre )
	{
		glEnable( GL_CULL_FACE );	
	}	else
	{
		glDisable( GL_CULL_FACE );
	}

	// blending
	if( src_blendfunc && dest_blendfunc )
	{
		switch( src_blendfunc ) {
			case 1: srcbl = GL_ZERO; break;
			case 2: srcbl = GL_ONE; break;
			case 3: srcbl = GL_DST_COLOR; break;
			case 4: srcbl = GL_DST_ALPHA; break;
			case 5: srcbl = GL_SRC_ALPHA; break;
			case 6: srcbl = GL_ONE_MINUS_DST_COLOR; break;
			case 7: srcbl = GL_ONE_MINUS_DST_ALPHA; break;
			case 8: srcbl = GL_ONE_MINUS_SRC_ALPHA; break;
			default: srcbl = GL_ONE; // unsupported
		}

		switch( dest_blendfunc ) {
			case 1: dstbl = GL_ZERO; break;
			case 2: dstbl = GL_ONE; break;
			case 3: dstbl = GL_SRC_COLOR; break;
			case 4: dstbl = GL_SRC_ALPHA; break;
			case 5: dstbl = GL_DST_ALPHA; break;
			case 6: dstbl = GL_ONE_MINUS_SRC_COLOR; break;
			case 7: dstbl = GL_ONE_MINUS_SRC_ALPHA; break;
			case 8: dstbl = GL_ONE_MINUS_DST_ALPHA; break;
			default: dstbl = GL_ONE; // unsupported
		}
				
		glBlendFunc( srcbl, dstbl );
		glEnable( GL_BLEND );
	} else
		glDisable( GL_BLEND );

	// set material		
	glMaterialfv( GL_FRONT_AND_BACK, GL_SHININESS, &material.shiness );
	glMaterialfv( GL_FRONT_AND_BACK, GL_SPECULAR, material.specular );
	glMaterialfv( GL_FRONT_AND_BACK, GL_DIFFUSE, material.diffuse );
	glMaterialfv( GL_FRONT_AND_BACK, GL_AMBIENT, material.ambient );
	glMaterialfv( GL_FRONT_AND_BACK, GL_EMISSION, material.emission );

	
	// set vertexpointer
	glInterleavedArrays( GL_T2F_C4F_N3F_V3F, 0, currframe );


	// these must be disabled AFTER glInterleavedArrays:
	
	if( !texturing )
	{
		glDisableClientState( GL_TEXTURE_COORD_ARRAY );
		glDisable( GL_TEXTURE_2D );
	} else
	{
		glEnableClientState( GL_TEXTURE_COORD_ARRAY );
		glEnable( GL_TEXTURE_2D );
	}

	// lighting flag handler
	if( !lighting && !hfre )
		glDisableClientState( GL_NORMAL_ARRAY );


	// coloring flag
	if( !coloring )
	{
		glDisableClientState( GL_COLOR_ARRAY );
		glDisable( GL_COLOR_MATERIAL );
		glColor4f( 1.0, 1.0, 1.0, 1.0 );
	} else
	{
		glEnableClientState( GL_COLOR_ARRAY );
		glEnable( GL_COLOR_MATERIAL );
	}

	// render
	if( numvertices < tgl_max_lock_count ) // don't try to lock if numvertices exceeds maximum lock count
		glLockArraysEXT_p( 0, numvertices );
	
	#ifdef GL_ARRAY_ELEMENT_LOCK_COUNT_EXT
	glGetIntegerv( GL_ARRAY_ELEMENT_LOCK_COUNT_EXT, &locked ); // how many locked vertices
//	glGetIntegerv( 0x81A9, &locked ); // mesa trick
	if( locked != numvertices )
	{
		tgl_msg( "Can't lock %d vertices, setting new lock count limit", TGL_WARNING, (void*)locked );
		tgl_max_lock_count = numvertices; // set new limit
	}
	#endif
		
	glDrawElements( GL_TRIANGLES, numindices, GL_UNSIGNED_INT, indices ); // draw object
	
	if( numvertices < tgl_max_lock_count ) // don't try to unlock if its not locked
		glUnlockArraysEXT_p();
	

	// undo transformations
	if( matpushed && matpushing )
	{
		glPopMatrix();
		matpushed = 0;
	}

	return 0;
}


// calculate all normals
void tgl_heightfield::calcnormals( void )
{
	int frame=0, tris, i;

	tris = numindices/3;

	// clean all normals
	for( i=0; i<(numvertices); i++ )
		vertices[i].vec[0] = vertices[i].vec[1] = vertices[i].vec[2] = 0.0;

	// loop for every face
	for( i=0; i<tris; i++ )
	{
		calcnormal( indices[i*3]+numvertices*frame, indices[i*3+1]+numvertices*frame, indices[i*3+2]+numvertices*frame );
	}

	normalize();

	if( tgl_debug>=2 )
		for( i=0; i<(numvertices); i++ )
			fprintf( stderr, "normal %d: %f %f %f\n", i, vertices[i].vec[0], vertices[i].vec[1], vertices[i].vec[2] );
}


// calculate facenormal
void tgl_heightfield::calcnormal( int i1, int i2, int i3 )
{
	GLfloat relx1 = vertices[i2].pos[0] - vertices[i1].pos[0],
					rely1 = vertices[i2].pos[1] - vertices[i1].pos[1],
					relz1 = vertices[i2].pos[2] - vertices[i1].pos[2],
					relx2 = vertices[i3].pos[0] - vertices[i1].pos[0],
					rely2 = vertices[i3].pos[1] - vertices[i1].pos[1],
					relz2 = vertices[i3].pos[2] - vertices[i1].pos[2],
					nx, ny, nz;

	nx = (rely1 * relz2 - relz1 * rely2);
	ny = (relz1 * relx2 - relx1 * relz2);
	nz = (relx1 * rely2 - rely1 * relx2);

	// add normals
	vertices[i1].vec[0] += nx;
	vertices[i1].vec[1] += ny;
	vertices[i1].vec[2] += nz;

	vertices[i2].vec[0] += nx;
	vertices[i2].vec[1] += ny;
	vertices[i2].vec[2] += nz;

	vertices[i3].vec[0] += nx;
	vertices[i3].vec[1] += ny;
	vertices[i3].vec[2] += nz;
}


void tgl_heightfield::normalize( void )
{
	GLfloat len;
	int i;
	
	for( i=0; i<(numvertices); i++ )
	{
		// calc length
		len = sqrt( vertices[i].vec[0]*vertices[i].vec[0] +
								vertices[i].vec[1]*vertices[i].vec[1] +
								vertices[i].vec[2]*vertices[i].vec[2] );

		// avoid div by zero
		if( len < 0.01 ) len = 0.01;

		// divide vectors
		vertices[i].vec[0] /= len;
		vertices[i].vec[1] /= len;
		vertices[i].vec[2] /= len;
	}
}

}

