/***************************************************************************
                          tgl_object.cpp  -  description
                             -------------------
    begin                : Mon Jun 26 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/

#include "tgl_object.h"
#include <GL/gl.h>

//using namespace TGL;
namespace TGL{

tgl_object::tgl_object( const char *fname )
{
	// select renderer
/*	switch( tgl_object_rendermode )
	{
		case TGL_ELEMENTS: draw = draw_elements; break;
		case TGL_ELEMENT: draw = draw_element; break;
		case TGL_TRIANGLES: draw = draw_triangles; break;
		default: draw = draw_elements; break;
	}*/
	
	
	morphing = 1;
	do_morph = 0;
	morphfrom = morphto = currframenum = prevframenum = 0;
	currmorph = 0.0;
	cubic_strength = 0.166666;
	model_filename=fname;
	
	// default material
	material.shiness = 0.0; // shiness
	material.diffuse[0] = material.diffuse[1] = material.diffuse[2] = 0.8; material.diffuse[3] = 1.0; // diffuse
	material.specular[0] = material.specular[1] = material.specular[2] = 0.0; material.specular[3] = 1.0; // specular
  material.ambient[0] = material.ambient[1] = material.ambient[2] = 0.2; material.ambient[3] = 1.0; // ambient
  material.emission[0] = material.emission[1] = material.emission[2] = 0.0; material.emission[3] = 1.0; // emission
	
	// default interpolation mode
	pos_morph = vec_morph = col_morph = tex_morph = TGL_LINEAR;
	
	loop_begin = loop_end = 0; // needed only with cubic interpolation
	
	load_model( fname );
	
	loop_end = numframes-1;
}

tgl_object::~tgl_object()
{
	int i;

	// delete textures
	if( tgl_texturecache == TGL_AUTO ) // auto/manual delete
	{
		for( i=0; i<numframes; i++ )
			delete_texture( textures[i], 1 );	
	}
	else if( tgl_texturecache == TGL_MANUALUNLOAD ) // delete client but don't unload texture
	{
		for( i=0; i<numframes; i++ )
			delete_texture( textures[i], 0 );	
	}

	// unallocate arrays:
	tgl_free( "~tgl_object", vertices );
	tgl_free( "~tgl_object", indices );
	tgl_free( "~tgl_object", textures );
	tgl_free( "~tgl_object", currframe );
	tgl_free( "~tgl_object", vertexmorph );

	numvertices = 0;
	numindices = 0;
}


GLvoid tgl_object::rotate( GLfloat x, GLfloat y, GLfloat z, GLfloat amount )
{
	if( !matpushed && matpushing )
	{
		glPushMatrix();
		matpushed = 1;
	}

	glRotatef( amount, x, y, z );
}

GLvoid tgl_object::move( GLfloat x, GLfloat y, GLfloat z )
{
	if( !matpushed && matpushing )
	{
		glPushMatrix();
		matpushed = 1;
	}

	glTranslatef( x, y, z );
}

GLvoid tgl_object::scale( GLfloat x, GLfloat y, GLfloat z )
{
	if( !matpushed && matpushing )
	{
		glPushMatrix();
		matpushed = 1;
	}

	glScalef( x, y, z );
}

GLint tgl_object::change_texture( int frame, const char *fname )
{
	int i;

	if( frame < 0 ) {
		tgl_msg( "change_texture: negative frame number (%d)", TGL_WARNING, (void*)frame );
		frame = 0;
	}
	if( frame >= numframes ) {
		tgl_msg( "change_texture: too big frame number (%d)", TGL_WARNING, (void*)frame );
		frame = numframes-1;
	}

	// delete old texture
	if( tgl_texturecache == TGL_AUTO ) // auto/manual delete
		delete_texture( textures[frame], 1 );	
	else if( tgl_texturecache == TGL_MANUALUNLOAD ) // delete client but don't unload texture
		delete_texture( textures[frame], 0 );	
		
//        textures[frame] = load_texture( (char*)fname );
       	if( strcasecmp( (const char*)fname, "notexture" ) ){
		textures[frame] = load_texture( tgl_parse_path(model_filename)+string(fname), fname ); // load texture
//		textures[frame] = load_texture( string(tgl_parse_path(model_filename)+string(fname)).c_str() ); // load texture
	}
	else
		textures[frame] = load_texture( "notexture", "notexture" ); // "load" notexture*/
//		textures[frame] = load_texture( (char*)fname ); // "load" notexture*/
	
	// check if texturing can be disabled
	texturing = 0;
	for( i=0; i<numframes; i++ )
		if( strcasecmp( get_texturename(i)->c_str(), "notexture" ) ) texturing = 1;
}

/*const char* tgl_object::get_texturename( int frame )
{
	if( frame < 0 ) {
		tgl_msg( "change_texture: negative frame number (%d)", TGL_WARNING, (void*)frame );
		frame = 0;
	}
	if( frame >= numframes ) {
		tgl_msg( "change_texture: too big frame number (%d)", TGL_WARNING, (void*)frame );
		frame = numframes-1;
	}

	return( (const char*)tgl_texturelist[textures[frame]].name );
}*/

string* tgl_object::get_texturename( int frame )
{
	if( frame < 0 ) {
		tgl_msg( "change_texture: negative frame number (%d)", TGL_WARNING, (void*)frame );
		frame = 0;
	}
	if( frame >= numframes ) {
		tgl_msg( "change_texture: too big frame number (%d)", TGL_WARNING, (void*)frame );
		frame = numframes-1;
	}

	return( tgl_texturelist[textures[frame]].relative_filename );
}


// calculate all normals
void tgl_object::calcnormals( void )
{
	int frame, tris, i;

	tris = numindices/3;

	// clean all normals
	for( i=0; i<(numvertices*numframes); i++ )
		vertices[i].vec[0] = vertices[i].vec[1] = vertices[i].vec[2] = 0.0;

	// loop for every face
	for( frame=0; frame<numframes; frame++ )
		for( i=0; i<tris; i++ )
		{
			calcnormal( indices[i*3]+numvertices*frame, indices[i*3+1]+numvertices*frame, indices[i*3+2]+numvertices*frame );
		}

	normalize();

	if( tgl_debug>=2 )
		for( i=0; i<(numvertices*numframes); i++ )
			fprintf( stderr, "normal %d: %f %f %f\n", i, vertices[i].vec[0], vertices[i].vec[1], vertices[i].vec[2] );
}


// calculate vertexnormal
void tgl_object::calcnormal( int i1, int i2, int i3 )
{
	GLfloat relx1 = vertices[i2].pos[0] - vertices[i1].pos[0],
					rely1 = vertices[i2].pos[1] - vertices[i1].pos[1],
					relz1 = vertices[i2].pos[2] - vertices[i1].pos[2],
					relx2 = vertices[i3].pos[0] - vertices[i1].pos[0],
					rely2 = vertices[i3].pos[1] - vertices[i1].pos[1],
					relz2 = vertices[i3].pos[2] - vertices[i1].pos[2],
					nx, ny, nz;

	nx = (rely1 * relz2 - relz1 * rely2);
	ny = (relz1 * relx2 - relx1 * relz2);
	nz = (relx1 * rely2 - rely1 * relx2);

	// add normals
	vertices[i1].vec[0] += nx;
	vertices[i1].vec[1] += ny;
	vertices[i1].vec[2] += nz;

	vertices[i2].vec[0] += nx;
	vertices[i2].vec[1] += ny;
	vertices[i2].vec[2] += nz;

	vertices[i3].vec[0] += nx;
	vertices[i3].vec[1] += ny;
	vertices[i3].vec[2] += nz;
}


void tgl_object::normalize( void )
{
	GLfloat len;
	int i;
	
	for( i=0; i<(numvertices*numframes); i++ )
	{
		// calc length
		len = sqrt( vertices[i].vec[0]*vertices[i].vec[0] +
								vertices[i].vec[1]*vertices[i].vec[1] +
								vertices[i].vec[2]*vertices[i].vec[2] );

		// avoid div by zero
		if( len < 0.01 ) len = 0.01;

		// divide vectors
		vertices[i].vec[0] /= len;
		vertices[i].vec[1] /= len;
		vertices[i].vec[2] /= len;
	}
}


void tgl_object::print_info( void )
{
	fprintf( tglerr, "Model %s (%s) information:\n"
		"vertices=%d indices=%d frames=%d currframenum=%d\n"
		"texturing=%d coloring=%d culling=%d lighting=%d blend_src=%d blend_dst=%d\n"
		"morphing=%d do_morph=%d morphfrom=%d morphto=%d\n",
		modelname, model_filename.c_str(), numvertices, numindices, numframes, currframenum,
		texturing, coloring, hfre, lighting, src_blendfunc, dest_blendfunc,
		morphing, do_morph, morphfrom, morphto );
}

// for T3Dedit (allocate new obj)
tgl_object::tgl_object( int verts, int frames, int inds, const char *name, int lit, int col, int culling, int blendsrc, int blenddst )
{
	// update object data
	morphing = 1;
	do_morph = 0;
	morphfrom = morphto = currframenum = 0;
	currmorph = 0.0;
	numindices = inds;
	numvertices = verts;
	numframes = frames;
	lighting = lit;
	texturing = 1;
	coloring = col;
	hfre = culling;
	src_blendfunc = blendsrc;
	dest_blendfunc = blenddst;

	morphing = 1;
	do_morph = 0;
	morphfrom = morphto = currframenum = prevframenum = 0;
	currmorph = 0.0;
	cubic_strength = 0.166666;
	
	// default material
	material.shiness = 0.0; // shiness
	material.diffuse[0] = material.diffuse[1] = material.diffuse[2] = 0.8; material.diffuse[3] = 1.0; // diffuse
	material.specular[0] = material.specular[1] = material.specular[2] = 0.0; material.specular[3] = 1.0; // specular
  material.ambient[0] = material.ambient[1] = material.ambient[2] = 0.2; material.ambient[3] = 1.0; // ambient
  material.emission[0] = material.emission[1] = material.emission[2] = 0.0; material.emission[3] = 1.0; // emission
	
	// default interpolation mode
	pos_morph = vec_morph = col_morph = tex_morph = TGL_LINEAR;
	loop_begin = 0; // needed only with cubic interpolation
	loop_end = numframes-1;
	
	
	
	
	// allocate memory
	textures = (GLuint*)tgl_alloc( "tgl_object", frames*sizeof(GLuint), true );
	vertices = (TVERTEX*)tgl_alloc( "tgl_object", numvertices*sizeof(TVERTEX)*numframes, true );
	indices = (GLuint*)tgl_alloc( "tgl_object", numindices*sizeof(GLuint), true );
	strcpy( modelname, name );
	currframe = (TVERTEX*)tgl_alloc( "tgl_object", numvertices*sizeof( TVERTEX ), true );
	vertexmorph = (TVERTEX*)tgl_alloc( "tgl_object", numvertices*sizeof( TVERTEX ), true );

	calcnormals();
	calc_morphtable();
	calc_currframe();
}

// for T3Dedit (copy setting from other object)
void tgl_object::copy_variables( tgl_object *obj )
{
	// update object data
	morphing = obj->morphing;
	do_morph = 0;
	morphfrom = morphto = currframenum = 0;
	currmorph = 0.0;
	lighting = obj->lighting;
	texturing = obj->texturing;
	coloring = obj->coloring;
	hfre = obj->hfre;
	src_blendfunc = obj->src_blendfunc;
	dest_blendfunc = obj->dest_blendfunc;

	morphing = obj->morphing;
	do_morph = 0;
	morphfrom = morphto = currframenum = prevframenum = 0;
	currmorph = obj->currmorph;
	cubic_strength = obj->cubic_strength;
	
	// default material
	material.shiness = obj->material.shiness; // shiness
	material.diffuse[0] = obj->material.diffuse[0]; material.diffuse[1] = obj->material.diffuse[1]; material.diffuse[2] = obj->material.diffuse[2]; material.diffuse[3] = obj->material.diffuse[3]; // diffuse
	material.specular[0] = obj->material.specular[0]; material.specular[1] = obj->material.specular[1]; material.specular[2] = obj->material.specular[2]; material.specular[3] = obj->material.specular[3]; // specular
  material.ambient[0] = obj->material.ambient[0]; material.ambient[1] = obj->material.ambient[1]; material.ambient[2] = obj->material.ambient[2]; material.ambient[3] = obj->material.ambient[3]; // ambient
  material.emission[0] =obj->material.emission[0]; material.emission[1] = obj->material.emission[1]; material.emission[2] = obj->material.emission[2]; material.emission[3] = obj->material.emission[3]; // emission
	
	// default interpolation mode
	pos_morph = obj->pos_morph; vec_morph = obj->vec_morph; col_morph = obj->col_morph; tex_morph = obj->tex_morph;
	loop_begin = 0; // needed only with cubic interpolation
	loop_end = numframes-1;
}


void tgl_object::pre_scale( float sx, float sy, float sz )
{
	int i;
	
	for( i=0; i<(numvertices*numframes); i++ )
	{
		vertices[i].pos[0]*=sx;
		vertices[i].pos[1]*=sy;
		vertices[i].pos[2]*=sz;
	}
	calcnormals();
	calc_currframe();
}



} //EO namespace
