/***************************************************************************
                          tgl_object.h  -  description
                             -------------------
    begin                : Mon Jun 26 2000
    copyright            : (C) 2000 by Tero Kontkanen
    email                : tkontkanen@mail.com
 ***************************************************************************/

/***************************************************************************
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 ***************************************************************************/


#ifndef TGL_OBJECT_H
#define TGL_OBJECT_H

#include "tgl.h"
#include <GL/gl.h>

//using namespace TGL;

namespace TGL{

/**
  *@author Tero Kontkanen
  */

class tgl_object {
public: 
	tgl_object( const char *fname );
	~tgl_object();
	GLvoid rotate( GLfloat x, GLfloat y, GLfloat z, GLfloat amount );
	GLvoid move( GLfloat x, GLfloat y, GLfloat z );
	GLvoid scale( GLfloat x, GLfloat y, GLfloat z );
	GLint draw();
	GLint change_texture( int frame, const char *fname );
//	const char* get_texturename( int frame );
	string* tgl_object::get_texturename( int frame );
	
	void tgl_object::calc_currframe( void ); // morph
//	GLint (*draw)();
	GLint draw_elements();
	GLint draw_element();
	GLint draw_triangles();
	void tgl_object::set_morph( int from, int to );
	void tgl_object::morph( float pos );
	void tgl_object::set_loop( int begin, int end );
	void tgl_object::calcnormals( void );
	void tgl_object::print_info( void );
	GLint save_t3dmodel( const char *fname, int ver );
	GLint tgl_object::save_t3dmodelbin( const char *fname, int ver );
	GLvoid tgl_object::debug_model( void );
	/** for T3Dedit */
	tgl_object( int verts, int frames, int inds, const char *name, int lit=3, int col=1, int culling=1, int blendsrc=0, int blenddst=0 );
	void tgl_object::copy_variables( tgl_object *obj );
	/// Scale object permanently (modifies vertices).
	void tgl_object::pre_scale( float sx, float sy, float sz );
	
	/** variables **/
	char modelname[64], filetype, fullpath[256];
	int texturing, coloring, lighting, hfre, morphing;
	int numindices, numvertices, numframes;
	int currframenum, prevframenum, morphfrom, morphto, loop_begin, loop_end, do_morph;
	float cubic_strength;
	int src_blendfunc, dest_blendfunc;
	int pos_morph, vec_morph, col_morph, tex_morph; // interpolation mode (0=none, 1=nearest, 2=linear, 3=cubic)
	float currmorph;
	string model_filename;

	/** arrays **/
	TVERTEX *vertices;
	TVERTEX *currframe;
	TVERTEX *vertexmorph;
	GLuint *indices;
	GLuint *textures;
	TMATERIAL material; // diffuse, shiness, specular...

private:
	// file io
	GLint load_model( const char *fname );
	GLint tgl_object::load_t3dmodel( FILE *f, const char *fname );
	GLint tgl_object::load_t3dmodelbin( FILE *f, const char *fname );
	GLint load_t3dmodel_100( FILE *f, const char *fname );
	GLint save_t3dmodel_100( const char *fname );
	GLint tgl_object::save_t3dmodelbin_100( const char *fname );
	GLint tgl_object::load_t3dmodelbin_100( FILE *f, const char *fname );
	/* loader for T3Dmodel ver 1.01 */
	GLint tgl_object::load_t3dmodel_101( FILE *f, const char *fname );
	/* save T3Dmodel v 1.01 */
	GLint tgl_object::save_t3dmodel_101( const char *fname );
	/* save T3DModelBin v 1.01 */
	GLint tgl_object::save_t3dmodelbin_101( const char *fname );
	/* loader for T3DModelBin ver 1.01 */
	GLint tgl_object::load_t3dmodelbin_101( FILE *f, const char *fname );

	
	
	void tgl_object::calcnormal( int i1, int i2, int i3 );
	void tgl_object::normalize( void );
	
	// morphing
	void tgl_object::calc_morphtable( void );
	void tgl_object::calc_currframe_pos( void );
	void tgl_object::calc_currframe_vec( void );
	void tgl_object::calc_currframe_col( void );
	void tgl_object::calc_currframe_tex( void );
	
	// cubic calculations
	float tgl_object::bezvalue( float mu, float *coor );
	float tgl_object::cubic_morph( int component, int from, int to, int ver, float phase );
	
};

}

#endif
